package com.tencent.edu.common.utils

import android.content.Context
import com.tencent.edu.kernel.report.Report
import com.tencent.edu.module.report.AutoReportMgr
import com.tencent.edu.module.report.ReportConstant.DETAIL_VER1_KEY
import com.tencent.edu.module.report.ReportConstant.DETAIL_VER2_KEY
import com.tencent.edu.module.utils.ExtraUtils
import java.util.*

/**
 * 灯塔日志简单上报
 * @author aidanyang
 * @date 2020-04-17
 */
class SimpleBeaconReportUtil {
    companion object {

        @JvmStatic
        @JvmOverloads
        fun reportExposureEvent(context: Context, module: String?, page: String? = null, ver1: String? = null, ver2: String? = null, params: Map<String, String>? = null) {
            reportEvent(context, module, ExtraUtils.EXTRA_REPORT_EXPOSURE_ACTION, page, ver1, ver2, params)
        }

        @JvmStatic
        @JvmOverloads
        fun reportClickEvent(context: Context, module: String?, page: String? = null, ver1: String? = null, ver2: String? = null, params: Map<String, String>? = null) {
            reportEvent(context, module, ExtraUtils.EXTRA_REPORT_CLICK_ACTION, page, ver1, ver2, params)
        }

        @JvmStatic
        @JvmOverloads
        fun reportEvent(context: Context, module: String?, action: String, page: String? = null, ver1: String? = null, ver2: String? = null, params: Map<String, String>? = null) {
            val reportExtraInfo = AutoReportMgr.getReportExtraInfo(context) ?: return
            reportExtraInfo.eventCode = action
            reportExtraInfo.module = module
            reportExtraInfo.page = page

            val customParams = HashMap<String, String>()
            customParams[DETAIL_VER1_KEY] = if (ver1.isNullOrBlank()) {
                ""
            } else {
                ver1
            }
            customParams[DETAIL_VER2_KEY] = if (ver2.isNullOrBlank()) {
                ""
            } else {
                ver2
            }

            if (!params.isNullOrEmpty()) {
                customParams.putAll(params)
            }

            reportExtraInfo.customDatas = customParams

            Report.autoReportData(reportExtraInfo)
        }
    }
}