package com.tencent.edu.common.utils

import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.TextView
import androidx.viewpager.widget.PagerAdapter
import com.androidkun.xtablayout.XTabLayout
import com.google.android.material.tabs.TabLayout
import com.tencent.edu.R


/**
 *
 * @author aidanyang
 * @date 2020-04-17
 */
class TabLayoutUtil {
    companion object {
        @JvmStatic
        fun bindViewPager(mTabLayout: XTabLayout, viewPager: androidx.viewpager.widget.ViewPager, listener: XTabLayout.OnTabSelectedListener? = null) {
            viewPager.addOnPageChangeListener(XTabLayout.TabLayoutOnPageChangeListener(mTabLayout))
            viewPager.adapter = object : PagerAdapter() {
                override fun getCount(): Int {
                    return viewPager.childCount
                }

                override fun isViewFromObject(view: View, `object`: Any): Boolean {
                    return view === `object`
                }

                override fun instantiateItem(container: ViewGroup, position: Int): Any {
                    return viewPager.getChildAt(position)
                }

                override fun destroyItem(container: ViewGroup, position: Int, `object`: Any) {
                    //                super.destroyItem(container, position, object);
                    container.removeView(`object` as View)
                }
            }
            mTabLayout.addOnTabSelectedListener(object : XTabLayout.OnTabSelectedListener {
                override fun onTabSelected(tab: XTabLayout.Tab) {
                    val position = tab.position
                    viewPager.currentItem = position

                    listener?.onTabSelected(tab)
                }

                override fun onTabUnselected(tab: XTabLayout.Tab) {
                    listener?.onTabUnselected(tab)
                }

                override fun onTabReselected(tab: XTabLayout.Tab) {
                    listener?.onTabReselected(tab)
                }
            })
        }

        @JvmStatic
        fun setIndicatorWidth(tabLayout: TabLayout) {
            tabLayout.post {
                //了解源码得知 线的宽度是根据 tabView的宽度来设置的
                for (i in 0 until tabLayout.tabCount) {
                    setIndicatorWidth(tabLayout, i)
                }
            }
        }

        @JvmStatic
        fun setIndicatorWidth(tabLayout: TabLayout, index: Int) {
            val tabView = (tabLayout.getChildAt(0) as ViewGroup).getChildAt(index)
            val textView = tabLayout.getTabAt(index)?.customView?.findViewById<TextView>(R.id.tab_reddot_text)
                    ?: return

            tabView.setPadding(0, 0, 0, 0)
            textView.measure(0, 0)

            var value = tabLayout.width / tabLayout.tabCount - textView.measuredWidth

            val layoutParams = tabView.layoutParams as LinearLayout.LayoutParams

            value = if (index == 2) {
                0
            } else {
                value / 2
            }

            layoutParams.leftMargin = value
            layoutParams.rightMargin = value
            tabView.layoutParams = layoutParams
        }

        @JvmStatic
        fun resetIndicator(tabLayout: TabLayout) {
            //了解源码得知 线的宽度是根据 tabView的宽度来设置的
            for (i in 0 until tabLayout.tabCount) {
                resetIndicator(tabLayout, i)
            }
        }

        @JvmStatic
        fun resetIndicator(tabLayout: TabLayout, index: Int) {
            val width = tabLayout.width / tabLayout.tabCount

            val tabView = (tabLayout.getChildAt(0) as ViewGroup).getChildAt(index)
            val layoutParams = tabView.layoutParams as LinearLayout.LayoutParams
            layoutParams.width = width
            layoutParams.leftMargin = 0
            layoutParams.rightMargin = 0
            tabView.layoutParams = layoutParams
        }

    }

}