package com.tencent.edu.kernel.account

import android.app.Activity
import android.content.Intent
import android.util.Log
import androidx.lifecycle.DefaultLifecycleObserver
import androidx.lifecycle.LifecycleOwner
import com.google.protobuf.ByteString
import com.tencent.connect.common.Constants
import com.tencent.edu.common.callback.Callback
import com.tencent.edu.common.callback.ResultCallback
import com.tencent.edu.common.callback.VoidCallback
import com.tencent.edu.common.core.RegisterEventConstants
import com.tencent.edu.common.core.ThreadMgr
import com.tencent.edu.common.event.ConfigEventMgr
import com.tencent.edu.common.event.EventMgr
import com.tencent.edu.common.misc.LocalUri
import com.tencent.edu.common.utils.LogUtils
import com.tencent.edu.common.utils.SharedPrefsUtil
import com.tencent.edu.common.utils.ToastUtil
import com.tencent.edu.commonview.dialog.EduCustomizedDialog
import com.tencent.edu.flutter.channel.FEBroadcastChannel
import com.tencent.edu.flutter.global.FlutterConstants
import com.tencent.edu.flutter.route.EduFlutterActivity
import com.tencent.edu.kernel.AppRunTime
import com.tencent.edu.kernel.KernelConfig.DebugConfig
import com.tencent.edu.kernel.KernelEvent
import com.tencent.edu.kernel.KernelUtil
import com.tencent.edu.kernel.login.mgr.AccountMgr
import com.tencent.edu.kernel.login.mgr.LoginMgr
import com.tencent.edu.kernel.login.misc.LoginStatus
import com.tencent.edu.kernel.tiny.TinyChannelMgr
import com.tencent.edu.module.login.LoadingDialog
import com.tencent.edu.module.setting.DebugAppActivity
import com.tencent.edu.module.share.qq.Share2QQ
import com.tencent.edu.wxapi.WXEntryActivity
import com.tencent.edu.wxapi.WXOpenApi
import com.tencent.mm.opensdk.modelbase.BaseResp
import com.tencent.mm.opensdk.modelmsg.SendAuth
import com.tencent.pbaccountbind.pbaccountbind
import com.tencent.pbaccountlogin.pbaccountlogin
import com.tencent.tauth.IUiListener
import com.tencent.tauth.Tencent
import com.tencent.tauth.UiError
import com.tencent.tinylogin.PbTinyLogin
import com.tencent.wns.data.AccountInfo
import com.tencent.wns.data.UserId
import org.json.JSONArray
import org.json.JSONObject
import java.util.*

object AccountLoginOrBindMgr {

    private const val TAG = "AccountLoginOrBindMgr"

    const val REQUEST_CODE_FORCE_BIND_TELEPHONE = 0x103

    var mAllAssCacheAccount: String? = null
    var mAllAssAccs: List<pbaccountbind.AccInfoRsp>? = null

    @JvmStatic
    @JvmOverloads
    fun getAllAssAccs(callback: Callback<List<pbaccountbind.AccInfoRsp>>? = null) {
        // todo 手机号需求第一期，不请求此接口
        if (!isMobileForceBindEnable()) {
            return
        }

        AccountLoginOrBindRequester.getAllAssetAccounts(object : Callback<List<pbaccountbind.AccInfoRsp>?> {
            override fun onSucc(t: List<pbaccountbind.AccInfoRsp>?) {
                setCacheAccs(t)
                if (mAllAssAccs?.isNotEmpty() == true)
                    LogUtils.i(TAG, "getAllAssAccs success," + getArgs())

                callback?.onSucc(t)
            }

            override fun onError(errorCode: Int, errorMsg: String?) {
                LogUtils.i(TAG, "getAllAssAccs error,errorCode:$errorCode,errorMsg:$errorMsg")

                if (mAllAssAccs != null) {
                    if (mAllAssCacheAccount == KernelUtil.getAssetAccountId()) {
                        callback?.onSucc(mAllAssAccs)
                    } else {
                        mAllAssCacheAccount = null
                    }
                }
                callback?.onError(errorCode, errorMsg)
            }
        })
    }

    private fun getArgs(): String {
        val array = JSONArray()
        mAllAssAccs?.forEach {
            val obj = JSONObject().apply {
                putOpt("uid", it.uid.get())
                putOpt("uid_type", it.uid_type.get())
                putOpt("role_type", it.role_type.get())
                putOpt("is_default_asset", it.is_default_asset.get())
                putOpt("is_asset_account", it.is_asset_account.get())
                putOpt("ext_info", it.ext_info.get())
            }
            array.put(obj)
        }
        val obj = JSONObject()
        obj.put("accList", array.toString())
        return obj.toString()
    }

    private fun setCacheAccs(t: List<pbaccountbind.AccInfoRsp>?, account: String? = null) {
        mAllAssAccs = t
        mAllAssCacheAccount = account ?: KernelUtil.getAssetAccountId()
    }

    @JvmStatic
    @JvmOverloads
    fun getUserAllInfo(callback: Callback<Boolean>? = null) {
        AccountLoginOrBindRequester.getUserAllInfo(object : Callback<pbaccountlogin.GetUserAllInfoRsp> {

            override fun onSucc(t: pbaccountlogin.GetUserAllInfoRsp) {
                LogUtils.i(TAG, "getUserAllInfo succ.name:${t.nickname.get()},url:${t.headimgurl.get()}")

                AccountMgr.getInstance().updateNameAndHeadUrl(t.nickname.get(), t.headimgurl.get())
                // 是QQ登录的话保存roleType，替换FastLogin.requestRoleType
                if (KernelUtil.isQQAsset()) {
                    AccountMgr.getInstance().saveQQAccountExtData(t.role_type.get().toString())
                }

                val phoneNum = t.phone_number.get()
                val nationCode = t.nation_code.get()
                val tinyId = t.tiny_id.get()
                val isCreator = t.is_creator.get()//添加is_creator判断是否显示创作者中心的入口˛
                Log.i(TAG, "isCreator" + isCreator)
                AccountMgr.getInstance().savePhoneNumberAndTinyId(phoneNum, nationCode, tinyId)
                AccountMgr.getInstance().saveIsCreator(isCreator)
                notifyFlutterPhoneNumAndTinyIdUpdate(phoneNum, nationCode, tinyId)
                callback?.onSucc(t.is_phone_bind.get() == 0)
            }

            override fun onError(errorCode: Int, errorMsg: String?) {
                LogUtils.i(TAG, "getUserAllInfo err.errorCode:${errorCode},errorMsg:${errorMsg}")

                callback?.onError(errorCode, errorMsg)
            }
        })
    }

    @JvmStatic
    fun notifyFlutterPhoneNumAndTinyIdUpdate(phoneNum: String, nationCode: String, tinyId: String) {
        val info: MutableMap<String?, Any?> = HashMap()
        info["phoneNum"] = phoneNum
        info["nationCode"] = nationCode
        info["tinyId"] = tinyId
        // 登录成功，通知Flutter
        FEBroadcastChannel.broadcastEventWithInfo(FlutterConstants.UPDATE_PHONE_NUM_AND_TINYID_NOTIFICATION, info)
    }

    @JvmStatic
    @JvmOverloads
    fun switchLogin(switchTo: String, switchToType: Int, callback: VoidCallback, showDialog: Boolean = false) {
        val switchToLong: Long
        try {
            switchToLong = switchTo.toLong()
        } catch (e: Exception) {
            callback.onError(-3, "parse switchTo error,switchTo = $switchTo")
            return
        }

        var waitingDialog: EduCustomizedDialog? = null
        if (showDialog) {
            waitingDialog = LoadingDialog.createWaitingDialog(AppRunTime.getInstance().currentActivity, "切换中，请稍候")
            waitingDialog.show()
        }
        LogUtils.i("switchLogin", "switchLogin step 1 switch login start,switchTo=$switchTo,switchToType=$switchToType,oldUid=${KernelUtil.getAssetAccountId()},oldType=${KernelUtil.getAssetLoginType()}")
        // 1.调用SwitchLogin接口
        AccountLoginOrBindRequester.switchLogin(switchToLong, switchToType, object : Callback<pbaccountlogin.SwitchLoginRsp> {
            override fun onSucc(t: pbaccountlogin.SwitchLoginRsp) {

                val token = PbTinyLogin.TinyUserToken.newBuilder()
                        .setQqTokenTypeValue(t.token.qq_token_type.get())
                        .setToken(ByteString.copyFrom(t.token.token.get().toByteArray()))
                        .setTokenTypeValue(t.token.token_type.get()).build()

                LogUtils.i("switchLogin", "switchLogin step 2 conn start,token=$token")

                val info = AccountInfo()
                info.nameAccount = switchTo
                info.userId = UserId().apply {
                    uid = switchTo
                    uin = switchToLong
                }
                // 需要在这个步骤更新资产账号信息，否则tiny.unifiedConn带过去的还是qq切换前的资产id
                LoginMgr.getInstance().saveAssAccInfoAndToken(info, token)

                // 2.用新的token执行tiny.conn
                TinyChannelMgr.getInstance().unifiedConn(token, object : VoidCallback {
                    override fun onError(errorCode: Int, errorMsg: String?) {
                        waitingDialog?.dismiss()
                        LogUtils.i("switchLogin", "switchLogin step 2 conn error,errorCode=$errorCode,errorMsg=$errorMsg")

                        switchLoginFail(errorMsg)
                    }

                    override fun onSucc() {
                        LogUtils.i("switchLogin", "switchLogin step 3 refresh user info start")
                        // 3.刷新用户信息，不管获取成功还是失败，都重新刷一遍Flutter侧信息
                        getUserAllInfo(object : Callback<Boolean> {
                            override fun onSucc(t: Boolean?) {
                                waitingDialog?.dismiss()
                                LogUtils.i("switchLogin", "switchLogin step 3 refresh user info success")

                                // 登录成功状态同步
                                LoginMgr.getInstance().notifyLoginSuccess(switchToType, null)

                                // 4.回调关闭切换账号页面
                                callback.onSucc()
                            }

                            override fun onError(errorCode: Int, errorMsg: String?) {
                                waitingDialog?.dismiss()
                                LogUtils.e("switchLogin", "switchLogin step 3 error,errorCode=$errorCode,errorMsg=$errorMsg")

                                switchLoginFail(errorMsg)
                            }

                        })
                    }

                })
            }

            private fun switchLoginFail(errorMsg: String?) {
                callback.onError(-999, errorMsg)

                ToastUtil.shortToast(AppRunTime.getApplicationContext(), "切换登录失败，请重新登录")
                // 500ms延时目的是让切换登录页先退出，
                ThreadMgr.postToUIThread({
                    ConfigEventMgr.getInstance().notifyRegisterClass(RegisterEventConstants.LOGIN_CONFIG, KernelEvent.EVENT_SWITCH_LOGIN_FAIL)
                }, 500)
            }

            override fun onError(errorCode: Int, errorMsg: String?) {
                waitingDialog?.dismiss()
                LogUtils.e("switchLogin", "switchLogin step 1 switch login error,errorCode=$errorCode,errorMsg=$errorMsg")

                callback.onError(errorCode, errorMsg)
            }

        })
    }

    fun bindPhoneAcc(phoneNum: String,
                     nationCode: String,
                     verifyCode: String,
                     codeType: String,
                     callback: Callback<pbaccountbind.BindAccRsp>) {
        AccountLoginOrBindRequester.bindPhoneAcc(phoneNum, nationCode, verifyCode, codeType, object : Callback<pbaccountbind.BindAccRsp> {

            override fun onSucc(t: pbaccountbind.BindAccRsp) {
                setCacheAccs(t.bind_accounts.get(), t.asset_uid.get().toString())

                callback.onSucc(t)
            }

            override fun onError(errorCode: Int, errorMsg: String?) {
                callback.onError(errorCode, errorMsg)
            }
        })
    }

    fun bindPhoneAccOneKey(encryptedPhone: String,
                           msgId: String,
                           token: String,
                           callback: Callback<pbaccountbind.BindAccRsp>) {
        AccountLoginOrBindRequester.bindPhoneAccOneKey(encryptedPhone, msgId, token, object : Callback<pbaccountbind.BindAccRsp> {

            override fun onSucc(t: pbaccountbind.BindAccRsp) {
                setCacheAccs(t.bind_accounts.get(), t.asset_uid.get().toString())

                callback.onSucc(t)
            }

            override fun onError(errorCode: Int, errorMsg: String?) {
                callback.onError(errorCode, errorMsg)
            }
        })
    }

    var mGrayCacheAccount: String? = null
    var isAssGray: Boolean? = null

    @JvmStatic
    fun isAssGray(callback: ResultCallback<Boolean>) {
        if (isMobileForceBindEnable()) {
            callback.onResult(true)
            return
        }

        // todo 手机号需求第一期,默认返回false
        if (true) {
            callback.onResult(false)
            return
        }

        if (AccountMgr.getInstance().currentAccountData == null || !LoginStatus.isLogin()) {
            callback.onResult(false)
            return
        }

        // 如果当前有缓存，则直接使用缓存数据
        fun useCache(): Boolean {
            return if (isAssGray != null && mGrayCacheAccount == KernelUtil.getAssetAccountId()) {
                callback.onResult(isAssGray)
                true
            } else false
        }

        if (useCache()) {
            return
        }

        AccountLoginOrBindRequester.isAssGray(object : Callback<Boolean> {
            override fun onSucc(t: Boolean?) {
                isAssGray = t
                mGrayCacheAccount = KernelUtil.getAssetAccountId()

                callback.onResult(t == true)
            }

            override fun onError(errorCode: Int, errorMsg: String?) {
                if (useCache()) {
                    return
                }

                callback.onResult(false)
            }
        })
    }

    private fun isMobileForceBindEnable(): Boolean {
        if (DebugConfig.MOBILE_FORCE_BIND == null) {
            DebugConfig.MOBILE_FORCE_BIND = SharedPrefsUtil.getInt(DebugAppActivity.SHARED_PRE_TABLE, DebugAppActivity.SHARED_PRE_KEY_MOBILE_FORCE_BIND, 0)
        }

        return DebugConfig.MOBILE_FORCE_BIND == 1 && !LoginStatus.isOriginSSOLogin()
    }

    @JvmStatic
    fun isCanSwitchLogin(): Boolean {
        return LoginMgr.getInstance().isLogin && (mAllAssAccs.isNullOrEmpty() || mAllAssAccs!!.size == 1).not()
    }

    @JvmStatic
    fun jump2SwitchLoginPage() {
        if (!isCanSwitchLogin()) {
            return
        }

        LocalUri.jumpToEduUri("tencentedu://openpage/flutter?route=switchLoginPage")
    }

    /**
     * 登录账号是否和当前资产账号一致
     */
    @JvmStatic
    fun isLoginAssetsSame(): Boolean {
        return AccountMgr.getInstance().currentAccountData.assetAccountId == AccountMgr.getInstance().currentAccountData.originAccountId
    }

    /**
     * 当前是否可以使用wns降级
     */
    @JvmStatic
    fun isCanUseWns(): Boolean {
        // 这里如果是sso登录，或是logout状态，或者是非手机号且登录账号和资产账号一致，则可以使用wns
        return LoginStatus.isOriginSSOLogin() || !LoginMgr.getInstance().isLogin || (!LoginStatus.isOriginMobileLogin() && isLoginAssetsSame())
//        return false
    }

    @JvmStatic
    fun jump2ChangeBindingMobilePage() {
        val info = AccountMgr.getInstance().currentAccountData
        LocalUri.jumpToEduUri("tencentedu://openpage/flutter?route=changeBindingMobilePage&args={\"mobile\":\"${info.phoneNumber}\",\"nation_code\":\"${info.nationCode}\"}")
    }

    fun jump2AccountAndSafePage() {
        LocalUri.jumpToEduUri("tencentedu://openpage/account_and_safe?route=acountsafe&args=${JSONObject().apply {
            AccountMgr.getInstance().currentAccountData.run {
                put("tinyId", tinyID)
                put("uid", assetAccountId)
                put("phone", phoneNumber)
                put("nationCode", nationCode)
            }
        }}")
    }

    @JvmStatic
    fun authQQ(activity: Activity, callback: Callback<String?>) {
        LogUtils.i("bindQQ", "authQQ start")
        Share2QQ.login(activity, object : IUiListener {
            override fun onComplete(o: Any) {
                var accessToken: String? = null
                try {
                    val obj = JSONObject(o.toString())
                    accessToken = obj.optString("access_token")
                } catch (e: java.lang.Exception) {
                }

                LogUtils.i("bindQQ", "handleAuthQQ code:$accessToken")
                callback.onSucc(accessToken)
            }

            override fun onError(uiError: UiError) {
                LogUtils.e("bindQQ", "handleAuthQQ error:$uiError")
                callback.onSucc(null)
            }

            override fun onCancel() {
                LogUtils.i("bindQQ", "handleAuthQQ cancel")
                callback.onSucc(null)
            }

            override fun onWarning(i: Int) {
                LogUtils.i("bindQQ", "handleAuthQQ warn:$i")
                callback.onSucc(null)
            }
        })
    }

    @JvmStatic
    fun handleAuthQQ(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
        return if (requestCode == Constants.REQUEST_LOGIN) {
            Tencent.onActivityResultData(requestCode, resultCode, data, Share2QQ.mLoginListener)
            true
        } else {
            false
        }
    }

    @JvmStatic
    fun authWX(activity: EduFlutterActivity, callback: Callback<String>) {
        LogUtils.i("bindWX", "authWX start")

        val openApi = WXOpenApi()
        openApi.initWXApi(activity)

        var timeOut: Boolean? = null
        var timeOutFlag = false
        val observer = object : DefaultLifecycleObserver {
            override fun onPause(owner: LifecycleOwner) {
                timeOutFlag = true
            }

            override fun onResume(owner: LifecycleOwner) {
                if (!timeOutFlag) {
                    LogUtils.i("bindWX", "authWX timeOutFlag = false")
                    return
                }

                LogUtils.i("bindWX", "authWX onResume")

                activity.lifecycle.removeObserver(this)

                if (timeOut != null) {
                    LogUtils.i("bindWX", "authWX already result")

                    return
                }
                ThreadMgr.postToUIThread({
                    if (timeOut != null) {
                        LogUtils.i("bindWX", "authWX already result")

                        return@postToUIThread
                    }

                    LogUtils.w("bindWX", "authWX time out")
                    timeOut = true
                    callback.onSucc(null)
                }, 1000)
            }

        }

        WXEntryActivity.mBindListener = object : Callback<BaseResp> {
            override fun onSucc(t: BaseResp) {
                activity.lifecycle.removeObserver(observer)

                if (timeOut == true) {
                    return
                }
                timeOut = false
                callback.onSucc(handleAuthWX(t))
            }

            override fun onError(errorCode: Int, errorMsg: String?) {
                activity.lifecycle.removeObserver(observer)

                if (timeOut == true) {
                    return
                }
                timeOut = false
                callback.onSucc(null)
            }
        }
        openApi.auth()

        // 如果账号与安全页重现出现，1秒后还没有动作，就认为绑定超时
        activity.lifecycle.addObserver(observer)
    }

    private fun handleAuthWX(rsp: BaseResp): String? {
        when (rsp.errCode) {
            BaseResp.ErrCode.ERR_OK -> {
                if (rsp is SendAuth.Resp) {
                    //获取微信返回的code
                    val code = rsp.code
                    LogUtils.i("bindWX", "handleAuthWX code:$code")
                    return code
                } else {
                    LogUtils.e("bindWX", "handleAuthWX unknown error," + rsp.errStr)
                }
            }
            BaseResp.ErrCode.ERR_USER_CANCEL -> {
                LogUtils.i("bindWX", "handleAuthWX user cancel," + rsp.errStr)
            }
            BaseResp.ErrCode.ERR_BAN -> {
                LogUtils.e("bindWX", "handleAuthWX err ban," + rsp.errStr)
            }
            else -> {
                LogUtils.e("bindWX", "handleAuthWX unknown error," + rsp.errStr)
            }
        }
        return null
    }

    @JvmStatic
    fun isPhoneBind(callback: Callback<Boolean>) {
        LogUtils.i(TAG, "isPhoneBind start")
        // 非登录状态不判断手机号是否绑定
        if (!LoginMgr.getInstance().isLogin) {
            LogUtils.i(TAG, "isPhoneBind not login,true")
            callback.onSucc(true)
            return
        }

        val accountData = AccountMgr.getInstance().currentAccountData
        // 如果用户有手机号，则已绑定
        if (!accountData.phoneNumber.isNullOrBlank()) {
            LogUtils.i(TAG, "isPhoneBind phoneNumber is not empty,true")
            callback.onSucc(true)
            return
        }

        isAssGray(ResultCallback<Boolean> { result ->
            // 命中灰度
            if (result) {
                // 通过接口判断用户是否已绑手机号
                AccountLoginOrBindRequester.isPhoneBind(object : Callback<Boolean> {
                    override fun onSucc(t: Boolean) {
                        LogUtils.i(TAG, "isPhoneBind success,$t")

                        callback.onSucc(t)
                    }

                    override fun onError(errorCode: Int, errorMsg: String?) {
                        LogUtils.e(TAG, "isPhoneBind error,errCode:$errorCode,errMsg:$errorMsg")

                        callback.onError(errorCode, errorMsg)
                    }

                })
            } else {
                LogUtils.i(TAG, "isPhoneBind not hit gray,true")
                callback.onSucc(true)
            }
        })


    }

}