package com.tencent.edu.kernel.dclog

import com.google.gson.JsonObject
import com.tencent.edu.kernel.dclog.model.ReportDcLogCgiReq
import com.tencent.edu.kernel.http.HttpModel
import java.lang.Exception
import java.util.concurrent.LinkedBlockingQueue
import java.util.concurrent.ThreadPoolExecutor
import java.util.concurrent.TimeUnit

internal object FullLinkLogUploader {

    private const val POOL_SIZE = 3
    private const val ALIVE_TIME = 60L
    private const val QUEUE_SIZE = 512

    private val mExecutor = ThreadPoolExecutor(
            POOL_SIZE,
            POOL_SIZE,
            ALIVE_TIME,
            TimeUnit.SECONDS,
            LinkedBlockingQueue(QUEUE_SIZE),
            ThreadPoolExecutor.DiscardOldestPolicy()
    )

    fun upload(
            req: ReportDcLogCgiReq,
            dealWithError: (String) -> Unit,
            dealWithResp: (JsonObject?) -> Unit
    ) {
        mExecutor.execute {
            val call = HttpModel.reportDcLogSync(req)

            try {
                val response = call.execute()
                if (!response.isSuccessful) {
                    dealWithError(response.message())
                }

                dealWithResp(response.body())
            } catch (e: Exception) {
                dealWithError(e.toString() + if (e.cause != null) " cause=" + e.cause.toString() else "")
            }
        }
    }

    fun unInit() {
        mExecutor.shutdownNow()
    }

}