package com.tencent.edu.kernel.dclog

import android.text.TextUtils
import com.google.gson.JsonObject
import com.tencent.edu.common.utils.LogUtils
import com.tencent.edu.kernel.KernelConfig
import com.tencent.edu.kernel.dclog.model.ReportDcLogCgiReq
import com.tencent.edu.kernel.http.HttpModel.reportDcLog
import com.tencent.edu.kernel.http.util.GsonUtil.getInt
import io.reactivex.Observer
import io.reactivex.disposables.Disposable

/**
 * 日志上报管理类-Cgi接口
 *
 * @author aidanyang
 * @date 2020-04-01
 */
object ReportDcLogCgiRequester {
    private const val TAG = "DcLogCgiReq"

    @JvmStatic
    fun upload(req: ReportDcLogCgiReq, listener: CgiRequesterListener, isLog: Boolean) {

        val dealWithError: (String) -> Unit = { msg ->
            listener.onError()
            le("request error=$msg")
        }

        val dealWithResp: (JsonObject?) -> Unit = { jsonResp ->
            if (jsonResp == null || TextUtils.isEmpty(jsonResp.toString())) {
                listener.onError()
                lw("onResponse: body is empty")
            } else {
                li(jsonResp.toString())

                // -1是禁止上传
                val logLevel = getInt(jsonResp, "log_level", -2)
                var uploadNum = getInt(jsonResp, "upload_num", -1)
                var intervalTime = getInt(jsonResp, "interval_time", -1)
                // 后台这两个返回值默认为0
                uploadNum = if (uploadNum == 0) -1 else uploadNum
                intervalTime = if (intervalTime == 0) -1 else intervalTime
                listener.onSuccess(logLevel, uploadNum, intervalTime)
            }
        }

        if (isLog) {
            FullLinkLogUploader.upload(req, dealWithError, dealWithResp)
            return
        }

        reportDcLog(req, object : Observer<JsonObject> {
            override fun onSubscribe(d: Disposable) {
            }

            override fun onNext(jsonResp: JsonObject) {
                dealWithResp(jsonResp)
            }

            override fun onError(e: Throwable) {
                dealWithError(e.toString() + if (e.cause != null) " cause=" + e.cause.toString() else "")
            }

            override fun onComplete() {}
        })
    }

    private val isShowLog: Boolean
        get() = KernelConfig.DebugConfig.FULLLINK_LOG_REQ_FLAG == 1

    private fun le(msg: String) {
        if (isShowLog) {
            LogUtils.le(TAG, msg)
        }
    }

    private fun li(msg: String) {
        if (isShowLog) {
            LogUtils.li(TAG, msg)
        }
    }

    private fun lw(msg: String) {
        if (isShowLog) {
            LogUtils.li(TAG, msg)
        }
    }

    @JvmStatic
    fun ld(log: String?) {
        if (isShowLog) {
            LogUtils.ld(TAG, log)
        }
    }

    interface CgiRequesterListener {
        fun onSuccess(logLevel: Int, uploadNum: Int, intervalTime: Int)
        fun onError()
    }
}