package com.tencent.edu.kernel.http

import android.util.Log
import androidx.lifecycle.DefaultLifecycleObserver
import androidx.lifecycle.LifecycleOwner
import com.google.gson.JsonObject
import com.tencent.edu.common.utils.MiscUtils
import com.tencent.edu.kernel.KernelUtil
import com.tencent.edu.kernel.http.data.HttpResponse
import com.tencent.edu.kernel.http.retrofit.RetrofitManager
import com.tencent.edu.kernel.tde.TdeEnvUtil
import com.tencent.edu.module.audiovideo.entity.ContractTeacherInfo
import com.tencent.edu.module.homepage.model.EnjoyStudyCardDetail
import com.tencent.edu.module.setting.DebugDcLogActivity
import io.reactivex.Observable
import io.reactivex.Observer
import io.reactivex.Scheduler
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import io.reactivex.schedulers.Schedulers
import org.json.JSONObject
import retrofit2.Call
import retrofit2.Response
import java.util.concurrent.TimeUnit


/**
 * @author aidanyang
 * @date 2020-08-11
 */
object HttpModel {

    private val service = RetrofitManager.create(HttpApi::class.java)

    private val disposableMap = hashMapOf<Any, ArrayList<Disposable>>()

    /**
     * 获取课件更新信息
     */
    @JvmStatic
    @JvmOverloads
    fun fetchCoursewareNotice(cid: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.fetchCoursewareNotice(cid), observer, tag)
    }

    /**
     * 获取课件下载url
     */
    @JvmStatic
    @JvmOverloads
    fun fetchDownloadUrl(id: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.fetchDownloadUrl(id), observer, tag)
    }

    /**
     * 获取直播间联系信息
     */
    @JvmStatic
    @JvmOverloads
    fun fetchContractTeacherInfo(courseId: String, observer: Observer<HttpResponse<ContractTeacherInfo>>, tag: Any? = null) {
        setSubscribe(service.fetchContractTeacherInfo(courseId), observer, tag)
    }

    /**
     * 设置直播间联系弹窗不再自动弹出
     */
    @JvmStatic
    @JvmOverloads
    fun setNoMoreNotify(termId: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.setNoMoreNotify(termId), observer, tag)
    }

    /**
     * 上传全链路日志
     */
    @JvmStatic
    @JvmOverloads
    fun reportDcLog(req: Any, observer: Observer<JsonObject>, tag: Any? = null) {
        var host : String
        if (DebugDcLogActivity.isTestEnv()) {
            if (TdeEnvUtil.isTdeOpen()) {
                host = TdeEnvUtil.getTdeUrl("http://debug.edu.oa.com/")
                host = host.substring(0, host.length - 7)  //删除最后加上的_tde/，否则拼接错误，会404
            } else {
                host = "http://debug.edu.oa.com"
            }
        } else {
            if (TdeEnvUtil.isTdeOpen()) {
                host = TdeEnvUtil.getTdeUrl("https://report.edu.qq.com/")
                host = host.substring(0, host.length - 7) //删除最后加上的_tde/，否则拼接错误，会404
            } else {
                host = "https://report.edu.qq.com"
            }
        }
        if (TdeEnvUtil.isTdeOpen()) {
            setSubscribe(service.reportDcLogWithTde(host, req), observer, tag)
        } else {
            setSubscribe(service.reportDcLog(host, req), observer, tag)
        }
    }

    /**
     * 上传全链路日志(同步)
     */
    @JvmStatic
    fun reportDcLogSync(req: Any): Call<JsonObject> {
        var host : String
        if (DebugDcLogActivity.isTestEnv()) {
            if (TdeEnvUtil.isTdeOpen()) {
                host = TdeEnvUtil.getTdeUrl("http://debug.edu.oa.com/")
                host = host.substring(0, host.length - 7) //删除最后加上的_tde/，否则拼接错误，会404
            } else {
                host = "http://debug.edu.oa.com"
            }
        } else {
            if (TdeEnvUtil.isTdeOpen()) {
                host = TdeEnvUtil.getTdeUrl("https://report.edu.qq.com/")
                host = host.substring(0, host.length - 7) //删除最后加上的_tde/，否则拼接错误，会404
            } else {
                host = "https://report.edu.qq.com"
            }
        }
        if (TdeEnvUtil.isTdeOpen()) {
            return service.reportDcLogSyncWithTde(host, req)
        } else {
            return service.reportDcLogSync(host, req)
        }
    }

    @JvmStatic
    @JvmOverloads
    fun checkUserSetPhone(courseId: String, termId: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.checkUserSetPhone(
                MiscUtils.getBkn(),
                courseId,
                termId
        ), observer, tag)
    }

    @JvmStatic
    @JvmOverloads
    fun getCollectStatus(contentId: String, externalFlag: Int, seriesId: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getCollectStatus(
                contentId,
                externalFlag,
                seriesId
        ), observer, tag)
    }

    @JvmStatic
    @JvmOverloads
    fun getCollectStatus(retId: String, retType: Int, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getCollectStatus(retId, retType), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun updateCollectStatuss(req: Any, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.updateCollectStatuss(req), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun updateCollectStatus(req: Any, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.updateCollectStatus(req), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun reportResOperate(opType: Int, actionTimestamp: Long, action: String, page: String, module: String, contentId: String, contentType: String, urlPage: String, urlModule: String, urlPosition: Int, sessionPath: String, platform: Int, userType: Int, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.reportResOperate(opType, actionTimestamp, action, page, module, contentId, contentType, urlPage, urlModule, urlPosition, sessionPath, platform, userType), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun getFavStatus(retId: String, retType: Int, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getFavStatus(retId, retType), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun updateFavStatus(req: Any, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.updateFavStatus(req), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun getCreatorUserBaseInfo(creatorId: Long, creatorUid: Long, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getCreatorUserBaseInfo(creatorId, creatorUid), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun getContentInfo(preview: Int, contentId: String, externalFlag: Int, seriesId: String?, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getContentInfo(
                preview,
                contentId,
                externalFlag,
                seriesId,
                1
        ), observer, tag)
    }

    @JvmStatic
    @JvmOverloads
    fun getContentRecommend(page: Int, count: Int, platform: Int, mt: String, st: String, tt: String, label: String, sceneId: Int, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getContentRecommend(
                page,
                count,
                platform,
                mt,
                st,
                tt,
                label,
                sceneId
        ), observer, tag)
    }

    @JvmStatic
    @JvmOverloads
    fun fetchPushMsgWithReqType(url: String, req: Any, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.fetchPushMsgWithReqType(
                url,
                req
        ), observer, tag)
    }

    @JvmStatic
    @JvmOverloads
    fun fetchExplainList(termId: String, pageSize: Int, lastMsgId: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.fetchExplainList(
                termId,
                pageSize,
                lastMsgId,
                1
        ), observer, tag)
    }

    @JvmStatic
    @JvmOverloads
    fun checkWithinWhiteList(q: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.checkWithinWhiteList(q), observer, tag)
    }

    @JvmStatic
    @JvmOverloads
    fun requestEmploymentData(id: Long, user_scene: Int, platform: Int, type: Int, term_id: Long, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.requestEmploymentData(id, user_scene, platform, type, term_id), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun requestBottomTabIcon(observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.requestBottomTabIcon(), observer, tag)
    }

    @JvmStatic
    @JvmOverloads
    fun flutterGet(url: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.flutterGet(url), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun getAppSplash(platform: Int, cookies: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getAppSplash(platform, cookies), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun checkSmartCampus(termId: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.checkSmartCampus(termId), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun requestSmartCampusLoginKey(url: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.requestSmartCampusLoginKey(url), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun getCourses(present_flag: Int, is_full_info: Int, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getCourses(present_flag, is_full_info), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun getAdBindPhoneChannel(observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getAdBindPhoneChannel(), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun getTermCanApply(termId: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getTermCanApply(termId), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun requestOperation(observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.requestOperation(), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun requestOperationDirect(observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.requestOperationDirect(), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun grantOrGetStudyCard(enterPageIndex: Int, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.grantOrGetStudyCard(enterPageIndex), observer, tag)
    }

    @JvmStatic
    @JvmOverloads
    fun getStudyCardGrantInfo(observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.studyCardGrantInfo, observer, tag)
    }

    @JvmStatic
    @JvmOverloads
    fun getCouponMarketing(url: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getCouponMarketing(url), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun getStudyCourseDetail(termId: String, observer: Observer<HttpResponse<EnjoyStudyCardDetail>>, tag: Any? = null) {
        setSubscribe(service.getStudyCourseDetail(termId), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun syncCanGrantStudyCard(termId: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.syncCanGrantStudyCard(termId), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun reportVideoAction(req: Map<String, Any>, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.reportVideoAction(req), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun requestSSOAuth(url: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.requestSSOAuth(url), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun getMobile(observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getMobile(MiscUtils.getBkn().toString()), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun searchOperationalPosition(observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.searchOperationalPosition(), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun uploadPicture(bitmapBase64: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.uploadPicture("2", bitmapBase64), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun fetchPartnerShareKey(actId: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.fetchPartnerShareKey(actId, "2"), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun getGroupEncode(group: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getGroupEncode(group), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun getSignInOffline(orderId: Long, examId: Long, longitude: Double, latitude: Double, uuid: String, busid: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.signInOffline(orderId, examId, longitude, latitude, uuid, busid), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun request(url: String, needCookie: Boolean = true, observer: Observer<JsonObject>, tag: Any? = null) {
        if (needCookie) {
            setSubscribe(service.requestWithCookie(url), observer, tag, observeOn = AndroidSchedulers.mainThread())
        } else {
            setSubscribe(service.request(url), observer, tag, observeOn = AndroidSchedulers.mainThread())
        }
    }

    @JvmStatic
    @JvmOverloads
    fun requestResponse(url: String, needCookie: Boolean = true, observer: Observer<Response<JsonObject>>, tag: Any? = null) {
        if (needCookie) {
            setSubscribe(service.requestResponseWithCookie(url), observer, tag, observeOn = AndroidSchedulers.mainThread())
        } else {
            setSubscribe(service.requestResponse(url), observer, tag, observeOn = AndroidSchedulers.mainThread())
        }
    }

    @JvmStatic
    @JvmOverloads
    fun requestResponsePost(url: String, needCookie: Boolean = true, observer: Observer<Response<JsonObject>>, tag: Any? = null) {
        if (needCookie) {
            setSubscribe(service.requestResponseWithCookiePost(url), observer, tag, observeOn = AndroidSchedulers.mainThread())
        } else {
            setSubscribe(service.requestResponsePost(url), observer, tag, observeOn = AndroidSchedulers.mainThread())
        }
    }

    @JvmStatic
    @JvmOverloads
    fun reportActivationEvent(imei: String, oaid: String, uin: String, platform: String, eventType: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.reportActivationEvent(imei, oaid, uin, platform, eventType), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun getWxAccountComponentInfo(
            courseId: String,
            termId: String,
            aid: String,
            isPkg: Boolean,
            timeout: Long,
            observer: Observer<JsonObject>,
            tag: Any? = null
    ) {
        setSubscribe(service.getWxAccountComponentInfo(
                courseId,
                termId,
                aid,
                if (isPkg) 1 else 0,
                KernelUtil.getAssetAccountId(),
                // 写死以下两个参数
                1,
                1),
                observer, tag,
                observeOn = AndroidSchedulers.mainThread(),
                timeout = timeout)
    }

    @JvmStatic
    @JvmOverloads
    fun getUserWindInfo(activityCode: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getUserWindInfo(activityCode), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun getUpperHasJoinActivity(activityCode: String, creatorId: Long, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getUpperHasJoinActivity(activityCode, creatorId), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun voteForCreator(activityCode: String, authorId: Long, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.voteForCreator(activityCode, authorId), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun getContentPendentInfo(fileId: String, platform: String, observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getContentPendentInfo(fileId, platform), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }
    @JvmStatic
    @JvmOverloads
    fun getUgcUploadSign(observer: Observer<JsonObject>,
                         tag: Any? = null) {
        setSubscribe(service.getUgcUploadSign(), observer, tag,
                observeOn = AndroidSchedulers.mainThread())
    }

    @JvmStatic
    @JvmOverloads
    fun getKeNickName(observer: Observer<JsonObject>, tag: Any? = null) {
        setSubscribe(service.getMobile(MiscUtils.getBkn().toString()), observer, tag, observeOn = AndroidSchedulers.mainThread())
    }

    /**
     * 监听生命周期，在onDestroy时清空tag为当前owner的所有请求
     */
    @JvmStatic
    fun addLifecycleObserver(owner: LifecycleOwner?) {
        if (owner == null) {
            return
        }

        owner.lifecycle.addObserver(object : DefaultLifecycleObserver {
            override fun onDestroy(owner: LifecycleOwner) {
                cancel(owner)
            }
        })
    }

    /**
     * 通过tag取消一个请求
     */
    @JvmStatic
    fun cancel(tag: Any?) {
        if (tag == null) {
            return
        }

        disposableMap.remove(tag)?.run {
            for (disposable:Disposable? in this) {
                if (disposable?.isDisposed == true) {
                    return@run
                }

                disposable?.dispose()
            }
            clear()
        }
    }

    /**
     * 取消当前所有请求
     */
    @JvmStatic
    fun cancelAll() {
        while (disposableMap.isNotEmpty()) {
            cancel(disposableMap.keys.last())
        }
    }

    /**
     * 订阅网络请求
     */
    private fun <T> setSubscribe(
            observable: Observable<T>,
            // 请求结果Observer
            observer: Observer<T>,
            // 设置tag，可用于取消请求
            tag: Any? = null,
            // IO线程访问网络
            subscribeOn: Scheduler = Schedulers.io(),
            // 设置请求结果在子线程或主线程回调
            observeOn: Scheduler = Schedulers.newThread(),
            // 默认15秒超时
            timeout: Long = 15,
            timeUnit: TimeUnit = TimeUnit.SECONDS
    ) {

        observable.subscribeOn(subscribeOn)
                .observeOn(observeOn)
                .timeout(timeout, timeUnit)
                .subscribe(if (tag == null) observer else object : Observer<T> {
                    var disposable: Disposable? = null

                    override fun onSubscribe(d: Disposable) {
                        disposable = d
                        if (disposableMap[tag] == null) {
                            disposableMap[tag] = arrayListOf(d)
                        } else {
                            disposableMap[tag]?.add(d)
                        }
                    }

                    override fun onNext(t: T) {
                        observer.onNext(t)
                    }

                    override fun onError(e: Throwable) {
                        observer.onError(e)
                    }

                    override fun onComplete() {
                        observer.onComplete()
                        disposableMap[tag]?.remove(disposable)
                    }
                })
    }

}