package com.tencent.edu.kernel.http.data

import com.google.gson.JsonObject
import com.tencent.edu.kernel.http.util.GsonUtil
import io.reactivex.Observer
import io.reactivex.disposables.Disposable

abstract class JsonDataObserver : Observer<JsonObject> {

    override fun onComplete() {}

    override fun onSubscribe(d: Disposable) {}

    override fun onNext(jsonResp: JsonObject) {
        try {
            if (jsonResp.toString().isBlank()) {
                onError(-996, "response body is empty")
                return
            }
            val retcode = GsonUtil.getInt(jsonResp, "retcode", -1)
            if (retcode != 0) {
                val msg = GsonUtil.getString(jsonResp, "msg", "")
                onError(retcode, msg.toString())
                return
            }
            if (onSuccess(jsonResp)) {
                return
            }
            val result = jsonResp.getAsJsonObject("result")
            if (result == null) {
                onError(-997, "result = null")
                return
            }
            onSuccessWithResult(result)
        } catch (e: Exception) {
            onError(-998, e.message.toString())
        }
    }

    override fun onError(e: Throwable) {
        onError(-999, e.toString() + if (e.cause != null) " cause=" + e.cause.toString() else "")
        // Observer在onError时不回调onComplete
        onComplete()
    }

    open fun onSuccess(jsonResp: JsonObject): Boolean = false

    open fun onSuccessWithResult(result: JsonObject) {}

    open fun onError(code: Int, msg: String) {
        onError(msg)
    }

    open fun onError(e: String) {}
}