package com.tencent.edu.kernel.http.data

import io.reactivex.Observer
import io.reactivex.disposables.Disposable
import java.lang.Exception

abstract class ObjectDataObserver<T> : Observer<HttpResponse<T>> {

    override fun onComplete() {}

    override fun onSubscribe(d: Disposable) {}

    override fun onNext(response: HttpResponse<T>) {
        try {
            if (response.retcode != 0) {
                onError("retcode=${response.retcode}")
                return
            }
            if (onSuccess(response)) {
                return
            }
            onSuccessWithResult(response.result)
        } catch (e: Exception) {
            onError(e.message.toString())
        }
    }

    override fun onError(e: Throwable) {
        onError(e.toString() + if (e.cause != null) " cause=" + e.cause.toString() else "")
        // Observer在onError时不回调onComplete
        onComplete()
    }

    open fun onSuccess(response: HttpResponse<T>): Boolean = false

    open fun onSuccessWithResult(result: T) {}

    abstract fun onError(msg: String)
}