package com.tencent.edu.kernel.http.retrofit

import android.util.Log
import com.tencent.edu.common.utils.LogUtils
import com.tencent.edu.kernel.KernelUtil
import com.tencent.edu.kernel.login.misc.LoginStatus
import com.tencent.edu.kernel.tde.TdeEnvUtil
import okhttp3.*
import okio.Buffer

/**
 * 日志拦截、注解处理
 * @author aidanyang
 * @date 2020-08-17
 */
class MyInterceptor(private val cookieList: HashSet<String>, private val referList: HashSet<String>) : Interceptor {

    override fun intercept(chain: Interceptor.Chain): Response {
        var request: Request = chain.request()

        val headers = request.headers().newBuilder()

        var needRebuild = false
        if (cookieList.contains(getUrlWithoutParams(request.url().toString())) || cookieList.contains(request.url().toString())) {
            var cookie : String
            //区分是否全链路上报，是的话直接用tde的cookie即可。以防取generalcookie次数太多，生成太多次数据库调用线程，导致其他线程崩了
            if (TdeEnvUtil.isTdeOpen()) {
                if (request.url().toString() == "https://tde.ke.qq.com/_tde_/report_edu_qq_com/data_report/fulllink_report/client_ke") {//tde时全链路的地址，非常硬的硬编码
                    cookie = TdeEnvUtil.getTdeCookie().substring(1); //去除第一个分号
                    headers.add("Host", "tde.ke.qq.com")
                } else {
                    cookie = KernelUtil.genGeneralCookie();
                }
            } else {
                cookie = KernelUtil.genGeneralCookie();
            }
            if (LoginStatus.isOriginQQLogin() && request.url().toString().contains("/employment/data") && !judgeNormalCookie(cookie) ) {
                chain.call().cancel()
            }
            headers.add("Cookie", cookie)
            needRebuild = true
        }
        if (referList.contains(getUrlWithoutParams(request.url().toString())) || referList.contains(request.url().toString())) {
            headers.add("Referer", REFERER)
            needRebuild = true
        }

        if (needRebuild) {
            request = request.newBuilder().headers(headers.build()).build()
        }

        val t1 = System.currentTimeMillis()
        val response: Response = chain.proceed(request)
        val t2 = System.currentTimeMillis()
        val mediaType = response.body()?.contentType()
        val content = response.body()?.string() ?: "no content"

        // 全链路不打印日志
        if (request.url().toString() != "https://report.edu.qq.com/data_report/fulllink_report/client_ke") {
            LogUtils.li(TAG,
                    """
-----LoggingInterceptor----- :
 url:${request.url()}
 header:${request.headers()}
 body:${bodyToString(request.body())}
 time:${t2 - t1}ms
 response:$content
 """.trimIndent())
        }

        return response.newBuilder()
                .body(ResponseBody.create(mediaType, content))
                .build()
    }

    private fun getUrlWithoutParams(url: String): String {
        val split = url.split("?")
        return if (split.isNullOrEmpty()) "" else split.first()
    }

    private fun bodyToString(request: RequestBody?): String? {
        val buffer = Buffer()
        if (request != null) request.writeTo(buffer) else return ""
        return buffer.readUtf8()
    }

    private fun judgeNormalCookie(cookie: String): Boolean {
        if (cookie.contains("p_skey")) {
            val pskey = cookie.substringAfter("p_skey=").substringBefore(";")
            if (pskey.isNotEmpty()) {
                return true
            }
            return false
        }
        return true
    }

    companion object {
        private const val TAG = "HttpLogging"
        private const val REFERER = "https://ke.qq.com"
    }
}