package com.tencent.edu.kernel.http.retrofit

import com.tencent.edu.kernel.http.anotation.Cookie
import com.tencent.edu.kernel.http.anotation.Referer
import retrofit2.http.*
import java.lang.reflect.InvocationHandler
import java.lang.reflect.Method

/**
 * Retrofit代理
 *
 * @describe 主要用来解析自定义注解[Cookie]、[Referer]
 *
 * @author aidanyang
 * @date 2020-08-17
 */
class RetrofitProxy<T>(
        private val apiService: T,
        private val callback: (
                url: String,
                cookie: Boolean,
                refer: Boolean
        ) -> Unit
) : InvocationHandler {

    override fun invoke(proxy: Any, method: Method, args: Array<out Any>?): Any {
        val cookie: Cookie? = method.getAnnotation(Cookie::class.java)
        val referer: Referer? = method.getAnnotation(Referer::class.java)

        var url: String? = null

        for (annotation in method.declaredAnnotations ?: arrayOf()) {
            // 获取URL
            when (annotation) {
                is GET -> url = annotation.value
                is POST -> url = annotation.value
                is PUT -> url = annotation.value
                is DELETE -> url = annotation.value
            }

            if (!url.isNullOrBlank()) {
                break
            }
        }

        val paths = hashMapOf<String, String>()
        if (!args.isNullOrEmpty()) {
            // 空安全
            val annotations = method.parameterAnnotations
            for (i in annotations.indices) {
                for (element in annotations[i]) {
                    if (element is Url) {
                        //解析@Url
                        url = args[i] as String
                    } else if (element is Path) {
                        //解析@Path
                        val path = args[i] as String
                        paths[element.value] = path
                    }
                }
            }
        }

        if (!url.isNullOrBlank()) {
            // 拼装@Path
            for (path in paths) {
                url = url!!.replace("{${path.key}}", path.value)
            }

            callback.invoke(
                    url!!,
                    cookie != null,
                    referer != null
            )
        }

        return if (args == null) {
            method.invoke(apiService)
        } else {
            method.invoke(apiService, *args)
        }
    }
}