package com.tencent.edu.module.audiovideo.marketing.pop

import android.content.Context
import android.text.TextUtils
import android.util.AttributeSet
import android.view.LayoutInflater
import android.widget.RelativeLayout
import android.widget.TextView
import com.tencent.edu.R
import com.tencent.edu.common.utils.Tips
import com.tencent.edu.module.audiovideo.marketing.MarketingRequester
import com.tencent.edu.module.audiovideo.marketing.entity.MarketingCoupon

/**
 * @author ulexzhong. Created on 2020-10-12.
 */
internal class MarketingCouponPortraitView : RelativeLayout {
    private var mCoupon: MarketingCoupon? = null
    private var mAmountTxt: TextView? = null
    private var mDescText: TextView? = null
    private var mGetView: TextView? = null

    constructor(context: Context) : super(context) {
        initView(context)
    }

    constructor(context: Context, attrs: AttributeSet?) : super(context, attrs) {
        initView(context)
    }

    private fun initView(context: Context) {
        LayoutInflater.from(context).inflate(R.layout.layout_live_welfare_coupon_portrait, this)
        mAmountTxt = findViewById(R.id.welfare_discount_amount)
        mDescText = findViewById(R.id.welfare_discount_desc)
        mGetView = findViewById(R.id.welfare_discount_get)
    }

    fun setView(coupon: MarketingCoupon) {
        mCoupon = coupon
        mAmountTxt?.text = coupon.priceCentStr
        mDescText?.text = if (coupon.mMinCost > 0) coupon.desc1 else "现金券"
    }

    override fun setOnClickListener(listener: OnClickListener) {
        super.setOnClickListener {
            getCoupon()
            listener.onClick(it)
        }
    }

    private fun getCoupon() {
        if (mCoupon == null) {
            return
        }

        MarketingRequester.getCouponMarketing(mCoupon!!.mTid, mCoupon!!.mCid, mCoupon!!.mPrice
        ) { code, msg ->
            var msg = msg
            when (code) {
                0 -> {
                    mCoupon?.mState = MarketingCoupon.STATE_HAD_GOT
                    msg = "领取成功"
                }
                113132, 113133 -> {
                    mCoupon?.mState = MarketingCoupon.STATE_INVALID
                    msg = "优惠券已失效，敬请关注下轮发放"
                }
                113134 -> {
                    mCoupon?.mState = MarketingCoupon.STATE_HAD_GOT
                    msg = "已经领取过,不能再领啦"
                }
                113135 -> {
                    mCoupon?.mState = MarketingCoupon.STATE_WITHOUT
                    msg = "优惠券被抢光啦，敬请关注下轮发放"
                }
                else -> msg = "领取失败，请稍候重试"
            }
            if (!TextUtils.isEmpty(msg)) {
                Tips.showToast(msg)
            }
        }
    }
}