package com.tencent.edu.module.audiovideo.marketing.pop

import android.content.Context
import android.graphics.Paint
import android.text.SpannableString
import android.text.Spanned
import android.text.TextUtils
import android.text.style.AbsoluteSizeSpan
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.view.View.OnClickListener
import android.view.ViewTreeObserver
import android.widget.ImageView
import android.widget.RelativeLayout
import android.widget.TextView
import com.tencent.edu.R
import com.tencent.edu.common.imageloader.BitmapDisplayOptionMgr
import com.tencent.edu.common.imageloader.ImageLoaderProxy
import com.tencent.edu.common.utils.PixelUtil
import com.tencent.edu.module.audiovideo.marketing.entity.MarketingCourse
import com.tencent.edu.module.course.detail.CourseDetailActivity
import com.tencent.edu.module.course.detail.CourseDetailExtraInfo


/**
 * @author ulexzhong. Created on 2020-10-12.
 */
internal class MarketingCoursePortraitView : RelativeLayout {
    private var mCoverView: ImageView? = null
    private var mTitle: TextView? = null
    private var mPrice: TextView? = null
    private var mUnit: TextView? = null
    private var mLabel: TextView? = null
    private var mOriPriceTxt: TextView? = null

    constructor(context: Context) : super(context) {
        initView(context)
    }

    constructor(context: Context, attrs: AttributeSet?) : super(context, attrs) {
        initView(context)
    }

    private fun initView(context: Context) {
        LayoutInflater.from(context).inflate(R.layout.layout_live_welfare_course_portrait, this)
        mCoverView = findViewById(R.id.welfare_course_iv)
        mTitle = findViewById(R.id.welfare_course_title)
        mPrice = findViewById(R.id.welfare_course_price)
        mUnit = findViewById(R.id.welfare_course_unit)
        mLabel = findViewById(R.id.welfare_course_label)
        mOriPriceTxt = findViewById(R.id.welfare_course_price_ori)

        val observer = mTitle?.viewTreeObserver
        observer?.addOnGlobalLayoutListener(object : ViewTreeObserver.OnGlobalLayoutListener {
            override fun onGlobalLayout() {
                val obs = mTitle!!.viewTreeObserver
                obs.removeOnGlobalLayoutListener(this)
                //判断行数大于多少时改变
                if (mTitle!!.lineCount > 2) {
                    val lineEndIndex = mTitle!!.layout.getLineEnd(1)
                    val text = mTitle!!.text.subSequence(0, lineEndIndex - 3).toString() + "..."
                    mTitle!!.text = text;
                }
            }
        })
    }

    fun setView(course: MarketingCourse) {
        ImageLoaderProxy.displayImage(course.mCover, mCoverView,
                BitmapDisplayOptionMgr.getRoundOptions(R.drawable.default_cover, PixelUtil.dp2px(4f)))

        if (course.mOriPrice == 0) {//免费
            mOriPriceTxt?.visibility = View.GONE
            mPrice?.text = "免费"
            mUnit?.visibility = View.GONE
            mLabel?.visibility = View.GONE
        } else if (!TextUtils.isEmpty(course.mTypeName)) {//优惠课
            mUnit?.visibility = View.VISIBLE
            if(course.mFinialPrice==0){
                mPrice?.text = "0";
            }else {
                val strPrice = course.getConvertPrice(course.mFinialPrice)
                mPrice?.text = getPriceStr(strPrice)
            }
            mOriPriceTxt?.visibility = View.VISIBLE
            mOriPriceTxt?.text = String.format("¥%s", course.getConvertPrice(course.mOriPrice))
            mOriPriceTxt?.paint?.flags = Paint.STRIKE_THRU_TEXT_FLAG
            mOriPriceTxt?.invalidate()

            mLabel?.text = course.mTypeName
            mLabel?.visibility = View.VISIBLE
        } else {//普通付费课
            mUnit?.visibility = View.VISIBLE
            val strPrice = course.getConvertPrice(course.mOriPrice)
            mPrice?.text = getPriceStr(strPrice)
            mLabel?.visibility = View.GONE
            mOriPriceTxt?.visibility = View.GONE
        }

        mTitle?.text = course.mCourseName
    }

    private fun getPriceStr(strPrice: String): SpannableString {
        val split = strPrice.split(".")
        val msp = SpannableString(strPrice)
        if (split.size > 1) {
            msp.setSpan(AbsoluteSizeSpan(PixelUtil.sp2px(12f)), split.first().length, strPrice.length, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
        }
        return msp
    }

}