package com.tencent.edu.module.audiovideo.marketing.pop

import android.app.Activity
import android.content.Context
import android.graphics.drawable.ColorDrawable
import android.view.View
import android.view.ViewGroup
import com.tencent.edu.R
import com.tencent.edu.common.utils.PixelUtil
import com.tencent.edu.commonview.activity.BaseActivity
import com.tencent.edu.kernel.report.Report
import com.tencent.edu.module.audiovideo.marketing.entity.MarketingCoupon
import com.tencent.edu.module.audiovideo.marketing.entity.MarketingCourse
import com.tencent.edu.module.audiovideo.widget.ClassroomActivity
import com.tencent.edu.module.course.detail.CourseDetailActivity
import com.tencent.edu.module.course.detail.CourseDetailExtraInfo
import com.tencent.edu.module.report.AutoReportMgr
import com.tencent.edu.module.report.ReportConstant
import com.tencent.edu.module.utils.ExtraUtils
import com.tencent.edu.module.vodplayer.widget.speedratio.RelativePopupWindow
import java.util.*

/**
 * 直播间更多action弹窗
 * @author aidanyang
 * @date 2020-5-27
 */

class MarketingPortraitPop(context: Context, tartView: View) {

    companion object {
        private const val DELAY_DISMISS: Long = 3000
    }

    private var mCoupon: MarketingCoupon? = null
    private var mCourse: MarketingCourse? = null

    private var mPopupWnd: RelativePopupWindow? = null
    private var mTargetView: View? = null
    private var mMenuView: View? = null

    private var mCourseLayout: MarketingCoursePortraitView? = null
    private var mCouponLayout: MarketingCouponPortraitView? = null

    val isShowing: Boolean
        get() = mPopupWnd != null && mPopupWnd!!.isShowing

    init {
        init(context, tartView)
    }

    private fun init(context: Context, target: View) {
        mTargetView = target
        mMenuView = View.inflate(context, R.layout.pop_classroom_welfare_portrait, null)

        initViews()
        initListener()
        initWindow(context)
    }

    private fun initListener() {
        mCouponLayout?.setOnClickListener {
            dismissMenu()
            reportBeacon(isClick = true, isCourse = false)
        }
        mCourseLayout?.setOnClickListener {
            dismissMenu()
            if (mCourse == null) {
                return@setOnClickListener
            }

            reportBeacon(isClick = true, isCourse = true)
            val courseDetailExtraInfo = CourseDetailExtraInfo()
            courseDetailExtraInfo.courseId = mCourse?.mCid
            courseDetailExtraInfo.urlPage = if ((mMenuView?.context as ClassroomActivity).mIsFullScreenNow)
                "livecourse_horizen_play"
            else
                "livecourse_vertical_play"
            CourseDetailActivity.startActivity(courseDetailExtraInfo)
        }
    }

    private fun initWindow(context: Context) {
        mPopupWnd = RelativePopupWindow(
                mMenuView,
                ViewGroup.LayoutParams.WRAP_CONTENT,
                ViewGroup.LayoutParams.WRAP_CONTENT)
        mMenuView!!.isFocusableInTouchMode = true

        mPopupWnd!!.setBackgroundDrawable(
                ColorDrawable(context.resources.getColor(android.R.color.transparent)))
        mPopupWnd!!.animationStyle = R.style.reverse_pop_animation
        mPopupWnd!!.isOutsideTouchable = false
        mPopupWnd!!.isFocusable = true

        mPopupWnd!!.update()
    }

    private fun initViews() {
        mCourseLayout = mMenuView?.findViewById(R.id.welfare_course)
        mCouponLayout = mMenuView?.findViewById(R.id.welfare_coupon)
    }

    private fun setCoupon(coupon: MarketingCoupon) {
        mCourseLayout?.visibility = View.GONE
        mCouponLayout?.visibility = View.VISIBLE
        mCouponLayout?.setView(coupon)
    }

    private fun setCourse(course: MarketingCourse) {
        mCouponLayout?.visibility = View.GONE
        mCourseLayout?.visibility = View.VISIBLE
        mCourseLayout?.setView(course)
    }

    private fun showPopWnd() {
        // 把上一次显示的隐藏
        dismissMenu()

        if((mTargetView!!.context as BaseActivity).isActivityDestroyed
                || (mTargetView!!.context as Activity).isFinishing){
            return
        }

        //在控件上方显示    向上移动y轴是负数
        mPopupWnd?.showOnAnchor(mTargetView!!, RelativePopupWindow.VerticalPosition.ABOVE,
                RelativePopupWindow.HorizontalPosition.ALIGN_RIGHT,
                0, -PixelUtil.dp2px(20f), true)

        // 3s后隐藏
        mMenuView?.postDelayed(mDismissTask, DELAY_DISMISS)
    }

    fun popup() {
        mCoupon = (mMenuView?.context as ClassroomActivity).livePresenter?.marketingPresenter?.marketingCoupon
        if (mCoupon != null) {
            popup(mCoupon!!)
        } else {
            mCourse = (mMenuView?.context as ClassroomActivity).livePresenter?.marketingPresenter?.marketingCourse
            if (mCourse != null) {
                popup(mCourse!!)
            } else {
                return
            }
        }
    }

    fun popup(course: MarketingCourse) {
        mCourse = course
        setCourse(course)

        showPopWnd()

        reportBeacon(isClick = false, isCourse = true)
    }

    fun popup(coupon: MarketingCoupon) {
        mCoupon = coupon
        setCoupon(coupon)

        showPopWnd()

        reportBeacon(isClick = false, isCourse = false)
    }

    private fun reportBeacon(isClick: Boolean, isCourse: Boolean) {
        if (mMenuView == null) {
            return
        }

        val action = if (isClick) {
            ExtraUtils.EXTRA_REPORT_CLICK_ACTION
        } else {
            ExtraUtils.EXTRA_REPORT_EXPOSURE_ACTION
        }

        val ver1 = if (isCourse) "1" else "0"

        val reportExtraInfo = AutoReportMgr.getReportExtraInfo(mMenuView!!.context) ?: return
        reportExtraInfo.eventCode = action
        reportExtraInfo.module = "discount_pop"
        reportExtraInfo.page = "livecourse_vertical_play"
        val customReportDatas: MutableMap<String, String> = HashMap()
        reportExtraInfo.customDatas = customReportDatas
        customReportDatas[ReportConstant.DETAIL_VER1_KEY] = ver1

        if (isCourse) {
            customReportDatas[ReportConstant.DETAIL_VER3_KEY] = mCourse!!.mCid
        } else {
            customReportDatas[ReportConstant.DETAIL_VER2_KEY] = mCoupon!!.mCouponId
        }

        customReportDatas[ReportConstant.COURSEID] = (mMenuView!!.context as ClassroomActivity).courseId

        Report.autoReportData(reportExtraInfo)
    }

    fun dismissMenu() {
        mMenuView?.removeCallbacks(mDismissTask)
        if (isShowing) {
            mPopupWnd?.dismiss()
        }
    }

    private val mDismissTask = Runnable {
        dismissMenu()
    }
}
