package com.tencent.edu.module.audiovideo.widget

import android.app.Activity
import android.app.Dialog
import android.content.Context
import android.graphics.Bitmap
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.WindowManager
import com.nostra13.universalimageloader.core.DisplayImageOptions
import com.tencent.edu.R
import com.tencent.edu.common.imageloader.ImageLoaderProxy
import com.tencent.edu.common.utils.IntentUtil
import com.tencent.edu.common.utils.MiscUtils
import com.tencent.edu.common.utils.PixelUtil
import com.tencent.edu.framework.utils.ToastUtil
import com.tencent.edu.kernel.AppRunTime
import com.tencent.edu.kernel.KernelUtil
import com.tencent.edu.kernel.csc.CSCMgr
import com.tencent.edu.kernel.csc.config.CSC
import com.tencent.edu.kernel.login.misc.LoginStatus
import com.tencent.edu.module.audiovideo.entity.ContractTeacherInfo
import com.tencent.edu.module.audiovideo.report.EduAVActionReport
import com.tencent.edu.module.utils.ExtraUtils
import com.tencent.edu.wxapi.WXOpenApi
import kotlinx.android.synthetic.main.dialog_contract_teacher_portrait.*

/**
 * @author aidanyang
 * @date 2020-5-25
 */
class ContractTeacherDialog(
        val mContext: Context,
        private val orientation: Int
) : Dialog(
        mContext,
        if (orientation == PORTRAIT) {
            R.style.BottomDialog
        } else {
            R.style.EndDialog
        }
) {

    // 是否已准备好显示
    var isReady = false
    private var mListener: NotMoreNotifyListener? = null

    private var mConf: ContractTeacherInfo.SgConfBean? = null
    private var mCourseId = 0

    // ImageLoaderOptions
    private var mOptions: DisplayImageOptions? = DisplayImageOptions.Builder()
            .cacheInMemory(true).cacheOnDisk(true)
            .bitmapConfig(Bitmap.Config.RGB_565)
            .build()

    companion object {
        const val PORTRAIT = 1
        const val LANDSCAPE = 2

        // 点击联系老师后延时200ms，防止用户点击"联系老师"后dialog有回弹现象
        private const val DISMISS_DELAY: Long = 200

        private const val DIALOG_HEIGHT_PORTRAIT = 446f
        private const val DIALOG_WIDTH_LANDSCAPE = 240f

        private const val MINI_PROGRAM_NAME: String = "gh_3040c6e63ddf"

        private const val MINI_PROGRAM_PATH: String = "pages/contact-guide/contact-guide?cid=%d&uin=%s"
    }

    init {
        initWindow()
        initContent()
        initViews()
    }

    private fun initWindow() {
        // 隐藏状态栏不占位
        window?.apply {
            setFlags(
                    WindowManager.LayoutParams.FLAG_FULLSCREEN,
                    WindowManager.LayoutParams.FLAG_FULLSCREEN)
            decorView.setPadding(0, 0, 0, 0)
            attributes.apply {
                if (orientation == PORTRAIT) {
                    height = PixelUtil.dp2px(DIALOG_HEIGHT_PORTRAIT, context)
                    width = WindowManager.LayoutParams.MATCH_PARENT
                    window!!.setGravity(Gravity.BOTTOM)
                } else {
                    height = WindowManager.LayoutParams.MATCH_PARENT
                    width = PixelUtil.dp2px(DIALOG_WIDTH_LANDSCAPE, context)
                    window!!.setGravity(Gravity.END)
                }
            }
        }

        setCanceledOnTouchOutside(false)
    }

    private fun initContent() {
        val contentView = LayoutInflater.from(context).inflate(
                if (orientation == PORTRAIT) {
                    R.layout.dialog_contract_teacher_portrait
                } else {
                    R.layout.dialog_contract_teacher_landscape
                }, null
        )
        setContentView(contentView)
    }

    private fun initViews() {
        classroom_dialog_btn.setOnClickListener {
            classroom_dialog_btn.postDelayed({
                dismiss()
            }, DISMISS_DELAY)

            onBtnClick()
        }

        classroom_dialog_close.setOnClickListener {
            dismiss()
        }

        classroom_dialog_checkbox.setOnClickListener {
            EduAVActionReport.reportClick(mContext, ExtraUtils.EXTRA_REPORT_NOT_NOTICE, isVertical(), null)
        }
    }

    fun setNotMoreNotifyListener(listener: NotMoreNotifyListener) {
        mListener = listener
    }

    override fun dismiss() {
        // 勾选了"不再提醒"并且弹窗关闭时，请求接口
        if (classroom_dialog_checkbox.visibility == View.VISIBLE && classroom_dialog_checkbox.isChecked) {
            mListener?.onCheck()
        }
        super.dismiss()
    }

    /**
     * 点击联系老师按钮
     */
    private fun onBtnClick() {
        if (mConf == null) {
            return
        }

        when {
            mConf!!.isQQ -> {
                if (mConf!!.teacher_qq == null || mConf!!.teacher_qq.isEmpty()) {
                    return
                }

                if (MiscUtils.checkIsInstallQQ()) {
                    IntentUtil.openQQChat(mConf!!.teacher_qq[(0 until mConf!!.teacher_qq.size).random()].toString(), AppRunTime.getInstance().currentActivity)
                } else {
                    ToastUtil.showToast(R.string.action_sheet_fail)
                }
                EduAVActionReport.reportClick(mContext, ExtraUtils.EXTRA_REPORT_QQGROUP_JOIN, isVertical(), "qqfriend")
            }
            mConf!!.isGroup -> {
                if (mConf!!.qq_num == null || mConf!!.qq_num.isEmpty()) {
                    return
                }

                if (MiscUtils.checkIsInstallQQ()) {
                    // 随机选一个
                    IntentUtil.openQQGroup(mConf!!.qq_num[(0 until mConf!!.qq_num.size).random()].toString(), AppRunTime.getInstance().currentActivity)
                } else {
                    ToastUtil.showToast(R.string.action_sheet_fail)
                }
                EduAVActionReport.reportClick(mContext, ExtraUtils.EXTRA_REPORT_QQGROUP_JOIN, isVertical(), "qqgroup")
            }
            mConf!!.isWx -> {
                if (mCourseId == 0) {
                    return
                }

                if (MiscUtils.checkIsInstallWX()) {
                    WXOpenApi.launchWxMiniProgram(MINI_PROGRAM_NAME, String.format(
                            MINI_PROGRAM_PATH,
                            mCourseId,
                            KernelUtil.getAssetAccountId()
                    ), CSCMgr.getInstance().queryInt(CSC.Misc.ID, CSC.Misc.CONTRACT_TEACHER_MINI_PROGRAM_TYPE))
                } else {
                    ToastUtil.showToast(R.string.action_wx_fail)
                }
                EduAVActionReport.reportClick(mContext, ExtraUtils.EXTRA_REPORT_WXGROUP_JOIN, isVertical(), null)
            }
        }
    }

    /**
     * 根据类型展示弹窗
     */
    fun show(manual: Boolean) {
        if (!isReady) {
            return
        }

        if ((mContext is Activity) && !MiscUtils.isActivityValid(mContext)) {
            return
        }
        // 手动不显示checkbox
        classroom_dialog_checkbox.visibility = if (manual) {
            View.INVISIBLE
        } else {
            EduAVActionReport.reportExposure(mContext, ExtraUtils.EXTRA_REPORT_NOT_NOTICE, isVertical(), null)
            View.VISIBLE
        }

        when {
            mConf!!.isQQ -> {
                EduAVActionReport.reportExposure(mContext, ExtraUtils.EXTRA_REPORT_QQGROUP_POP, isVertical(), "qqfriend")
                // 手动的话一定是外部竖屏点击固定入口
                if (manual) {
                    EduAVActionReport.reportClick(mContext, ExtraUtils.EXTRA_REPORT_QQGROUP_ENTRY, true, "qqfriend")
                }
            }
            mConf!!.isWx -> {
                EduAVActionReport.reportExposure(mContext, ExtraUtils.EXTRA_REPORT_WXGROUP_POP, isVertical(), null)
                if (manual) {
                    EduAVActionReport.reportClick(mContext, ExtraUtils.EXTRA_REPORT_WXGROUP_ENTRY, true, null)
                }
            }
            mConf!!.isGroup -> {
                EduAVActionReport.reportExposure(mContext, ExtraUtils.EXTRA_REPORT_QQGROUP_POP, isVertical(), "qqgroup")
                if (manual) {
                    EduAVActionReport.reportClick(mContext, ExtraUtils.EXTRA_REPORT_QQGROUP_ENTRY, true, "qqfriend")
                }
            }
        }

        show()
    }

    fun isVertical(): Boolean {
        return orientation == PORTRAIT
    }

    fun setInfo(info: ContractTeacherInfo): ContractTeacherInfo.SgConfBean? {
        val qqFlag = info.sg_qq_conf != null && info.sg_qq_conf.state == "1"
        val wxFlag = info.sg_wx_conf != null && info.sg_wx_conf.state == "1"

        mCourseId = info.course_id

        // 如果只有一种配置存在，只使用
        mConf = when (LoginStatus.getAssetLoginType()) {
            LoginStatus.LoginType.LOGIN_QQ -> {
                when {
                    qqFlag -> setQQDialog(info.sg_qq_conf)
                    wxFlag -> setWxDialog(info.sg_wx_conf)
                    else -> null
                }
            }
            else -> {
                when {
                    wxFlag -> setWxDialog(info.sg_wx_conf)
                    qqFlag -> setQQDialog(info.sg_qq_conf)
                    else -> null
                }
            }
        }

        isReady = mConf != null

        return mConf
    }

    private fun setWxDialog(sgWxConf: ContractTeacherInfo.SgConfBean): ContractTeacherInfo.SgConfBean? {
        sgWxConf.mode = ContractTeacherInfo.MODE_WX
        return sgWxConf.apply {
            ImageLoaderProxy.displayImage(getRealUrl(url), classroom_dialog_iv, mOptions)
            classroom_dialog_title.text = title
            classroom_dialog_content.text = content
        }
    }

    private fun getRealUrl(url: String?): String {
        if (url.isNullOrBlank()) {
            return ""
        }

        return if (!url.startsWith("http:")) {
            "http:$url"
        } else {
            url
        }
    }

    private fun setQQDialog(sgQqConf: ContractTeacherInfo.SgConfBean): ContractTeacherInfo.SgConfBean? {
        sgQqConf.mode = ContractTeacherInfo.MODE_QQ
        return sgQqConf.apply {
            classroom_dialog_title.text = title
            classroom_dialog_content.text = content
            classroom_dialog_iv.setImageResource(if (sgQqConf.isQQ) {
                R.drawable.icon_contract_qq
            } else {
                R.drawable.icon_contract_group
            })
        }
    }

    interface NotMoreNotifyListener {
        fun onCheck()
    }

}
