package com.tencent.edu.module.audiovideo.widget

import android.content.Context
import android.graphics.Color
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import androidx.constraintlayout.widget.ConstraintLayout
import com.tencent.edu.R
import com.tencent.edu.common.utils.ClickTooFastProtectUtil
import com.tencent.edu.common.utils.LogUtils
import com.tencent.edu.common.utils.PixelUtil
import com.tencent.edu.common.utils.SimpleBeaconReportUtil
import com.tencent.edu.flutter.channel.FEBroadcastChannel
import com.tencent.edu.flutter.global.FlutterConstants
import com.tencent.edu.kernel.KernelUtil
import com.tencent.edu.module.audiovideo.entity.ContractTeacherInfo
import com.tencent.edu.module.audiovideo.report.EduAVActionReport
import com.tencent.edu.module.audiovideo.widget.ContractTeacherDialog.NotMoreNotifyListener
import com.tencent.edu.module.utils.ExtraUtils
import kotlinx.android.synthetic.main.layout_contract_teacher.view.*
import java.util.*

class ContractTeacherView : ConstraintLayout {

    constructor(context: Context) : super(context)

    constructor(context: Context, attrs: AttributeSet) : super(context, attrs)

    constructor(context: Context, attrs: AttributeSet, style: Int) : super(context, attrs, style)

    companion object {
        private const val TAG = "ContractTeacherView"
        private const val CLICK_INTERVAL = 2000L
    }

    private var mPresenter: ContractTeacherPresenter? = null
    private var mContractTeacherDialog: ContractTeacherDialog? = null
    private var isContractTeacherShow = false

    private var mCourseId = 0
    private var mTermId = 0
    private var mAgencyId = 0
    private var isShowRobot = 0

    // 是否获取入群通知（没有配置群，且show_robot的时候）
    private var isRemind = 0
    private var mPage: String? = null


    init {
        initView()
    }

    fun setPageName(pageName: String) {
        mPage = pageName;
    }

    fun setTaskPage() {
        val lp = layoutParams
        lp.height = PixelUtil.dp2px(44f)
        layoutParams = lp

        contract_teacher_qq.setBackgroundResource(R.drawable.icon_contract_teacher_2)
        val ilp = contract_teacher_qq.layoutParams
        ilp.height = PixelUtil.dp2px(16f)
        ilp.width = PixelUtil.dp2px(16f)
        contract_teacher_qq.layoutParams = ilp

        contract_teacher_divider.setBackgroundColor(Color.parseColor("#c8c7cc"))
        val dlp = contract_teacher_divider.layoutParams as LayoutParams
        dlp.height = PixelUtil.dp2px(0.5f)
        dlp.setMargins(0, 0, 0, 0)
        contract_teacher_divider.layoutParams = dlp
    }

    private fun initView() {
        LayoutInflater.from(context).inflate(R.layout.layout_contract_teacher, this)

        setOnClickListener {
            // 2s才能点击一次
            if (ClickTooFastProtectUtil.isClickTooFast(TAG, CLICK_INTERVAL)) {
                return@setOnClickListener
            }

            SimpleBeaconReportUtil.reportClickEvent(
                    context,
                    if (isRemind == 0) "join_xzsgroup_entry" else "join_xzsgroup_entry_nogroup",
                    ver2 = mCourseId.toString()
            )

            if (isShowRobot()) {
                showRobotDialog()
            } else {
                // 不是初始化，请求成功后会更新ui并弹出
                mPresenter?.fetchData(true)
            }
        }
    }

    fun setContractTeacherPresenter(presenter: ContractTeacherPresenter) {
        mPresenter = presenter
    }

    fun setInfo(info: ContractTeacherInfo?): Boolean {
        LogUtils.li(TAG, "setContractTeacher:info = $info")
        if (info == null) {
            isContractTeacherShow = false
            visibility = View.GONE
            mContractTeacherDialog?.isReady = false
            return false
        }

        mCourseId = info.course_id
        mTermId = info.term_id
        mAgencyId = info.agency_id
        isShowRobot = info.show_robot

        if (mContractTeacherDialog == null) {
            mContractTeacherDialog = ContractTeacherDialog(context, ContractTeacherDialog.PORTRAIT)
            mContractTeacherDialog!!.setNotMoreNotifyListener(object : NotMoreNotifyListener {
                override fun onCheck() {
                    if (context is ClassroomActivity) {
                        (context as ClassroomActivity).setNoMoreNotify()
                    }
                }
            })
        }

        val conf = mContractTeacherDialog!!.setInfo(info)
        isRemind = 0
        // 判断是否有联系老师
        if (conf == null) {
            if (info.show_robot == 1 && KernelUtil.isWXAsset()) {
                SimpleBeaconReportUtil.reportExposureEvent(context, "join_xzsgroup_entry_nogroup", mPage, ver2 = mCourseId.toString())
                isContractTeacherShow = true
                visibility = View.VISIBLE
                contract_teacher_btn.text = context.getString(R.string.contract_teacher_remind)
                contract_teacher_text.text = context.getString(R.string.contract_teacher_remind_brief)
                isRemind = 1
            } else {
                isContractTeacherShow = false
                visibility = View.GONE
            }
        } else {
            if (visibility != View.VISIBLE) {
                EduAVActionReport.reportExposure(context,
                        if (conf.isWx) ExtraUtils.EXTRA_REPORT_WXGROUP_ENTRY else ExtraUtils.EXTRA_REPORT_QQGROUP_ENTRY,
                        true,
                        if (conf.isWx) null else if (conf.isGroup) "qqgroup" else "qqfriend")

                SimpleBeaconReportUtil.reportExposureEvent(context, "join_xzsgroup_entry", mPage, ver2 = mCourseId.toString())
            }
            isContractTeacherShow = true
            visibility = View.VISIBLE
            contract_teacher_text.text = conf.brief

            if (!KernelUtil.isWXAsset())
                contract_teacher_btn.text = context.getString(R.string.contract_teacher)
            else
                contract_teacher_btn.text = context.getString(R.string.get_group_info)
        }
        return visibility == View.VISIBLE
    }

    fun showDialog(isManual: Boolean) {
        LogUtils.i(TAG, "showContractTeacher:isManual = $isManual")
        mContractTeacherDialog?.show(isManual)
    }

    fun isDialogShow(): Boolean {
        return isContractTeacherShow;
    }

    fun dismissDialog() {
        mContractTeacherDialog?.dismiss()
    }

    private fun showRobotDialog() {
        val info = HashMap<String, Any?>()
        info["cid"] = mCourseId
        info["tid"] = mTermId
        info["aid"] = mAgencyId
        info["isPkg"] = 0
        info["type"] = 0
        info["isRemind"] = isRemind
        info["page"] = mPage

        FEBroadcastChannel.broadcastEventWithInfo(FlutterConstants.SHOW_ROBOT_DIALOG, info)
    }

    private fun isShowRobot(): Boolean {
        return isShowRobot == 1
    }

}