package com.tencent.edu.module.audiovideo.widget

import android.content.Context
import android.graphics.drawable.ColorDrawable
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import com.tencent.edu.R
import com.tencent.edu.common.utils.PixelUtil
import com.tencent.edu.module.course.detail.operate.CourseCancelRemindRequester
import com.tencent.edu.module.course.task.entity.TaskCourseInfo
import com.tencent.edu.module.vodplayer.widget.speedratio.RelativePopupWindow
import kotlinx.android.synthetic.main.pop_classroom_more_portrait.view.*

/**
 * 直播间更多action弹窗
 * @author aidanyang
 * @date 2020-5-27
 */

class MoreActionLayout {

    private var mJumpToDetailLayout: View? = null
    private var mRemindMeLayout: View? = null
    private var mRemindMeIv: ImageView? = null
    private var mRemindMeTv: TextView? = null
    private var mPrivateChatLayout: View? = null
    private var mMenuView: View? = null

    private var mSysMenuPopupWnd: RelativePopupWindow? = null
    private var mMoreBtn: View? = null

    val isShowing: Boolean
        get() = mSysMenuPopupWnd != null && mSysMenuPopupWnd!!.isShowing

    fun init(context: Context, moreBtn: View) {
        mMenuView = View.inflate(context, R.layout.pop_classroom_more_portrait, null)
        mJumpToDetailLayout = mMenuView?.findViewById(R.id.jump_to_detail)
        mPrivateChatLayout = mMenuView?.findViewById(R.id.more_private_chat)
        mRemindMeLayout = mMenuView?.findViewById(R.id.more_remind)
        mRemindMeIv = mMenuView?.findViewById(R.id.more_remind_iv)
        mRemindMeTv = mMenuView?.findViewById(R.id.more_remind_tv)

        mMoreBtn = moreBtn
        mSysMenuPopupWnd = RelativePopupWindow(
                mMenuView,
                ViewGroup.LayoutParams.WRAP_CONTENT,
                ViewGroup.LayoutParams.WRAP_CONTENT)
        mMenuView!!.isFocusableInTouchMode = true

        mSysMenuPopupWnd!!.setBackgroundDrawable(
                ColorDrawable(context.resources.getColor(android.R.color.transparent)))
        mSysMenuPopupWnd!!.animationStyle = R.style.mypopwindow_anim_style
        mSysMenuPopupWnd!!.isOutsideTouchable = false
        mSysMenuPopupWnd!!.isFocusable = true

        mSysMenuPopupWnd!!.update()
    }

    fun popup() {
        //在控件上方显示    向上移动y轴是负数
        mSysMenuPopupWnd?.showOnAnchor(mMoreBtn!!, RelativePopupWindow.VerticalPosition.ABOVE,
                RelativePopupWindow.HorizontalPosition.ALIGN_LEFT,
                -PixelUtil.dp2px(10f), -PixelUtil.dp2px(8f), true)
    }

    fun dismissMenu() {
        mSysMenuPopupWnd?.dismiss()
    }

    fun updateRemindMe(taskCourseInfo: TaskCourseInfo?, needShowRemind: Boolean) {
        if (taskCourseInfo == null || !needShowRemind) {
            mRemindMeLayout?.visibility = View.GONE
            mSysMenuPopupWnd?.contentView?.more_divider?.visibility = View.GONE
            return
        } else if (mRemindMeLayout?.visibility != View.VISIBLE) {
            mRemindMeLayout?.visibility = View.VISIBLE
            mSysMenuPopupWnd?.contentView?.more_divider?.visibility = View.VISIBLE
        }

        if (taskCourseInfo.isRemind == CourseCancelRemindRequester.COURSE_REMIND) {
            updateRemindBtn(R.string.edu_course_remind_close, R.drawable.classroom_remind_close)
        } else {
            updateRemindBtn(R.string.edu_course_remind_open, R.drawable.classroom_remind_open)
        }
    }

    private fun updateRemindBtn(strRes: Int, drawableRes: Int) {
        mRemindMeTv?.setText(strRes)
        mRemindMeIv?.setBackgroundResource(drawableRes)
    }

    fun setOnJumpToDetailListener(privateJumpToDetailClickListener: View.OnClickListener) {
        mJumpToDetailLayout?.setOnClickListener(privateJumpToDetailClickListener)
    }

    fun setOnChatClickListener(privateChatClickListener: View.OnClickListener) {
        mPrivateChatLayout?.setOnClickListener(privateChatClickListener)
    }

    fun setOnRemindClickListener(privateRemindClickListener: View.OnClickListener) {
        mRemindMeLayout?.setOnClickListener(privateRemindClickListener)
    }

    fun showRedDot() {
        mPrivateChatLayout?.more_private_chat_reddot?.visibility = View.VISIBLE
    }

    fun hideRedDot() {
        mPrivateChatLayout?.more_private_chat_reddot?.visibility = View.GONE
    }

    fun isRedDotShow(): Boolean {
        return mPrivateChatLayout?.more_private_chat_reddot?.visibility == View.VISIBLE
    }

}
