package com.tencent.edu.module.audiovideo.widget

import android.view.LayoutInflater
import android.view.View
import android.view.View.GONE
import android.view.View.VISIBLE
import android.view.ViewGroup
import android.widget.CheckBox
import android.widget.FrameLayout
import android.widget.RelativeLayout
import com.androidkun.xtablayout.XTabLayout
import com.tencent.edu.R
import com.tencent.edu.common.core.ThreadMgr
import com.tencent.edu.common.utils.LogUtils
import com.tencent.edu.common.utils.MiscUtils
import com.tencent.edu.common.utils.PixelUtil
import com.tencent.edu.common.utils.TabLayoutUtil
import com.tencent.edu.commonview.widget.TeacherAnswerButton
import com.tencent.edu.module.audiovideo.report.EduAVActionReport
import com.tencent.edu.module.audiovideo.session.RequestInfo
import com.tencent.edu.module.campaign.LivePageCampaignPresenter
import com.tencent.edu.module.course.util.IntroduceWhiteListRequester
import com.tencent.edu.module.coursemsg.misc.UtilMsg
import com.tencent.edu.module.coursemsg.msg.BaseMessage
import com.tencent.edu.module.coursemsg.msg.ChatMessage
import com.tencent.edu.module.coursemsg.msg.MsgItemDef
import com.tencent.edu.module.utils.ExtraUtils
import com.tencent.edu.module.webapi.CourseWebView
import com.tencent.edu.utils.EduLog
import kotlinx.android.synthetic.main.layout_classroom_portrait.view.*
import kotlinx.android.synthetic.main.layout_classroom_tabview.view.*

/**
 *
 * @author aidanyang
 * @date 2020-04-29
 */
class RecruitClassroomPortrait(activity: ClassroomActivity) : ClassroomPortrait(activity) {

    companion object {

        private const val TAG = "RecruitClassroomPortrait"

        const val TAB_INTRO = 0
        const val TAB_EXPLAIN = 1
        const val TAB_DISCUSS = 2

        // 介绍页URL
        private const val INTRO_WEB_URL = "https://m.ke.qq.com/m-core/chatIntro.html?term_id=%s"
    }

    // 5.1 高招会 标识
    var isRecruitCourse = false

    // 是否已经初始化讨论区了
    private var isInitDiscussArea = false

    // 自己是否上台
    private var isMyselfOnTalk = false

    private var mOldInflatedViewStub: View? = null

    private var mPortraitQuestionCheckBox: CheckBox? = null
    private var mTabLayout: XTabLayout? = null
    private var mViewPager: androidx.viewpager.widget.ViewPager? = null
    private var mStudentRollContainerRootView: FrameLayout? = null

    // 介绍页WebView
    private var mIntroWebView: CourseWebView? = null

    // 讲解区视图
    internal var mExplainView: ClassRoomExplainView? = null

    private var mPortraitSeeAnswerBtn: TeacherAnswerButton? = null

    // 在线人数，主要用作旧页面切换到新页面处理
    private var mOnlineNumber: Int = 0

    // 老师回复的消息
    private var mSeeAnswerMsg: ChatMessage? = null

    /**
     * 5.1 高招会 必定显示更名按钮
     */
    override fun initBottomBar() {
        super.initBottomBar()

        if (isRecruitCourse) {
            hideMoreBtn()
            showChangeNameView()
        }
    }

    /**
     * 5.1 高招会 提问按钮及其监控
     */
    override fun initPortraitInputLayout() {
        super.initPortraitInputLayout()
        mPortraitQuestionCheckBox = mPortraitInputLinearLayout.findViewById(R.id.input_question)
        mPortraitQuestionCheckBox?.setOnClickListener {
            EduAVActionReport.reportExposure(mContext, ExtraUtils.EXTRA_REPORT_QUESTION_CHAT, true, null)
        }
    }

    /**
     * 5.1 高招会 发送消息，提问checkBox
     */
    override fun sendMsg() {
        val succ = mPresenter.sendMsg(mPortraitInputMsgEdit, mPortraitQuestionCheckBox)
        if (succ) {
            smoothToBottom()
        }
    }

    /**
     * 5.1 高招会 初始化聊天列表
     */
    override fun initPortraitChatLayout() {
        // 5.1 高招会 设置监听器监听课程是否在白名单内
        // 如果初始化时已有结果，直接使用
        isRecruitCourse = IntroduceWhiteListRequester.getInstance().setListener { isWithinWhiteList, _ ->
            ThreadMgr.postToUIThread {
                // 如果原来是旧页面才切换
                if (!isRecruitCourse && isWithinWhiteList) {
                    isRecruitCourse = true
                    change2RecruitCourse()
                }
            }
        }

        if (isRecruitCourse) {
            initRecruitCourse()
        } else {
            super.initPortraitChatLayout()
            isInitDiscussArea = true
        }
    }

    private fun initRecruitCourse() {
        LogUtils.i(TAG, "5.1高招会，初始化新界面")
        initCampaign()
        initTabLayout()
    }

    /**
     * 初始化挂件
     */
    private fun initCampaign() {
        mCampaignPresenter?.load(IntroduceWhiteListRequester.getInstance().extraData)
    }

    /**
     * 初始化标签栏
     */
    private fun initTabLayout() {
        // 提问按钮可见
        mPortraitQuestionCheckBox?.visibility = VISIBLE

        val tabViewStup = mPortraitChatMsgContainer.classroom_new_stup
        //如果旧布局已经初始化，则保存旧布局用来给讨论区复用
        if (isInitDiscussArea) {
            mOldInflatedViewStub = mInflatedViewStub
        }
        mInflatedViewStub = tabViewStup.inflate()

        mRecruitVideoBottomPortraitLayout = mInflatedViewStub.video_bottom_layout

        mTabLayout = mInflatedViewStub.chat_msg_tab_layout

        mViewPager = mInflatedViewStub.chat_msg_view_pager
        // 懒加载,防止第三个页面被销毁
        mViewPager?.offscreenPageLimit = 3

        // 介绍区
        initIntroArea(mViewPager!!)

        // 讲解区
        initExplainArea(mViewPager!!)

        // 讨论区
        initDiscussArea(mViewPager!!)

        //初始化点名区域
        mStudentRollContainerRootView = mInflatedViewStub.student_roll_call_container
        initStudentRollCallArea(mStudentRollContainerRootView!!)

        TabLayoutUtil.bindViewPager(mTabLayout!!, mViewPager!!, object : XTabLayout.OnTabSelectedListener {
            override fun onTabSelected(tab: XTabLayout.Tab) {
                resetPortraitInputZone()

                val module = when (mViewPager?.currentItem) {
                    TAB_INTRO -> {
                        if (isMyselfOnTalk && mRecruitVideoBottomPortraitLayout?.visibility != VISIBLE) {
                            mRecruitVideoBottomPortraitLayout?.visibility = VISIBLE
                        }
                        ExtraUtils.EXTRA_REPORT_FASTLIVE_INTRODUCTION
                    }
                    TAB_EXPLAIN -> {
                        if (isMyselfOnTalk && mRecruitVideoBottomPortraitLayout?.visibility != VISIBLE) {
                            mRecruitVideoBottomPortraitLayout?.visibility = VISIBLE
                        }
                        mExplainView?.requestInfo = mPresenter.requestInfo
                        if (mPresenter.requestInfo != null) {
                            ClassroomUtils.fetchExplainList(mPresenter.requestInfo.mTermId, 50, "", mExplainView?.onFetchExplainDataListener, true)
                        }
                        ExtraUtils.EXTRA_REPORT_SELECTED_CHAT
                    }
                    TAB_DISCUSS -> {
                        if (mRecruitVideoBottomPortraitLayout?.visibility != GONE) {
                            mRecruitVideoBottomPortraitLayout?.visibility = GONE
                        }
                        ExtraUtils.EXTRA_REPORT_CHATROOM
                    }
                    else -> ExtraUtils.EXTRA_REPORT_SELECTED_CHAT
                }
                EduAVActionReport.reportExposure(mContext, module, true, null)

                setTabSelected(tab, true)
            }

            override fun onTabUnselected(tab: XTabLayout.Tab) {
                setTabSelected(tab, false)
            }

            override fun onTabReselected(tab: XTabLayout.Tab) {}
        })

        // 默认选中讲解区
        mViewPager?.currentItem = TAB_EXPLAIN
    }

    override fun initViewStubInflatedView(parent: View?) {
        super.initViewStubInflatedView(parent)
        // 老师回复
        mPortraitSeeAnswerBtn = parent?.findViewById(R.id.chatmsg_teacher_answer)
        initSeeAnswerBtn()
    }

    /**
     * 初始化老师回复按钮
     */
    private fun initSeeAnswerBtn() {
        mPortraitSeeAnswerBtn?.setOnClickListener(View.OnClickListener {
            if (mSeeAnswerMsg == null) {
                return@OnClickListener
            }

            val position = mPresenter.mChatAdapter.findMsgPos(mSeeAnswerMsg) + 1
            if (position != 0) {
                // 定位到chatlistview中间位置
                mPortraitChatListView.smoothScrollToPositionFromTop(position, mPortraitChatListView.height / 2)
            }

            mPortraitSeeAnswerBtn?.hide()
            mSeeAnswerMsg = null
        })
    }

    /**
     * 设置自定义tab被选中
     */
    private fun setTabSelected(tab: XTabLayout.Tab, isSelected: Boolean) {
        val redDotTab = tab.customView as RedDotTab? ?: return

        // 选中一个tab之后，相应红点消失
        if (isSelected) {
            redDotTab.showRedDot(false)
        }

        redDotTab.setTabSelected(isSelected)
    }

    /**
     * 初始化介绍区
     */
    private fun initIntroArea(viewPager: androidx.viewpager.widget.ViewPager) {
        createRedDotTab(R.string.classroom_tab_intro)

        mIntroWebView = CourseWebView(mContext)
        loadIntroWebViewUrl()
        viewPager.addView(mIntroWebView)
    }

    /**
     * 初始化讲解区
     */
    private fun initExplainArea(viewPager: androidx.viewpager.widget.ViewPager) {
        createRedDotTab(R.string.classroom_tab_explain)
        mExplainView = ClassRoomExplainView(mContext)
        mExplainView?.setClassRoomHolder(mPresenter.mClassroomInfoHolder)
        viewPager.addView(mExplainView)
    }

    fun addMsgToExplainView(msgPack: MsgItemDef.MsgPack) {
        if (mExplainView != null) {
            val msg = UtilMsg.transMsgPackToBaseMessage(msgPack.mRole, msgPack, null)
                    ?: return

            mExplainView?.addMsg(msg)
        }
    }

    /**
     * 初始化讨论区
     */
    private fun initDiscussArea(viewPager: androidx.viewpager.widget.ViewPager) {
        createRedDotTab(R.string.classroom_tab_discuss)

        if (isInitDiscussArea) {
            EduLog.d(TAG, "initDiscussArea ：复用旧布局")
            //复用正常布局
            if (mOldInflatedViewStub != null) {
                mOldInflatedViewStub!!.visibility = VISIBLE
                (mOldInflatedViewStub!!.parent as? ViewGroup)?.removeView(mOldInflatedViewStub)
                viewPager.addView(mOldInflatedViewStub as RelativeLayout)
            } else {
                EduLog.d(TAG, "mOldInflatedViewStub is null")
            }
        } else {
            val linearLayout = LayoutInflater.from(mContext).inflate(R.layout.layout_classroom_listview, null) as RelativeLayout
            initViewStubInflatedView(linearLayout)
            viewPager.addView(linearLayout)
        }
    }

    /**
     * 初始化点名区域
     * 将点名区域从正常直播间布局移到tap布局上面来
     */
    private fun initStudentRollCallArea(rootView: View) {
        EduLog.d(TAG, "高招会：初始化点名区域")
        ((mPortraitStudentRollCallContainer?.parent) as? ViewGroup)?.removeView(mPortraitStudentRollCallContainer)
        if (mPortraitStudentRollCallContainer != null) {
            (rootView as? ViewGroup)?.addView(mPortraitStudentRollCallContainer, ViewGroup.LayoutParams.MATCH_PARENT, PixelUtil.dp2px(96F));
        }
    }

    //是否展示点名区
    override fun setStudentRollCallVisible(isShow: Boolean) {
        if (isRecruitCourse) {
            if (isShow) {
                mStudentRollContainerRootView?.visibility = VISIBLE
                mPortraitStudentRollCallContainer?.visibility = VISIBLE
            } else {
                mStudentRollContainerRootView?.visibility = GONE
                mPortraitStudentRollCallContainer?.visibility = GONE
            }
            return
        }
        super.setStudentRollCallVisible(isShow)
    }

    //自己是否被点名或者成功举手
    override fun setRollCallShow(isShow: Boolean) {
        isMyselfOnTalk = isShow
        if (isRecruitCourse) {
            if (isShow) {
                //如果当前的item不是讨论区，才显示视频连麦底部控制按钮
                if (mViewPager?.currentItem != TAB_DISCUSS) mRecruitVideoBottomPortraitLayout?.visibility = VISIBLE
                mRecruitVideoBottomPortraitLayout?.setMikeSelect(false)
                mRecruitVideoBottomPortraitLayout?.setCameraSelect(false)
            } else {
                mRecruitVideoBottomPortraitLayout?.visibility = GONE
            }
        }
        super.setRollCallShow(isShow)
    }

    /**
     * 生成红点Tab
     */
    private fun createRedDotTab(resId: Int) {
        createRedDotTab(MiscUtils.getString(resId))
    }

    /**
     * 生成红点Tab
     */
    private fun createRedDotTab(str: String) {
        if (mTabLayout == null) {
            return
        }

        val tab = mTabLayout!!.newTab()
        val redDotTab = RedDotTab(mContext)
        redDotTab.setText(str)
        // indicator长度
        tab.text = str
        tab.customView = redDotTab

        mTabLayout!!.addTab(tab)
    }

    override fun updateOnlineNumberTextView(num: Int) {
        mOnlineNumber = num

        if (isRecruitCourse) {
            // 防止在切换新旧页面过程中更新人数
            if (mTabLayout == null) {
                LogUtils.e(TAG, "直播页新TabLayout未初始化")
                return
            }
            // 找到讨论区tab
            val tab = mTabLayout?.getTabAt(TAB_DISCUSS)
            if (tab == null) {
                LogUtils.e(TAG, "直播页新TabLayout找不到讨论区Tab")
                return
            }

            val customView = tab.customView as RedDotTab?
            if (customView == null) {
                LogUtils.e(TAG, "讨论区Tab的customView == null")
                return
            }

            val text = String.format("%s(%d人)", MiscUtils.getString(R.string.classroom_tab_discuss), num)

            customView.setText(text)
            // 动态indicator
            tab.text = text
        } else {
            super.updateOnlineNumberTextView(num)
        }
    }

    override fun updateTeacherOnly(isSeeTeacherOnly: Boolean) {
        super.updateTeacherOnly(isSeeTeacherOnly)

        // 5.1 高招会 切换模式后，检查 老师回复消息 是否还在相应缓存内
        if (isRecruitCourse) {
            if (mPortraitSeeAnswerBtn == null || mPortraitSeeAnswerBtn?.visibility != VISIBLE || mSeeAnswerMsg == null) {
                return
            }

            val position = mPresenter.mChatAdapter.findMsgPos(mSeeAnswerMsg)
            if (position == -1) {
                mPortraitSeeAnswerBtn?.hide()
            }
        }
    }

    /**
     * 5.1 高招会 加载介绍页URL
     */
    private fun loadIntroWebViewUrl() {
        if (mRequestInfo != null) {
            mIntroWebView?.loadUrl(String.format(INTRO_WEB_URL, mRequestInfo.mTermId))
        }
    }

    override fun setRequestInfo(requestInfo: RequestInfo) {
        super.setRequestInfo(requestInfo)


        // 5.1 高招会 如果还不是白名单课程，检查，内部会有防重复检查逻辑
        if (!isRecruitCourse) {
            //检查是否白名单列表内
            IntroduceWhiteListRequester.getInstance().checkWithinWhiteList(requestInfo.mTermId)
        }

        // 相应初始化
        loadIntroWebViewUrl()

        // 刷新讲解区
        refreshExpandViewData()
    }

    override fun onInputLayoutVisible() {
        super.onInputLayoutVisible()
        // 5.1 高交会 提问按钮曝光
        if (isRecruitCourse && mPortraitInputLinearLayout.visibility != VISIBLE) {
            EduAVActionReport.reportExposure(mContext, ExtraUtils.EXTRA_REPORT_QUESTION_CHAT, true, null)
        }
    }

    override fun updateOnlineNumberTextViewVisibility(show: Boolean) {
        // 5.1 高招会 在线人数不显示
        super.updateOnlineNumberTextViewVisibility(show && !isRecruitCourse)
    }

    /**
     * 5.1 高招会 TabLayout红点消息
     *
     * @param tabIndex {[.TAB_INTRO]、[.TAB_EXPLAIN]、[.TAB_DISCUSS]}
     */
    fun notifyRedDotTab(tabIndex: Int) {
        val position = mTabLayout?.selectedTabPosition
        if (tabIndex == position) {
            return
        }

        val tab = mTabLayout?.getTabAt(tabIndex) ?: return

        val customView = tab.customView as RedDotTab? ?: return

        customView.showRedDot(true)
    }

    /**
     * 老师回答了用户信息通知
     */
    fun notifyTeacherAnswer(chatMessage: ChatMessage) {
        mSeeAnswerMsg = chatMessage
        mPortraitSeeAnswerBtn?.show()
    }

    /**
     * 5.1 高招会 老师回复消息冲出讨论区，判断是否还显示 老师回复了你 按钮
     *
     * @param msg 被删除的消息
     */
    fun teacherAnswerRemoved(msg: BaseMessage) {
        if (mSeeAnswerMsg != msg || mPortraitSeeAnswerBtn?.visibility != VISIBLE) {
            return
        }

        mSeeAnswerMsg = null
        mPortraitSeeAnswerBtn?.hide()
    }

    /**
     * 5.1 高招会 切换旧界面到新界面
     */
    private fun change2RecruitCourse() {
        LogUtils.i(TAG, "5.1 高招会，从旧界面切换到新界面")

        if (mPipTeachLayout != null) {
            mPipTeachLayout.removeView()
            mPipTeachLayout.visibility = GONE
        } else {
            LogUtils.i(TAG, "mPipTeachLayout == null")
        }

        if (mInflatedViewStub != null) {
            mInflatedViewStub.visibility = GONE
        } else {
            LogUtils.i(TAG, "mInflatedViewStub == null")
        }

        if (mPortraitOnlineNumberTextView != null) {
            mPortraitOnlineNumberTextView.visibility = GONE
        } else {
            LogUtils.i(TAG, "mPortraitOnlineNumberTextView == null")
        }

        if (mPortraitChatMsgContainer != null) {
            if (mInflatedViewStub != null) {
                initRecruitCourse()
            }
        } else {
            LogUtils.i(TAG, "mPortraitChatMsgContainer == null")
        }

        // 更新讨论区人数
        updateOnlineNumberTextView(mOnlineNumber)
    }

    fun refreshExpandViewData() {
        if (mExplainView != null) {
            // 这里塞如termId
            mExplainView!!.requestInfo = mRequestInfo
            // 初始化讲解区请求
            ClassroomUtils.fetchExplainList(mRequestInfo.mTermId, 50, "", mExplainView!!.onFetchExplainDataListener, true)
        }
    }

    fun notifyExplainDataSetChanged() {
        mExplainView?.notifyDataSetChanged()
    }

    fun onDestroy() {
        mCampaignPresenter?.onDestroy()
    }

}