package com.tencent.edu.module.audiovideo.widget

import android.app.Activity
import com.tencent.edu.kernel.login.mgr.AccountMgr
import com.tencent.edu.kernel.push.EduPushManager
import com.tencent.edu.module.audiovideo.session.RequestInfo
import com.tencent.edu.module.course.detail.util.CourseDetailUtil
import com.tencent.edu.module.coursemsg.misc.UtilMsg
import com.tencent.edu.module.coursemsg.msg.ChatAdapter
import com.tencent.edu.module.coursemsg.msg.ChatMessage
import com.tencent.edu.module.coursemsg.msg.MsgItemDef

/**
 * 5.1 高招会 LivePresenter
 * @author aidanyang
 * @date 2020-04-29
 */
class RecruitLivePresenter(context: Activity, live: LiveBaseCourseContract.ILiveView, liveBaseView: ILiveBaseView) : LivePresenter(context, live, liveBaseView) {

    override fun setRequestInfo(info: RequestInfo) {
        mRequestInfo = info
        EduPushManager.getInstance().setFetchInfo("edu", mRequestInfo.mTermId)
        // 不是极速课强制刷新adapter，目的是显示 点击咨询
        if (!CourseDetailUtil.isJsCourse(info.mTermBitFlag)) {
            mChatAdapter.notifyDataSetChanged()
            getClassroomPortrait().notifyExplainDataSetChanged()
        }
    }

    override fun initAdapt() {
        super.initAdapt()
        // 聊天消息删除回调
        mChatAdapter.setMsgRemoveListener(ChatAdapter.MsgRemoveListener { msg ->
            // 5.1高招会 主要用来控制 "老师回复了你" 按钮的显示
            if (!getClassroomPortrait().isRecruitCourse) {
                return@MsgRemoveListener
            }

            if (msg !is ChatMessage) {
                return@MsgRemoveListener
            }

            if (!msg.isAnswerMe) {
                return@MsgRemoveListener
            }

            getClassroomPortrait().teacherAnswerRemoved(msg)
        })
    }

    override fun showTeacherInfo() {
        // 5.1 高招会 如果是白名单课，不显示老师信息
        if (getClassroomPortrait().isRecruitCourse) {
            return
        }

        super.showTeacherInfo()
    }

    override fun handleMsgCome(msg: MsgItemDef.MsgPack) {
        // 用户uin
        val accountId = AccountMgr.getInstance().currentAccountData.assetAccountId

        // 是否发送者，且是普通消息，只更新seq
        val isSender = msg.mFromUin.toString() == accountId
        val isNormalMsg = msg.mExplainType == MsgItemDef.MsgPack.MSG_TYPE_NORMAL
        if (isSender && isNormalMsg) {
            mChatAdapter.updateSeq(msg.mClientSeq, msg.mSeq)
            return
        }

        // 5.1 高招会 是否回复我
        val isAnswerMe = msg.mReplyUin != null && msg.mReplyUin.contains(accountId)
        // 5.1 高招会 讨论区tab红点
        // 如果是纯讲解操作，则不进行余下聊天消息操作
        if (getClassroomPortrait().isRecruitCourse && handleNewFlagMsg(msg)) {
            return
        }

        // 获取角色
        val baseMessage = UtilMsg.transMsgPackToBaseMessage(msg.mRole, msg, isAnswerMe) ?: return

        // 添加到聊天列表
        mChatAdapter.addMsg(baseMessage)

        if (isAnswerMe) {
            getClassroomPortrait().notifyTeacherAnswer(baseMessage as ChatMessage)
        }
    }

    /**
     * 5.1 高招会 处理5.1高招会新增消息
     */
    private fun handleNewFlagMsg(msg: MsgItemDef.MsgPack): Boolean {
        var isOnlyExplain = false

        when (msg.mExplainType) {
            // 单独新增讲解
            MsgItemDef.MsgPack.EXPLAIN_TYPE_EXPLAIN -> {
                isOnlyExplain = true
                // 提示红点消息
                getClassroomPortrait().notifyRedDotTab(RecruitClassroomPortrait.TAB_EXPLAIN)
                // 添加一条讲解消息
                getClassroomPortrait().addMsgToExplainView(msg)
            }
            // 同时是讲解和聊天
            MsgItemDef.MsgPack.EXPLAIN_TYPE_BOTH -> {
                getClassroomPortrait().notifyRedDotTab(RecruitClassroomPortrait.TAB_EXPLAIN)
                getClassroomPortrait().addMsgToExplainView(msg)
            }
            // 删除讲解
            MsgItemDef.MsgPack.EXPLAIN_TYPE_DELETE -> {
                isOnlyExplain = true

                getClassroomPortrait().refreshExpandViewData()
            }
        }

        if (isOnlyExplain) {
            return true
        }

        when (msg.mMsgType) {
            MsgItemDef.MsgPack.MSG_TYPE_QUESTION,
                // 老师回答
            MsgItemDef.MsgPack.MSG_TYPE_ANSWER,
                // 普通消息
            MsgItemDef.MsgPack.MSG_TYPE_NORMAL ->
                // 提示红点消息
                getClassroomPortrait().notifyRedDotTab(RecruitClassroomPortrait.TAB_DISCUSS)
        }

        return false
    }

    private fun getClassroomPortrait(): RecruitClassroomPortrait {
        return mClassroomPortrait as RecruitClassroomPortrait
    }

}