package com.tencent.edu.module.course.detail.util

import android.app.Activity
import android.net.Uri
import android.view.View
import com.tencent.edu.R
import com.tencent.edu.common.utils.GifImageUtil
import com.tencent.edu.common.utils.LogUtils
import com.tencent.edu.common.utils.MiscUtils
import com.tencent.edu.common.utils.SharedPrefsUtil
import com.tencent.edu.commonview.dialogplus.DialogPlus
import com.tencent.edu.commonview.dialogplus.ViewHolder
import com.tencent.edu.framework.utils.ToastUtil
import com.tencent.edu.kernel.KernelConfig.DebugConfig
import com.tencent.edu.kernel.KernelUtil
import com.tencent.edu.kernel.report.Report
import com.tencent.edu.module.course.detail.CourseDetailActivity
import com.tencent.edu.module.report.ReportConstant
import com.tencent.edu.module.setting.DebugAppActivity
import com.tencent.edu.module.utils.ExtraUtils
import com.tencent.edu.wxapi.WXOpenApi
import kotlinx.android.synthetic.main.dialog_course_helper.view.*
import java.io.File
import java.io.FileInputStream
import java.io.FileNotFoundException
import java.io.IOException
import java.util.*

/**
 * @author aidanyang
 * @date 2020-5-25
 */
class CourseHelperDialog(
        val activity: Activity
) {
    private var mInfo: ComponentInfo? = null
    private var mOnDismissListener: (() -> Unit)? = null

    private var mView: View = View.inflate(activity, R.layout.dialog_course_helper, null)
    private var mDialog: DialogPlus

    companion object {
        private const val DISMISS_DELAY: Long = 200

        private const val MINI_PROGRAM_NAME: String = "gh_3040c6e63ddf"

        var COURSE_HELPER_PROGRAM_TYPE: Int? = null
    }

    init {
        LogUtils.le("CourseHelperDialog", "initPushEvent")
        mDialog = DialogPlus.Builder(activity)
                .setContentHolder(ViewHolder(mView))
                .setGravity(DialogPlus.Gravity.BOTTOM)
                .setContentBackgroundColorResourceId(R.color.transparent)
                .setCancelable(false)
                .create()
        initViews()
    }

    private fun initViews() {
        mView.course_apply_close?.setOnClickListener {
            dismiss()
        }
    }

    fun dismiss() {
        LogUtils.le("CourseHelperDialog", "dismiss")

        mOnDismissListener?.invoke()
        mView.course_apply_gif.stop()
        mView.course_apply_gif.clear()
        mView.course_apply_gif.destroy()
        mDialog.dismiss()
    }

    fun show(): Boolean {
        LogUtils.le("CourseHelperDialog", "show")

        if (!MiscUtils.isActivityValid(activity) || mDialog.isShowing) {
            return false
        }

        if (mInfo != null)
            report(mInfo!!.isHelper, mInfo!!.componentId, false)
        mDialog.show()
        return true
    }

    fun setInfo(info: ComponentInfo, isHelper: Boolean, onDismissListener: () -> Unit): CourseHelperDialog {
        this.mInfo = info
        mOnDismissListener = onDismissListener

        mView.course_apply_btn.setOnClickListener {
            if (mInfo != null)
                report(mInfo!!.isHelper, mInfo!!.componentId, true)

            if (MiscUtils.checkIsInstallWX()) {
                WXOpenApi.launchWxMiniProgram(
                        MINI_PROGRAM_NAME,
                        getRealPath(info),
                        getMiniProgramType()
                )
            } else {
                ToastUtil.showToast(R.string.action_wx_fail)
            }

            mView.course_apply_btn.postDelayed({
                dismiss()
            }, DISMISS_DELAY)
        }

        mView.course_apply_btn.text = info.button_wording
                ?: activity.getString(R.string.course_helper_btn)
        mView.course_apply_content.text = info.discrible
                ?: activity.getString(R.string.course_helper_content)
        GifImageUtil.downloadGif(activity, info.gif_url, object : GifImageUtil.OnGifReadyListener {
            override fun onGifReady(gitFile: File?) {
                LogUtils.le("CourseHelperDialog", "gif ready")

                if (gitFile == null || !gitFile.exists()) {
                    onGifError()
                    return
                }

                if (isGifFile(gitFile)) {
                    mView.course_apply_gif.initGif(gitFile)
                    mView.course_apply_gif.start()
                    mView.course_apply_gif.visibility = View.VISIBLE
                    mView.course_apply_iv.visibility = View.GONE
                } else {
                    mView.course_apply_iv.setImageURI(Uri.fromFile(gitFile))
                    mView.course_apply_gif.visibility = View.INVISIBLE
                    mView.course_apply_iv.visibility = View.VISIBLE
                }
            }

            override fun onGifError() {
                LogUtils.le("CourseHelperDialog", "gif error")

                // 如果下不到图片，则使用默认图
                mView.course_apply_gif.initGif(
                        if (isHelper)
                            R.raw.course_helper_default
                        else
                            R.raw.wx_account_default
                )
                mView.course_apply_gif.start()
            }

        })

        return this
    }

    private fun getRealPath(info: ComponentInfo): String {
        return info.mini_program_url!!.plus(if (info.mini_program_url.contains("?")) "&" else "?")
                .plus("uin=${KernelUtil.getAssetAccountId()}&cid=${info.courseId}&componentId=${info.componentId}&entranceId=1&from=app")
                .plus("&page=${
                if (activity is CourseDetailActivity)
                    activity.mReportInfos.page
                else
                    ""
                }")
                .plus("&module=${
                if (info.isHelper)
                    "followxzs_click"
                else
                    "followgzh_click"
                }")
    }

    private fun getMiniProgramType(): Int {
        if (COURSE_HELPER_PROGRAM_TYPE != null) {
            return COURSE_HELPER_PROGRAM_TYPE!!
        }

        COURSE_HELPER_PROGRAM_TYPE = SharedPrefsUtil.getInt(
                DebugAppActivity.SHARED_PRE_TABLE,
                DebugAppActivity.SHARED_PRE_KEY_COURSE_HELPER,
                DebugConfig.COURSE_HELPER_FLAG
        )
        return COURSE_HELPER_PROGRAM_TYPE!!
    }

    private fun isGifFile(file: File): Boolean {
        try {
            val inputStream = FileInputStream(file)
            val flags = IntArray(5)
            flags[0] = inputStream.read()
            flags[1] = inputStream.read()
            flags[2] = inputStream.read()
            flags[3] = inputStream.read()
            inputStream.skip((inputStream.available() - 1).toLong())
            flags[4] = inputStream.read()
            inputStream.close()
            return flags[0] == 71 && flags[1] == 73 && flags[2] == 70 && flags[3] == 56 && flags[4] == 0x3B
        } catch (e: FileNotFoundException) {
            e.printStackTrace()
        } catch (e: IOException) {
            e.printStackTrace()
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return false
    }

    private fun report(isHelper: Boolean, componentId: Int, isClick: Boolean) {
        val reportExtraInfo = if (activity is CourseDetailActivity) {
            activity.mReportInfos
        } else {
            return
        }
        reportExtraInfo.eventCode = if (isClick)
            ExtraUtils.EXTRA_REPORT_CLICK_ACTION
        else
            ExtraUtils.EXTRA_REPORT_EXPOSURE_ACTION
        reportExtraInfo.module = (if (isHelper) "followxzs_" else "followgzh_") +
                if (isClick) "click" else "pop"
        val customReportDatas: MutableMap<String, String> = HashMap()
        reportExtraInfo.customDatas = customReportDatas
        customReportDatas[ReportConstant.DETAIL_VER1_KEY] = componentId.toString()
        customReportDatas[ReportConstant.DETAIL_VER2_KEY] = if (isHelper)
            "1"
        else
            "enroll"
        customReportDatas[ReportConstant.DETAIL_VER3_KEY] = "APP"
        Report.autoReportData(reportExtraInfo)
    }

}
