package com.tencent.edu.module.course.detail.util

import android.app.Activity
import com.google.gson.Gson
import com.google.gson.JsonObject
import com.tencent.edu.common.utils.LogUtils
import com.tencent.edu.kernel.KernelUtil
import com.tencent.edu.kernel.http.HttpModel
import com.tencent.edu.kernel.http.data.JsonDataObserver
import com.tencent.edu.kernel.http.util.GsonUtil

object CourseHelperUtil {

    const val TAG = "CourseHelperUtil"
    @JvmStatic
    var isDialogShowing = false

    @JvmStatic
    @JvmOverloads
    fun checkNeedShowApplyDialog(
            activity: Activity,
            courseId: String,
            aid: String,
            isPkg: Boolean,
            termId: String = "",
            listener: OnNeedShowApplyDialogListener? = null
    ) {
        // 只有微信账户才会弹窗
        if (!KernelUtil.isWXAsset()) {
            listener?.onResult()
            return
        }

        HttpModel.getWxAccountComponentInfo(
                courseId,
                termId,
                aid,
                isPkg,
                3,
                object : JsonDataObserver() {
                    override fun onError(e: String) {
                        LogUtils.e(TAG, "getWxAccountComponentInfo request error")
                        listener?.onResult()
                    }

                    override fun onSuccessWithResult(result: JsonObject) {
                        val show = GsonUtil.getInt(result, "show_flag")
                        if (show != 1) {
                            LogUtils.i(TAG, "getWxAccountComponentInfo not show")

                            listener?.onResult()
                            return
                        }

                        val type = GsonUtil.getInt(result, "guide_info_type")
                        // 如果没有返回引导信息类型，直接进行外部跳转操作
                        when (type) {
                            // 公众号二维码
                            1 -> manageInfo(result, false)
                            // 小助手
                            2 -> manageInfo(result, true)
                            else -> {
                                LogUtils.w(TAG, "getWxAccountComponentInfo no type")

                                listener?.onResult()
                                return
                            }
                        }

                        LogUtils.i(TAG, "getWxAccountComponentInfo request success")
                    }

                    private fun manageInfo(result: JsonObject, isHelper: Boolean) {
                        val info = parseInfo(result, isHelper)
                        if (info == null || !info.isAvailable()) {
                            LogUtils.w(TAG, "getWxAccountComponentInfo info null or unavailable")
                            listener?.onResult()
                            return
                        }

                        if (CourseHelperDialog(activity).setInfo(info, isHelper) {
                                    isDialogShowing = false
                                    listener?.onResult()
                                }.show())
                            isDialogShowing = true
                    }

                    private fun parseInfo(result: JsonObject, helper: Boolean): ComponentInfo? {
                        try {
                            val obj = result.getAsJsonObject(if (helper) "app_program_gif_info_v2" else "gif_info")
                                    ?: return null
                            val info = Gson().fromJson(obj.toString(), ComponentInfo::class.java)
                                    ?: return null
                            info.componentId = GsonUtil.getInt(result, "component_id", 1)
                            info.isHelper = helper
                            info.courseId = courseId
                            return info
                        } catch (e: Exception) {
                            LogUtils.e(TAG, "getWxAccountComponentInfo parse error:$e")
                        }

                        return null
                    }

                }
        )
    }

    interface OnNeedShowApplyDialogListener {
        fun onResult()
    }
}