package com.tencent.edu.module.course.task.courseware

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.view.View
import android.view.ViewGroup
import android.widget.RelativeLayout
import com.tencent.edu.R
import com.tencent.edu.common.compat.WindowCompat
import com.tencent.edu.common.misc.StatusBarPlaceHolder
import com.tencent.edu.common.utils.MiscUtils
import com.tencent.edu.common.utils.SimpleBeaconReportUtil
import com.tencent.edu.commonview.activity.BaseActivity
import com.tencent.edu.framework.utils.ToastUtil
import com.tencent.edu.module.course.task.courseware.FilePreviewFlutterActivity.*
import com.tencent.edu.webview.superFileView.SuperFileView2
import com.tencent.smtt.sdk.WebView
import kotlinx.android.synthetic.main.activity_file_preview.*
import java.io.File

/**
 * 文档预览界面.
 *
 * @author aidanyang
 * @date 2020-6-24
 */
class FilePreviewActivity : BaseActivity() {

    private lateinit var mType: String
    private lateinit var mPath: String
    private lateinit var mName: String
    private var mFileType = 0
    private var mSize: Int = 0

    private lateinit var mFileShareUtil: FileShareUtil

    // 预览界面
    private var mSuperFileView: SuperFileView2? = null
    private var mActionBar: FilePreviewActionBar? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        WindowCompat.setStatusBarTranslucent(this)

        setContentView(R.layout.activity_file_preview)

        initData()
        initShare()
        initActionBar()
        switchView()
    }

    private fun initShare() {
        mFileShareUtil = FileShareUtil(this, FileShareUtil.OnShareSelectedListener { isWx: Boolean ->
            if (isWx && (mSize >= WX_SHARE_LIMIT)) {
                ToastUtil.showToast("超10M文件暂不支持分享微信")
                false
            } else true
        })
    }

    private fun initData() {
        mName = intent.getStringExtra(DATA_NAME)

        // 如果有课件id，直接通过课件id和文件名获取课件路径
        val id = intent.getStringExtra(DATA_ID)
        mPath = if (id.isNullOrBlank()) {
            intent.getStringExtra(DATA_PATH)
        } else {
            FilePreviewUtils.getFilePath(id, mName)
        }

        val fileType = intent.getStringExtra(DATA_FILE_TYPE)
        mFileType = if (fileType.isNullOrBlank()) {
            intent.getIntExtra(DATA_FILE_TYPE, FilePreviewUtils.OTHER)
        } else {
            fileType.toInt()
        }

        val strSize = intent.getStringExtra(DATA_SIZE)
        mSize = if (strSize.isNullOrBlank()) {
            intent.getIntExtra(DATA_SIZE, 0)
        } else {
            strSize.toInt()
        }

        mType = intent.getStringExtra(DATA_TYPE)
    }

    private fun initActionBar() {
        mActionBar = file_preview_bar

        mActionBar?.setOnShareClickListener {
            mFileShareUtil.share(mName, File(mPath), FilePreviewUtils.isImg(mFileType))
        }

        mActionBar?.setOnOpenThirdPartyClickListener {
            MiscUtils.openLocalFile(File(mPath))
        }
    }

    private fun switchView() {
        when (mFileType) {
            FilePreviewUtils.VIDEO -> {
                mActionBar?.setDarkMode(false)

                WindowCompat.setStatusBarDarkMode(this, false)
                setActionBarMarginTop()

                initVideoView()
            }
            FilePreviewUtils.IMG -> {
                mActionBar?.setDarkMode(false)

                WindowCompat.setStatusBarDarkMode(this, false)
                setActionBarMarginTop()

                initImageView()
            }
            FilePreviewUtils.AUDIO -> {
                mActionBar?.setDarkMode(true)

                mActionBar?.setTitleCenterText(mName)
                WindowCompat.setStatusBarDarkMode(this, true)

                initAudioView()

                StatusBarPlaceHolder.create(this, preview_ll, 0)
            }
            else -> {
                mActionBar?.setDarkMode(true)

                mActionBar?.setTitleCenterText(mName)
                WindowCompat.setStatusBarDarkMode(this, true)

                if (mType.contains("htm")) {
                    initWebView()
                } else {
                    initDocView()
                }
                StatusBarPlaceHolder.create(this, preview_ll, 0)
            }
        }
    }

    private fun initWebView() {
        val webView = WebView(this)
        webView.settings.javaScriptEnabled = false
        addView(webView)
        webView.loadUrl("file://$mPath")
    }

    private fun initImageView() {
        val view = ImagePreviewView(this)
        addView(view, false)
        lifecycle.addObserver(view)
        view.setFile(mPath)
    }

    private fun setActionBarMarginTop() {
        val layoutParams = mActionBar?.layoutParams as RelativeLayout.LayoutParams?
        layoutParams?.topMargin = StatusBarPlaceHolder.getStatusBarHeight(this)
        mActionBar?.layoutParams = layoutParams
    }

    private fun initVideoView() {
        val view = VideoPreviewView(this)
        lifecycle.addObserver(view)
        addView(view, false)
        view.setFile(mPath)
    }

    private fun initAudioView() {
        val view = AudioPreviewView(this)
        lifecycle.addObserver(view)
        addView(view)
        view.setFile(mPath)
    }

    private fun initDocView() {
        mSuperFileView = SuperFileView2(this)
        addView(mSuperFileView!!)
        mSuperFileView?.displayFile(File(mPath))
    }

    private fun addView(view: View, below: Boolean = true) {
        root_view.addView(view, 0)
        val layoutParams = view.layoutParams as RelativeLayout.LayoutParams
        layoutParams.width = ViewGroup.LayoutParams.MATCH_PARENT
        layoutParams.height = ViewGroup.LayoutParams.MATCH_PARENT
        if (below) {
            layoutParams.addRule(RelativeLayout.BELOW, mActionBar?.id!!)
        }
        view.layoutParams = layoutParams
    }

    public override fun onDestroy() {
        super.onDestroy()
        if (mSuperFileView != null) {
            mSuperFileView!!.onStopDisplay()
        }
    }

    companion object {
        private val TAG = FilePreviewActivity::class.java.simpleName

        @JvmStatic
        fun start(context: Context, path: String, fileType: Int, name: String, size: Int, type: String) {
            val intent = Intent()
            intent.putExtra(DATA_PATH, path)
            intent.putExtra(DATA_FILE_TYPE, fileType)
            intent.putExtra(DATA_NAME, name)
            intent.putExtra(DATA_SIZE, size)
            intent.putExtra(DATA_TYPE, type)
            intent.setClass(context, FilePreviewActivity::class.java)
            context.startActivity(intent)
        }
    }
}