/*
 * Copyright © 1998 - 2020 Tencent. All Rights Reserved.
 * 腾讯公司 版权所有
 */
package com.tencent.edu.module.course.task.courseware

import android.app.Activity
import android.content.Context
import android.graphics.Bitmap
import android.text.TextUtils
import android.view.MotionEvent
import android.view.View
import android.widget.FrameLayout
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LifecycleObserver
import androidx.lifecycle.OnLifecycleEvent
import com.nostra13.universalimageloader.core.assist.FailReason
import com.nostra13.universalimageloader.core.listener.ImageLoadingListener
import com.tencent.edu.R
import com.tencent.edu.common.core.ThreadMgr
import com.tencent.edu.common.imageloader.ImageLoaderProxy
import com.tencent.edu.common.permission.PermissionManager
import com.tencent.edu.common.permission.PermissionManager.GrantListener
import com.tencent.edu.common.permission.PermissionsDispatcher
import com.tencent.edu.common.utils.BitmapUtil
import com.tencent.edu.common.utils.FileUtils
import com.tencent.edu.common.utils.Tips
import com.tencent.edu.module.webinfopages.data.ImageHandlerThread
import kotlinx.android.synthetic.main.layout_preview_image.view.*
import uk.co.senab.photoview.PhotoViewAttacher
import java.io.File

/**
 * @author aidanyang
 * @date 2020-6-28
 */
class ImagePreviewView constructor(context: Context) : FrameLayout(context), LifecycleObserver {

    private var path: String? = null
    private var mPermissionManager: PermissionManager? = null

    init {
        initView()
    }

    private fun initView() {
        View.inflate(context, R.layout.layout_preview_image, this)

        save_image.setOnClickListener {
            if (mPermissionManager == null) {
                mPermissionManager = PermissionManager()
                mPermissionManager?.registerGrantObserver { requestCode, _, grantResults ->
                    if (requestCode == PermissionsDispatcher.REQUEST_GROUP_STORAGE && PermissionsDispatcher.isGrant(grantResults)) {
                        ThreadMgr.postToSubThread {
                            if (path.isNullOrBlank()) {
                                return@postToSubThread
                            }
                            val file = File(path)
                            val target = File("${FileUtils.getExternalDirectoryPath()}/${FileUtils.PIC_ROOT_PATH}/${System.currentTimeMillis()}.${file.extension}")
                            file.copyTo(target)
                            BitmapUtil.notifyGallery(target)
                            BitmapUtil.onSaveSuccess(target)
                        }
                    }
                }
            }
            mPermissionManager?.checkStoragePermission(context as Activity)
        }
    }

    fun setFile(path: String) {
        this.path = path
        val file = File(path)
        if (file.length() == 0L) {
            return
        }

        if ("gif" == FileUtils.getFileNameExtension(path)) {
            addGifView(path)
        } else {
            addImageView(path)
        }
    }

    private fun addGifView(path: String) {
        preview_gif.visibility = View.VISIBLE
        preview_gif.initGif(File(path))
        preview_gif.start()
    }

    private fun addImageView(path: String) {
        preview_img.visibility = View.VISIBLE
        // 有需要再加
        ImageLoaderProxy.displayImage("file://$path", preview_img, object : ImageLoadingListener {
            override fun onLoadingComplete(p0: String?, p1: View?, p2: Bitmap?) {
                val attacher = PhotoViewAttacher(preview_img)
                attacher.update()

                preview_img.setOnTouchListener(object : OnTouchListener {
                    val CLICK_DURATION = 300L
                    var start = 0L
                    var clickRun: Runnable? = null

                    override fun onTouch(v: View, event: MotionEvent): Boolean {
                        when (event.action) {
                            MotionEvent.ACTION_DOWN -> {
                                start = System.currentTimeMillis()
                            }
                            MotionEvent.ACTION_UP -> {
                                if (System.currentTimeMillis() - start < CLICK_DURATION) {
                                    if (clickRun == null) {
                                        clickRun = Runnable {
                                            save_image.run {
                                                visibility = if (visibility == View.VISIBLE) {
                                                    View.GONE
                                                } else {
                                                    View.VISIBLE
                                                }
                                            }
                                            clickRun = null
                                        }
                                        // 200ms内再次点击算双击
                                        ThreadMgr.postToUIThread(clickRun, 200)
                                    } else {
                                        ThreadMgr.removeUIRunnable(clickRun)
                                        clickRun = null
                                    }
                                }
                            }
                        }

                        return attacher.onTouch(v, event)
                    }

                })
            }

            override fun onLoadingStarted(p0: String?, p1: View?) {}

            override fun onLoadingCancelled(p0: String?, p1: View?) {}

            override fun onLoadingFailed(p0: String?, p1: View?, p2: FailReason?) {}
        })
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_DESTROY)
    fun onDestroy() {
        preview_gif.destroy()
    }

    companion object {

        private const val TAG = "ImagePreviewView"

    }
}