package com.tencent.edu.module.course.util

import android.app.Service
import android.media.AudioManager
import android.os.Build
import androidx.annotation.RequiresApi
import com.tencent.edu.kernel.AppRunTime

object AudioUtil {

    private val mAudioManager = AppRunTime.getApplicationContext().getSystemService(Service.AUDIO_SERVICE) as AudioManager
    private var mLastVolume = -1

    fun getSystemMaxVolume(): Int {
        return mAudioManager.getStreamMaxVolume(AudioManager.STREAM_MUSIC)
    }

    @RequiresApi(Build.VERSION_CODES.P)
    fun getSystemMinVolume(): Int {
        return mAudioManager.getStreamMinVolume(AudioManager.STREAM_MUSIC)
    }

    fun getSystemVolume(): Int {
        return mAudioManager.getStreamVolume(AudioManager.STREAM_MUSIC)
    }

    @JvmOverloads
    fun setSystemVolume(volume: Int, isReset: Boolean = false) {
        if (!isReset) {
            mLastVolume = getSystemVolume()
        }
        mAudioManager.setStreamVolume(AudioManager.STREAM_MUSIC, volume, 0)
    }

    fun getPercentSystemVolume(percent: Int): Int {
        return getSystemMaxVolume() * percent / 10
    }

    fun getLowVolume(): Int {
        return getSystemMaxVolume() * PlayMuteMgr.DEFAULT_VOLUME / 10
    }

    fun isLowVolume(volume: Int): Boolean {
        return volume <= getLowVolume()
    }

    fun resetVolume() {
        if (mLastVolume == -1 || !PlayMuteMgr.isUserNotChangedVolume) {
            return
        }

        setSystemVolume(mLastVolume, true)
        mLastVolume = -1
    }

}