package com.tencent.edu.module.homepage.widget

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import com.tencent.edu.R
import com.tencent.edu.common.core.ThreadMgr
import com.tencent.edu.common.misc.LocalUri
import com.tencent.edu.common.utils.PixelUtil
import com.tencent.edu.common.utils.SharedPrefsUtil
import com.tencent.edu.common.utils.VersionUtils
import com.tencent.edu.commonview.dialog.EduCustomizedDialog
import com.tencent.edu.framework.EduFramework
import com.tencent.edu.kernel.KernelUtil

object LiaoningDialogUtil {

    private const val SP_TABLE = "Liaoning"
    private const val SP_KEY = "Dialog"

    /**
     * 是辽宁号渠道
     */
    private fun isLiaoningChannel(): Boolean {
        return "800031628" == VersionUtils.getChannelIdFromIni(EduFramework.getApplicationContext())
    }

    @JvmStatic
    fun showDialog(c: Context): Boolean {
        if (!isLiaoningChannel() || checkIfAlreadyShow()) {
            return false
        }

        SharedPrefsUtil.putBoolean(SP_TABLE, getKey(), true)

        ThreadMgr.postToUIThread {
            EduCustomizedDialog(c).run {
                setCanceledOnTouchOutside(false)
                val layout = LayoutInflater.from(c).inflate(R.layout.dialog_liaoning, null)
                setContentView(layout)
                layout.findViewById<View>(R.id.liaoning_dialog_get).setOnClickListener {
                    LocalUri.jumpToEduUri("https://m.ke.qq.com/m-core/actInvite.html")
                    dismiss()
                }
                layout.findViewById<View>(R.id.liaoning_dialog_close).setOnClickListener {
                    dismiss()
                }
                show()
                window?.setLayout(PixelUtil.dp2px(288f), PixelUtil.dp2px(330f))
                window?.setBackgroundDrawableResource(R.color.transparent)
            }
        }

        return true
    }

    private fun checkIfAlreadyShow(): Boolean {
        return SharedPrefsUtil.getBoolean(SP_TABLE, getKey(), false)
    }

    private fun getKey(): String {
        return SP_KEY + KernelUtil.getAssetAccountId()
    }

}