package com.tencent.edu.module.kingcard

import android.content.Context
import com.tencent.edu.R
import com.tencent.edu.common.core.ThreadMgr
import com.tencent.edu.common.misc.SharedPrefsConstants
import com.tencent.edu.common.utils.DateUtil
import com.tencent.edu.common.utils.LogUtils
import com.tencent.edu.common.utils.SharedPrefsUtil
import com.tencent.edu.common.utils.SimpleBeaconReportUtil
import com.tencent.edu.kernel.KernelUtil
import com.tencent.edu.module.homepage.newhome.mine.HomepageMineItemView
import com.tencent.edu.module.homepage.newhome.mine.entity.MineListItemEntity
import com.tencent.edu.module.utils.ExtraUtils
import okhttp3.Call
import java.util.*

/**
 * 大王卡资源配置工具类
 * @author aidanyang
 * @date 2020-05-07
 */
object KingCardUtil {

    private const val TAG = "KingCardUtil"
    private const val REPORT_MODULE = ExtraUtils.EXTRA_REPORT_WANGKA
    private const val CACHE_KEY = "king_card_resource"


    private var mCall: Call? = null
    private var mShowResourceValue: String? = null
    var mUrl: String? = null

    fun manageKingCardResource(itemEntity: MineListItemEntity.ItemEntity, itemView: HomepageMineItemView) {
        mUrl = itemEntity.mLink
        val kingCardResourceList = itemEntity.kingCardResource ?: return

        LogUtils.i(TAG, "kingCardResourceList:${kingCardResourceList.contentToString()}")

        // 取最新一条数据
        var index = kingCardResourceList.size - 1
        while (index >= 0) {
            val resource = kingCardResourceList[index--]

            LogUtils.d(TAG, "check:${resource.name}")

            //不在有效时间内时循环上一条数据
            if (outOfDate(resource)) {
                continue
            }

            ThreadMgr.postToUIThread {
                // 有活动文案，优先显示活动文案，否则显示红点
                // 是否显示活动
                if (isShowResource(resource)) {
                    if (resource.text.isNotBlank()) {
                        LogUtils.d(TAG, "showSubText")

                        itemView.hideRedDot()
                        itemView.setSubText(resource.text, R.color.color_22B7FF)
                    } else {
                        itemView.hideSubText()
                        if (resource.reddot != 1) {
                            return@postToUIThread
                        }
                        LogUtils.d(TAG, "showRedDot")

                        itemView.showRedDot()
                    }
                } else {
                    LogUtils.d(TAG, "hideAll")

                    itemView.hideRedDot()
                    itemView.hideSubText()
                }
            }
            break
        }
    }

    /**
     * 判断王卡活动是否已过期
     */
    private fun outOfDate(resource: MineListItemEntity.ItemEntity.KingCardResource?): Boolean {
        val current = KernelUtil.currentTimeMillis()
        val begin = DateUtil.parseString2MilSecond(resource?.begin)

        if (current < begin) {
            LogUtils.d(TAG, "current < begin")
            return true
        }

        val end = DateUtil.parseString2MilSecond(resource?.end)
        if (current > end) {
            LogUtils.d(TAG, "current > begin")
            return true
        }

        return false
    }

    /**
     * 检查是否已被看过
     */
    private fun isShowResource(resource: MineListItemEntity.ItemEntity.KingCardResource): Boolean {
        mShowResourceValue = resource.name
        val lastClick = SharedPrefsUtil.getString(SharedPrefsConstants.TABLE_NORMAL_DATA, CACHE_KEY, Random().nextInt(10).toString())
        LogUtils.d(TAG, "lastClick:${lastClick}")
        return lastClick != resource.name
    }

    fun release() {
        mUrl = null
        mShowResourceValue = null
        mCall?.cancel()
    }

    fun click(itemView: HomepageMineItemView) {
        LogUtils.d(TAG, "click:${mShowResourceValue}")

        if (mShowResourceValue?.isNotBlank() == true) {
            SharedPrefsUtil.putSting(SharedPrefsConstants.TABLE_NORMAL_DATA, CACHE_KEY, mShowResourceValue)
            itemView.hideRedDot()
            itemView.hideSubText()
        }

        SimpleBeaconReportUtil.reportClickEvent(itemView.context, REPORT_MODULE)
    }

    fun expo(context: Context) {
        SimpleBeaconReportUtil.reportExposureEvent(context, REPORT_MODULE, "wangka")
    }

}