package com.tencent.edu.module.knowledge

import com.google.gson.JsonObject
import com.tencent.edu.common.utils.LogUtils
import com.tencent.edu.common.utils.Tips
import com.tencent.edu.kernel.http.HttpModel
import com.tencent.edu.kernel.http.data.JsonDataObserver
import com.tencent.edu.kernel.http.util.GsonUtil
import io.reactivex.disposables.Disposable

/**
 * 涨知识收藏请求类
 *
 * @author aidanyang
 * @date 2020-7-14
 */
class KnowledgeCollectRequester {

    private var mGetCollectStatusCall: Disposable? = null
    private var mSetCollectStatusCall: Disposable? = null

    /**
     * 获取收藏状态
     */
    fun getCollectStatus(resType: Int, contentId: String?, seriesId: String?, externalFlag: Int, listenerGet: GetCollectStatusListener) {
        if (mGetCollectStatusCall != null) {
            mGetCollectStatusCall?.dispose()
        }

        // contentId与seriesId互斥
        val resId = if (contentId.isNullOrBlank()) {
            if (seriesId.isNullOrBlank()) "" else seriesId
        } else contentId

        HttpModel.getCollectStatus(resId, resType, object : JsonDataObserver() {
            override fun onSubscribe(d: Disposable) {
                mGetCollectStatusCall = d
                LogUtils.i(TAG, "getCollectStatus request start")
            }

            override fun onError(code: Int, msg: String) {
                LogUtils.e(TAG, "getCollectStatus request error,code:$code msg:$msg")
                listenerGet.isCollect(false)
            }

            override fun onSuccessWithResult(result: JsonObject) {
                val isCollect = GsonUtil.getBoolean(result, "is_collect", false)
                listenerGet.isCollect(isCollect)
                LogUtils.i(TAG, "getCollectStatus request success")
            }
        })
    }

    /**
     * 更新收藏状态
     */
    fun updateCollectStatus(resType: Int, contentId: String?, externalFlag: Int, type: Int, seriesId: String?, listener: UpdateCollectStatusListener) {
        if (mSetCollectStatusCall != null) {
            mSetCollectStatusCall?.dispose()
        }

        val jObj = JsonObject()
        // contentId与seriesId互斥
        if (!contentId.isNullOrBlank())
            jObj.addProperty("res_id", contentId)
        else
            jObj.addProperty("res_id", seriesId)

        jObj.addProperty("res_type", resType)
        jObj.addProperty("op", if (type == 1) 2 else 1)

        HttpModel.updateCollectStatus(jObj, object : JsonDataObserver() {
            override fun onSubscribe(d: Disposable) {
                mSetCollectStatusCall = d
                LogUtils.i(TAG, "updateCollectStatus request start")
            }

            override fun onError(code:Int,msg: String) {
                LogUtils.e(TAG, "updateCollectStatus request error,code:$code msg:$msg")
                listener.onError(type)
            }

            override fun onSuccess(jsonResp: JsonObject): Boolean {
                LogUtils.i(TAG, "updateCollectStatus request success")
                listener.onSuccess(type)
                return true
            }

        })
    }

    interface GetCollectStatusListener {
        fun isCollect(isCollect: Boolean)
    }

    interface UpdateCollectStatusListener {
        fun onSuccess(externalFlag: Int)
        fun onError(externalFlag: Int)
    }

    open class UpdateCollectStatusListenerImpl : UpdateCollectStatusListener {
        override fun onSuccess(externalFlag: Int) {
            Tips.showShortToast((if (externalFlag == 0) "" else "取消") + "收藏成功")
        }

        override fun onError(externalFlag: Int) {
            Tips.showShortToast((if (externalFlag == 0) "" else "取消") + "收藏失败")
        }
    }

    companion object {
        private const val TAG = "KnowledgeCollectRequester"
    }
}