package com.tencent.edu.module.login

import android.annotation.SuppressLint
import android.app.Activity
import android.content.Context
import android.graphics.Color
import android.os.Bundle
import android.os.CountDownTimer
import android.text.*
import android.text.method.LinkMovementMethod
import android.text.style.ClickableSpan
import android.text.style.ForegroundColorSpan
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.widget.FrameLayout
import android.widget.RelativeLayout
import androidx.constraintlayout.widget.ConstraintLayout
import com.tencent.edu.R
import com.tencent.edu.common.callback.Callback
import com.tencent.edu.common.callback.VoidCallback
import com.tencent.edu.common.core.ThreadMgr
import com.tencent.edu.common.event.EventMgr
import com.tencent.edu.common.event.EventObserver
import com.tencent.edu.common.utils.*
import com.tencent.edu.common.utils.ClickTooFastProtectUtil.TIME_500_MS
import com.tencent.edu.commonview.dialog.DialogUtil
import com.tencent.edu.commonview.dialog.EduCustomizedDialog
import com.tencent.edu.framework.EduFramework
import com.tencent.edu.kernel.KernelEvent
import com.tencent.edu.kernel.KernelUtil
import com.tencent.edu.kernel.account.AccountLoginOrBindMgr
import com.tencent.edu.kernel.login.mgr.LoginMgr
import com.tencent.edu.kernel.login.misc.LoginDef.LOGIN_ERR_SCENE_MOBILE_BIND
import com.tencent.edu.kernel.login.misc.LoginStatus
import com.tencent.edu.kernel.report.Report
import com.tencent.edu.module.LoginActivity
import com.tencent.edu.module.login.nationdialog.LoginNationCodeDialog
import com.tencent.edu.module.login.nationdialog.NationCodeItem
import com.tencent.edu.module.mobileverify.MobileVerifyCenter
import com.tencent.edu.module.mobileverify.MobileVerifyDialog
import com.tencent.edu.module.report.AutoReportMgr
import com.tencent.edu.module.report.LoginMonitor
import com.tencent.edu.module.report.ReportConstant
import com.tencent.edu.module.utils.ExtraUtils
import com.tencent.edu.module.vodplayer.widget.GifImageViewExt
import com.tencent.edu.module.webapi.H5Config
import com.tencent.edu.module.webapi.WebOpenUrlActivity
import com.tencent.edu.wxapi.WXEntryActivity
import com.tencent.edu.wxapi.WXOpenApi
import com.tencent.oedmobileverifyexample.OEDMobileVerify
import com.tencent.pbaccountbind.pbaccountbind
import com.tencent.tinylogin.PhoneCodeType
import kotlinx.android.synthetic.main.layout_cutomloginview.view.*
import kotlinx.android.synthetic.main.layout_login_gray.view.*
import kotlinx.android.synthetic.main.layout_login_mobile_sms.view.*
import kotlinx.android.synthetic.main.layout_login_onekey.view.*
import java.util.*

class LoginCustomView : RelativeLayout, View.OnClickListener {

    private var mIsFromDialog = false
    private var mIsLoginToHomePage = true
    private var mActivity: Activity? = null
    private var mLastClickTime: Long = 0
    private var mWXShareAPI: WXOpenApi? = null
    private var mMobileLoadingGif: GifImageViewExt? = null
    private var mAssetLoginType: Int = -1

    //    private Bitmap mBackgroudBitmap;
    private var mLoginLoadingDialog: EduCustomizedDialog? = null
    private var mLoginActionListener: OnLoginActionListener? = null
    private var mIsCheckProtocol = false

    private val isUnCheckedAgree: Boolean
        get() {
            if (!mIsCheckProtocol) {
                EduToast.show("请先勾选相关协议")
                return true
            }
            return false
        }

    // LoginView的类型
    private var mType = TYPE_MOBILE

    // 获取验证码倒计时
    private var mVerifyTimer: CountDownTimer? = null

    // 显示语音验证码倒计时
    private var mUnreceivedTimer: CountDownTimer? = null

    // 验证码按钮正在读秒
    private var isVerifyCountDowning = false

    // 验证码类型
    private var mCodeType = PhoneCodeType.PHONE_CODE_TEXT

    // 获取手机号信息
    private var mOEDMobileInfo: OEDMobileVerify.OEDMobileInfo? = null

    // 获取手机号超时
    private var isGetMobileTimeout = false

    // 上一次获取验证码的手机号，或者自动获取的手机号，仅做上报
    private var mMobile: String? = null


    constructor(context: Context) : super(context) {
        init(context)
    }

    constructor(context: Context, attrs: AttributeSet?) : super(context, attrs) {
        init(context)
    }

    constructor(context: Context, attrs: AttributeSet?, defStyle: Int) : super(context, attrs, defStyle) {
        init(context)
    }

    fun unInit() {
        mVerifyTimer?.cancel()
        mUnreceivedTimer?.cancel()
        EventMgr.getInstance().delEventObserver(KernelEvent.EVENT_THIRD_LOGIN_SUCC, mThirdLoginObserver)
    }

    private fun init(context: Context) {
        View.inflate(context, R.layout.layout_cutomloginview, this)
        mWXShareAPI = WXOpenApi()
        mWXShareAPI!!.initWXApi(getContext())
        // 登录页面打开，就一定不是绑定场景
        WXEntryActivity.mBindListener = null
        EventMgr.getInstance().addEventObserver(KernelEvent.EVENT_THIRD_LOGIN_SUCC, mThirdLoginObserver)
    }

    @JvmOverloads
    fun initCustomLogin(
            rootActivity: Activity?,
            listener: OnLoginActionListener?,
            isLoginToHomePage: Boolean = true,
            isFromDialog: Boolean = false,
            type: Int = TYPE_NONE,
            assetLoginType: Int = -1
    ) {
        if (rootActivity == null) {
            LogUtils.e(TAG, "No RootActivity!")
            return
        }
        mActivity = rootActivity
        mLoginActionListener = listener
        mAssetLoginType = assetLoginType
        mIsLoginToHomePage = isLoginToHomePage
        mIsFromDialog = isFromDialog
        login_mobile_tv.setOnClickListener(this)
        login_qq_tv.setOnClickListener(this)
        login_wx_tv.setOnClickListener(this)
        login_sso_tv.setOnClickListener(this)
        login_agree.setOnClickListener(this)

        initType(type)
        initLoginView()
        setupProtocolText()
    }

    private fun initType(type: Int) {
        // 判断是不是强制用手机号
        mType = when {
            type != TYPE_NONE -> {
                type
            }
            // 手机号登录需求，灰度登录页面，暂时不判断上一次的登录状态
            else -> {
                TYPE_GRAY
                // 拿缓存判断上次使用的登录方式
//                getLoginType()
            }
        }
    }

    private fun initLoginView() {
        // 灰度页面
        if (mType == TYPE_GRAY) {
            initGrayLogin()
        } else if (mType == TYPE_MOBILE || mType == TYPE_BIND_MOBILE) {
            initMobileLogin()
        } else {
            initOneKeyLogin()
            // 页面曝光上报
            reportBeacon()
        }

        setLoginButton()
    }

    private fun initGrayLogin() {
        setLoginView(R.layout.layout_login_gray)

        login_gray_qq_container.setOnClickListener {
            clickQQ(false)
        }
        login_gray_wx_container.setOnClickListener {
            clickWX(false)
        }
        login_gray_more_way.setOnClickListener {
            if (!ClickTooFastProtectUtil.isClickTooFast(login_gray_more_way.id.toString(), TIME_500_MS.toLong()))
                LoginMoreWayActivity.start(context, mIsLoginToHomePage)
        }
        login_gray_notify_tv.visibility = View.VISIBLE
    }

    private fun initOneKeyLogin() {
        setOneKeyLoginView()
    }

    /**
     * 需要有READ_PHONE_STATE权限才能调用SDK获取手机号
     */
    private fun initMobileLogin() {
        setMobileLoadingView()

        MobileVerifyCenter.getLocalPhoneNumber(mActivity!!, OEDMobileVerify.GetLocalPhoneNumberCallback { oedMobileInfo ->
            if (isGetMobileTimeout) {
                return@GetLocalPhoneNumberCallback
            }

            ThreadMgr.postToUIThread {
                if (oedMobileInfo != null && oedMobileInfo.bizErrCode == 0) {
                    mOEDMobileInfo = oedMobileInfo
                    mMobile = oedMobileInfo.mobile
                    appendMobileAgreement(oedMobileInfo.vType, oedMobileInfo.carrierType)
                    initOneKeyLogin()
                    login_mobile_auto_tv.visibility = View.VISIBLE
                    login_mobile_auto_tv.text = oedMobileInfo.mobile
                    login_mobile_change_tv.visibility = View.VISIBLE
                    login_mobile_change_tv.setOnClickListener(this@LoginCustomView)

                    if (mType == TYPE_BIND_MOBILE) {
                        login_onekey_tv.text = context.getString(R.string.onekey_bind)
                    }

                    reportBeacon(mobile = mMobile)
                } else {
                    setMobileLoginView()

                    reportBeacon()
                }
            }
        })

        if (mType == TYPE_BIND_MOBILE) {
            bind_mobile_close.visibility = View.VISIBLE
            bind_mobile_close.setColorFilter(Color.WHITE)
            bind_mobile_close.setOnClickListener {
                mActivity?.finish()
            }
            setBindMobileProtocol()
        }
    }

    private fun reportBeacon(isClick: Boolean = false, module: String? = null, mobile: String? = null, contentId: String? = null, ver2: Int? = null, ver3: String? = null) {
        val reportExtraInfo = AutoReportMgr.getReportExtraInfo(context) ?: return
        reportExtraInfo.eventCode = if (isClick) ExtraUtils.EXTRA_REPORT_CLICK_ACTION else ExtraUtils.EXTRA_REPORT_EXPOSURE_ACTION
        reportExtraInfo.page = getPage() ?: return
        reportExtraInfo.module = module

        val customReportDatas: MutableMap<String, String> = HashMap()
        reportExtraInfo.customDatas = customReportDatas

        if (mobile != null)
            customReportDatas[ReportConstant.MOBILE] = mobile
        if (contentId != null) {
            customReportDatas[ReportConstant.CONTENT_ID] = contentId
        }
        if (ver2 != null)
            customReportDatas[ReportConstant.DETAIL_VER2_KEY] = ver2.toString()
        if (ver3 != null)
            customReportDatas[ReportConstant.DETAIL_VER3_KEY] = ver3

        Report.autoReportData(reportExtraInfo)
    }

    private fun getPage(): String? {
        return when (mType) {
            TYPE_MOBILE -> {
                "mob_login"
            }
            TYPE_WX -> {
                "wx_login"
            }
            TYPE_QQ -> {
                "qq_login"
            }
            TYPE_BIND_MOBILE -> {
                if (login_mobile_auto_tv?.visibility == View.VISIBLE) {
                    "mob_oneclickbind"
                } else {
                    "mob_bind"
                }
            }
            else -> null
        }
    }

    private fun setMobileLoadingView() {
        setMobileLoadingTimeout()
        if (mMobileLoadingGif == null) {
            mMobileLoadingGif = GifImageViewExt(context).apply {
                initGif(R.raw.loading_gif)
                setLoginView(this)
                val params = layoutParams as FrameLayout.LayoutParams
                params.height = PixelUtil.dp2px(26f)
                params.width = PixelUtil.dp2px(26f)
                layoutParams = params
                setPadding(0, PixelUtil.dp2px(87f), 0, 0)
            }
        } else {
            setLoginView(mMobileLoadingGif!!)
        }
    }

    private fun setMobileLoadingTimeout() {
        ThreadMgr.postToUIThread({
            if (!MiscUtils.isActivityValid(mActivity) || mMobileLoadingGif == null) {
                return@postToUIThread
            }

            isGetMobileTimeout = true
            setMobileLoginView()
        }, GET_MOBILE_TIMEOUT)
    }

    private fun setBindMobileProtocol() {
        login_agree_container.visibility = View.GONE
        bind_mobile_protocol_tv.visibility = View.VISIBLE
    }

    private fun setupProtocolText() {
        val spannableString = SpannableString(context.getString(R.string.login_account_protocol))
        spannableString.setSpan(ForegroundColorSpan(Color.WHITE), 12, 16, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
        spannableString.setSpan(ForegroundColorSpan(Color.WHITE), 19, 23, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
        spannableString.setSpan(object : ClickableSpan() {
            override fun updateDrawState(ds: TextPaint) {}
            override fun onClick(widget: View) {
                showServiceProtocol()
            }
        }, 12, 16, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
        spannableString.setSpan(object : ClickableSpan() {
            override fun updateDrawState(ds: TextPaint) {}
            override fun onClick(widget: View) {
                showPrivacyProtocol()
            }
        }, 19, 23, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
        login_account_protocol_txt.text = spannableString
        login_account_protocol_txt.movementMethod = LinkMovementMethod.getInstance()
        login_account_protocol_txt.highlightColor = resources.getColor(android.R.color.transparent)
    }

    private fun appendMobileAgreement(mVType: Int, carrierType: Int) {
        val agreementSb = StringBuilder()
        var url: String? = null
        if (mVType == carrierType) {
            when (mVType) {
                MobileVerifyDialog.MobileType.MOBILE -> { //移动
                    agreementSb.append("以及《中国移动认证服务协议》")
                    url = "https://wap.cmpassport.com/resources/html/contract.html"
                }
                MobileVerifyDialog.MobileType.UNICOM -> { //联通
                    agreementSb.append("以及《联通统一认证服务条款》")
                    url = "https://opencloud.wostore.cn/authz/resource/html/disclaimer.html?fromsdk=true"
                }
                MobileVerifyDialog.MobileType.TELECOM -> { //电信
                    agreementSb.append("以及《天翼账号提供认证服务与隐私协议》")
                    url = "https://e.189.cn/sdk/agreement/show.do?order=2&type=main&appKey=&hidetop=true&returnUrl="
                }
            }
        }
        if (TextUtils.isEmpty(url)) {
            login_account_protocol_txt.append(agreementSb.toString())
            return
        }
        val agreement = agreementSb.toString()
        val spannableString = SpannableString(agreement)
        spannableString.setSpan(ForegroundColorSpan(Color.WHITE), 2, agreement.length, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
        spannableString.setSpan(object : ClickableSpan() {
            override fun updateDrawState(ds: TextPaint) {}
            override fun onClick(widget: View) {
                WebOpenUrlActivity.start(widget.context, url)
            }
        }, 2, agreement.length, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
        login_account_protocol_txt.append(spannableString)
    }

    private fun setOneKeyLoginView() {
        mMobileLoadingGif = null
        setLoginView(R.layout.layout_login_onekey)
        login_onekey_container.setOnClickListener(this)
    }

    private fun setMobileLoginView() {
        mMobileLoadingGif = null
        setLoginView(R.layout.layout_login_mobile_sms)
        if (mType == TYPE_MOBILE) {
            setupProtocolText()
        } else {
            login_mobile_btn.text = context.getText(R.string.bind_mobile)
        }

        login_mobile_btn.setOnClickListener(this)
        login_verify_btn.setOnClickListener(this)
        login_verify_unreceive.setOnClickListener(this)
        login_nation_code_tv.setOnClickListener(this)

        login_verify_et.addTextChangedListener(object : TextWatcher {
            override fun afterTextChanged(s: Editable?) {
            }

            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
            }

            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
                checkIsMobileButtonEnable()
            }
        })

        login_mobile_et.addTextChangedListener(object : TextWatcher {
            override fun afterTextChanged(s: Editable?) {
            }

            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
            }

            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
                if (!isVerifyCountDowning) {
                    setVerifyButtonEnable(login_verify_btn, checkIsMobile(s?.toString()))
                }
                checkIsMobileButtonEnable()
            }
        })

        setKeyboardListener()
    }

    private fun checkIsMobile(mobile: String?): Boolean {
        // 当区号不是中国时，只要手机号不为空，就可以获取验证码
        if ("86" != getTvNationCode()) {
            return !mobile.isNullOrBlank()
        }

        return RegexUtils.isMobile(mobile)
    }

    private var isKeyboardShow = false
    private fun setKeyboardListener() {
        val softKeyboardStateHelper = KeyboardChangeListener(mActivity)
        softKeyboardStateHelper.setKeyBoardListener { isShow, keyboardHeight ->
            if (isKeyboardShow == isShow) {
                return@setKeyBoardListener
            }

            val params = container_login_action.layoutParams as ConstraintLayout.LayoutParams
            if (isShow) {
                //键盘的弹出
//                ToastUtil.shortToast(context, "软键盘弹出")
                params.bottomMargin = keyboardHeight + PixelUtil.dp2px(20f)
                params.topToTop = ConstraintLayout.LayoutParams.UNSET
            } else {
                //键盘的收起
//                ToastUtil.shortToast(context, "软键盘收起")
                params.bottomMargin = 0
                params.topToTop = ConstraintLayout.LayoutParams.PARENT_ID
            }
            container_login_action.layoutParams = params

            isKeyboardShow = isShow
        }
    }

    /**
     * 开始显示语音验证码倒计时
     */
    private fun startUnreceivedCountDown() {
        // 如果已经显示语音验证码按钮，直接返回
        if (login_verify_unreceive.visibility == View.VISIBLE) {
            return
        }

        // 开始读秒
        if (mUnreceivedTimer == null) {
            mUnreceivedTimer = object : CountDownTimer(COUNTDOWN_UNRECEIVE_TOTAL, COUNTDOWN_INTERVAL) {
                override fun onFinish() {
                    login_verify_unreceive.visibility = View.VISIBLE

                    reportBeacon(module = "recvoicecode", mobile = mMobile, contentId = getContentId())
                }

                override fun onTick(millisUntilFinished: Long) {
                }
            }
        } else {
            mUnreceivedTimer?.cancel()
        }

        mUnreceivedTimer?.start()
    }

    private fun checkIsMobileButtonEnable() {
        val mobile = getEtMobile()
        val verify = getEtVerify()

        setLoginButtonEnable(login_mobile_btn, checkIsMobile(mobile) && !verify.isNullOrBlank())
    }

    private fun getEtVerify(): String? {
        return login_verify_et.text?.toString()
    }

    private fun getEtMobile(): String? {
        return login_mobile_et.text?.toString()
    }

    private fun getTvNationCode(): String? {
        return login_nation_code_tv.text?.toString()?.replace("+", "")
    }

    private var isLoginBtnEnable = false

    private fun setLoginButtonEnable(v: View, enable: Boolean) {
        if (isLoginBtnEnable == enable) {
            return
        }
        isLoginBtnEnable = enable

        v.setBackgroundResource(if (enable) R.drawable.bg_login_btn_bg else R.drawable.bg_login_btn_alpha)
    }

    private fun setVerifyButtonEnable(v: View, enable: Boolean) {
        // 不重复设置button状态
        if ((enable && v.alpha == 1f) || (!enable && v.alpha == 0.3f)) {
            return
        }

        v.alpha = if (enable) 1f else 0.3f
    }

    /**
     * 目前获取验证码和验证手机号非激活状态的透明度为30%，可以用以下方式判断是否可用
     */
    private fun isButtonEnable(v: View): Boolean {
        return v.alpha != 0.3f
    }

    private fun setLoginView(layoutId: Int) {
        val view = LayoutInflater.from(context).inflate(layoutId, null)
        setLoginView(view)
    }

    private fun setLoginView(view: View) {
        container_login_action.removeAllViews()
        container_login_action.addView(view)
    }

    private fun setLoginButton() {
        when (mType) {
            TYPE_GRAY -> {
                login_mobile_tv.visibility = View.GONE
                login_wx_tv.visibility = View.GONE
                login_qq_tv.visibility = View.GONE
                login_sso_tv.visibility = View.GONE
            }
            TYPE_MOBILE -> {
                login_mobile_tv.visibility = View.GONE
                setLoginButtonMiddle(login_wx_tv)
            }
            TYPE_WX -> {
                login_wx_tv.visibility = View.GONE
                setOneKeyDrawable(R.drawable.ic_login_wx_s)
                setLoginButtonMiddle(login_qq_tv)
            }
            TYPE_QQ -> {
                login_qq_tv.visibility = View.GONE
                setOneKeyDrawable(R.drawable.ic_login_qq_s)
                setLoginButtonMiddle(login_wx_tv)
            }
            TYPE_BIND_MOBILE -> {
                login_mobile_tv.visibility = View.GONE
                login_wx_tv.visibility = View.GONE
                login_qq_tv.visibility = View.GONE
                login_sso_tv.visibility = View.GONE
            }
        }
    }

    private fun setOneKeyDrawable(resId: Int) {
        login_onekey_tv.setCompoundDrawablesWithIntrinsicBounds(resId, 0, 0, 0)
        login_onekey_tv.compoundDrawablePadding = PixelUtil.dp2px(8f)
    }

    private fun setLoginButtonMiddle(v: View) {
        val params = v.layoutParams as ConstraintLayout.LayoutParams
        params.leftMargin = PixelUtil.dp2px(22f)
        params.rightMargin = PixelUtil.dp2px(22f)
        v.layoutParams = params
    }

    private fun showServiceProtocol() {
        WebOpenUrlActivity.start(context, H5Config.PRO_SERVICE, false)
    }

    private fun showPrivacyProtocol() {
        WebOpenUrlActivity.start(context, H5Config.PRIVACY_SERVICE, true)
    }

    private fun showWeixinInstallDialog(activity: Activity?) {
        DialogUtil.createOneBtnDialog(activity, null, activity!!.getString(R.string.not_install_wx_msg), activity.getString(R.string.ok), true).show()
    }

    private fun showLoginLoadingDialog(rootActivity: Activity?) {
        if (rootActivity == null) {
            return
        }

        try {
            if (mLoginLoadingDialog == null || !mLoginLoadingDialog!!.isShowing) {
                mLoginLoadingDialog = LoadingDialog.createWaitingDialog(
                        rootActivity,
                        rootActivity.getString(
                                if (mType == TYPE_BIND_MOBILE)
                                    R.string.bind_waiting
                                else
                                    R.string.login_waiting)
                )
                mLoginLoadingDialog?.setOnBackPressListener { mLoginLoadingDialog = null }?.show()
            }
        } catch (e: Exception) {
            LogUtils.assertCondition(false, "LoginCustomView", "loadingDialog show failed:" + e.message)
        }
    }

    fun closeLoginLoadingDialog() {
        try {
            if (mLoginLoadingDialog != null && mLoginLoadingDialog!!.isShowing) {
                mLoginLoadingDialog!!.dismiss()
                mLoginLoadingDialog = null
            }
        } catch (e: Exception) {
            LogUtils.assertCondition(false, "LoginCustomView", "loadingDialog close failed:" + e.message)
        }
    }

    override fun onClick(v: View) {
        when (v.id) {
            R.id.login_qq_tv -> clickQQ(false)
            R.id.login_wx_tv -> clickWX(false)
            R.id.login_sso_tv -> clickSSO()
            R.id.login_agree -> clickAgree()
            R.id.login_onekey_container -> clickOneKey()
            R.id.login_mobile_tv -> startMobileLoginActivity()
            R.id.login_mobile_btn -> verifyMobile()
            R.id.login_mobile_change_tv -> {
                setMobileLoginView()
                reportBeacon(true, "changemob", contentId = "mob_oneclicklogin")
            }
            R.id.login_verify_btn -> getSmsVerifyCode()
            R.id.login_verify_unreceive -> getVoiceVerifyCode()
            R.id.login_nation_code_tv -> showNationCodeDialog()
        }
    }

    @SuppressLint("SetTextI18n")
    private fun showNationCodeDialog() {
        MiscUtils.hideSoftInput(this)

        if (LoginNationCodeDialog(mActivity!!).setListener(object : LoginNationCodeDialog.NationCodeDialogListener {
                    override fun onSelected(item: NationCodeItem) {
                        login_nation_code_tv.text = "+${item.dialing_code}"
                        MiscUtils.setNavigationBarColor(mActivity, R.color.login_bg_color)

                        if (!isVerifyCountDowning) {
                            setVerifyButtonEnable(login_verify_btn, checkIsMobile(getEtMobile()))
                        }
                    }

                    override fun onCancel() {
                        MiscUtils.setNavigationBarColor(mActivity, R.color.login_bg_color)
                    }
                }).show()) {
            MiscUtils.setNavigationBarColor(mActivity, null)
        }
    }

    private fun getVoiceVerifyCode() {
        mCodeType = PhoneCodeType.PHONE_CODE_VOICE

        login_verify_unreceive.visibility = View.GONE

        mMobile = getEtMobile()

        if (mMobile.isNullOrBlank()) {
            Tips.showShortToast("请输入手机号")
            return
        }

        Tips.showShortToast("即将语音提供验证码，请注意接听")
        MobileVerifyCenter.requireVoiceVerify(mMobile!!, getTvNationCode()!!, getPage())

        reportBeacon(true, "recvoicecode", mobile = mMobile, contentId = getContentId())
    }

    private fun getSmsVerifyCode() {
        if (!isButtonEnable(login_verify_btn) || isVerifyCountDowning) {
            return
        }
        mCodeType = PhoneCodeType.PHONE_CODE_TEXT
        mMobile = getEtMobile()!!

        MobileVerifyCenter.requireSmsVerify(mMobile!!, getTvNationCode()!!, getPage())
        startVerifyCountDown()
        startUnreceivedCountDown()

        reportBeacon(true, "recmsgcode", mobile = mMobile, contentId = getContentId())
    }

    @SuppressLint("SetTextI18n")
    private fun startVerifyCountDown() {
        // 开始读秒
        isVerifyCountDowning = true
        if (mVerifyTimer == null) {
            mVerifyTimer = object : CountDownTimer(COUNTDOWN_VERIFY_TOTAL, COUNTDOWN_INTERVAL) {
                override fun onFinish() {
                    // 结束读秒
                    isVerifyCountDowning = false
                    login_verify_btn.text = context.getString(R.string.get_verify_code)
                    // 刷新验证码按钮
                    setVerifyButtonEnable(login_verify_btn, checkIsMobile(login_mobile_et?.text?.toString()))
                }

                override fun onTick(millisUntilFinished: Long) {
                    login_verify_btn.text = "${millisUntilFinished / 1000}秒后重试"
                }
            }
        }

        mVerifyTimer?.start()
    }

    private fun verifyMobile() {
        if (!isButtonEnable(login_mobile_btn)) {
            return
        }

        val nationCode = getTvNationCode()
        val mobile = getEtMobile()
        val code = getEtVerify()

        LogUtils.i(TAG, "on verifyMobile,nationCode：$nationCode , mobile:$mobile , code:$code")
        if (nationCode.isNullOrBlank() || mobile.isNullOrBlank() || code.isNullOrBlank()) {
            return
        }

        if (mActivity != null)
            MiscUtils.hideSoftInput(mActivity!!.window.decorView)

        if (mType == TYPE_MOBILE) {
            LogUtils.i(TAG, "click to login mobile")
            if (isUnCheckedAgree) {
                return
            }

            reportBeacon(true, "mobloginbutton", mMobile)

            showLoginLoadingDialog(mActivity)
            LoginMgr.getInstance().phoneLogin(nationCode, mobile, mCodeType, code)
        } else {
            reportBeacon(true, "mobbindbutton", mMobile, "msgcode")

            AccountLoginOrBindMgr.bindPhoneAcc(mobile, nationCode, code, mCodeType, createBindPhoneCallback())
        }

//        AccountLoginOrBindMgr.jump2ChangeBindingMobilePage(phoneNum, nationCode)
    }

    // 绑定手机号回调
    private fun resultBindMobile() {
        if (mAssetLoginType != -1)
            LoginMgr.getInstance().notifyLoginSuccess(mAssetLoginType, LoginActivity.FROM_BIND_MOBILE)

        mActivity?.setResult(Activity.RESULT_OK)
        mActivity?.finish()
    }

    private fun clickOneKey() {
        when (mType) {
            TYPE_QQ -> clickQQ(true)
            TYPE_WX -> clickWX(true)
            TYPE_MOBILE -> clickMobileOneKey()
            TYPE_BIND_MOBILE -> clickBindOneKey()
        }
    }

    private fun clickBindOneKey() {
        if (mOEDMobileInfo == null) {
            LogUtils.e(TAG, "clickBindMobile,mOEDMobileInfo == null")
            return
        }

        AccountLoginOrBindMgr.bindPhoneAccOneKey(
                mOEDMobileInfo!!.encryptedPhone,
                mOEDMobileInfo!!.msgId,
                mOEDMobileInfo!!.token,
                createBindPhoneCallback()
        )

        reportBeacon(true, module = "moboneclickbindbutton", mobile = mMobile)
    }

    /**
     * 绑定手机号回调
     */
    private fun createBindPhoneCallback(): Callback<pbaccountbind.BindAccRsp> {
        showLoginLoadingDialog(mActivity)
        return object : Callback<pbaccountbind.BindAccRsp> {
            override fun onSucc(t: pbaccountbind.BindAccRsp) {
                // 手机号登录需求，若绑定手机号后，返回的资产id和当前资产id不一致，则自动切换到返回的资产账号
                LogUtils.i(TAG, "BindPhone success")
                if (t.asset_uid.get().toString() != KernelUtil.getAssetAccountId()) {
                    LogUtils.i(TAG, "BindPhone switchLogin asset_uid != currentAssetId,switch login to uid:${t.asset_uid.get()},uidType:${t.asset_uid_type.get()}")

                    AccountLoginOrBindMgr.switchLogin(t.asset_uid.get().toString(), t.asset_uid_type.get(), object : VoidCallback {
                        override fun onError(errorCode: Int, errorMsg: String?) {
                            LogUtils.i(TAG, "BindPhone switchLogin fail,errorCode:$errorCode,errorMsg:$errorMsg")

                            closeLoginLoadingDialog()
                            LoginMgr.getInstance().loginFail(errorCode, LOGIN_ERR_SCENE_MOBILE_BIND)
                            mActivity?.finish()
                        }

                        override fun onSucc() {
                            LogUtils.i(TAG, "BindPhone switchLogin success")

                            mAssetLoginType = t.asset_uid_type.get()
                            closeLoginLoadingDialog()
                            resultBindMobile()
                        }

                    })
                } else {
                    closeLoginLoadingDialog()
                    resultBindMobile()
                }
            }

            override fun onError(errorCode: Int, errorMsg: String?) {
                LogUtils.i(TAG, "BindPhone fail,errorCode:$errorCode,errorMsg$errorMsg")

                closeLoginLoadingDialog()
                Tips.showShortToast(errorMsg?.replace("d_", "") ?: "绑定手机失败，请重试（$errorCode）")
            }
        }
    }

    private fun clickMobileOneKey() {
        if (mOEDMobileInfo == null) {
            LogUtils.e(TAG, "clickBindMobile,mOEDMobileInfo == null")
            return
        }

        LogUtils.i(TAG, "click to login mobile onekey")
        if (isUnCheckedAgree) {
            return
        }

        reportBeacon(true, "moboneclickloginbutton", mMobile, "mob_oneclicklogin")

        showLoginLoadingDialog(mActivity)
        LoginMgr.getInstance().phoneLoginOneKey(mOEDMobileInfo!!.encryptedPhone,
                mOEDMobileInfo!!.token,
                mOEDMobileInfo!!.msgId)
    }

    private fun startMobileLoginActivity() {
        LoginActivity.startLoginActivity(context, TYPE_MOBILE, mIsFromDialog)
    }

    private fun clickAgree() {
        mIsCheckProtocol = !mIsCheckProtocol
        login_agree.setImageResource(if (mIsCheckProtocol) R.drawable.icon_agree_checked else R.drawable.icon_agree_uncheck)
    }

    private fun clickSSO() {
        LogUtils.i(TAG, "click to login sso")
        if (isUnCheckedAgree) {
            return
        }
        LoginSSOActivity.start(mActivity, true)
        reportClickCampus()
        LoginMonitor.startLogin(LoginStatus.LoginType.LOGIN_SSO, 0)

        reportBeacon(true,
                "ssologinbutton",
                mobile = mMobile,
                contentId = getContentId()
        )
    }

    private fun getContentId(): String? {
        if (mType != TYPE_MOBILE) {
            return null
        }

        return if (login_mobile_auto_tv?.visibility == View.VISIBLE) "mob_oneclicklogin" else "msgcode"
    }

    private fun reportClickCampus() {
        val extraInfo = AutoReportMgr.getReportExtraInfo(mActivity)
        if (extraInfo != null) {
            extraInfo.eventCode = "click"
            extraInfo.page = "start_login"
            extraInfo.module = "campuslogin"
            Report.autoReportData(extraInfo)
        }
    }

    private fun clickQQ(isOneKey: Boolean) {
        LogUtils.i(TAG, "click to login QQ")
        if (isUnCheckedAgree) {
            return
        }
        if (!TextUtils.isEmpty(EduFramework.getAccountManager().uin)) {
            val dialog = DialogUtil.createDialog(mActivity,
                    null, "切换账号需要首先退出当前账号，确定退出",
                    "不退出",
                    "退出",
                    { dialog, _ -> dialog.dismiss() }
            ) { dialog, _ ->
                dialog.dismiss()
                LogUtils.i(TAG, "call logout before login")
                LoginMgr.getInstance().logout()
                qqLogin()
            }
            dialog.show()
        } else {
            qqLogin()
        }

        reportBeacon(true,
                if (isOneKey) "qqoneclicklogin" else "qqloginbutton",
                mobile = mMobile,
                contentId = getContentId()
        )
    }

    private fun qqLogin() {
        val time = System.currentTimeMillis()
        if (time - mLastClickTime < 1000) {
            LogUtils.d(TAG, "click too fast")
            return
        }
        mLastClickTime = time
        if (!checkNetwork()) {
            return
        }
        showLoginLoadingDialog(mActivity)
        LoginMgr.getInstance().fastLogin(mActivity)
        if (mLoginActionListener != null) {
            mLoginActionListener!!.onStartLogin()
            EventMgr.getInstance().notify(KernelEvent.EVENT_START_LOGIN, null)
        }
    }

    private fun clickWX(isOneKey: Boolean) {
        LogUtils.i(TAG, "click to login wechat")
        if (isUnCheckedAgree) {
            return
        }
        val time = System.currentTimeMillis()
        if (time - mLastClickTime < 1000) {
            LogUtils.d(TAG, "click too fast")
            return
        }
        mLastClickTime = time
        if (!mWXShareAPI!!.isWXInstalled) {
            showWeixinInstallDialog(mActivity)
            return
        }
        if (!checkNetwork()) {
            return
        }
        showLoginLoadingDialog(mActivity)
        if (mLoginActionListener != null) {
            mLoginActionListener!!.onStartLogin()
            EventMgr.getInstance().notify(KernelEvent.EVENT_START_LOGIN, null)
        }
        mWXShareAPI!!.auth()
        LoginMonitor.startLogin(LoginStatus.LoginType.LOGIN_WX, 0)

        reportBeacon(true,
                if (isOneKey) "wxoneclicklogin" else "wxloginbutton",
                mobile = mMobile,
                contentId = getContentId()
        )
    }

    private val mThirdLoginObserver: EventObserver<*> = object : EventObserver<Any?>(null) {
        override fun onEvent(eventName: String?, data: Any?) {
            if (KernelEvent.EVENT_THIRD_LOGIN_SUCC == eventName && data is Bundle) {
                dealCampusLogin(data)
            }
        }
    }

    private fun dealCampusLogin(bundle: Bundle) {
        val tinyId = bundle.getString("tinyId")
        val a2 = bundle.getString("a2")
        LoginMgr.getInstance().loginWns(tinyId, a2)
    }

    private fun checkNetwork(): Boolean {
        if (!NetworkUtil.isNetworkAvailable(mActivity)) {
            Tips.showShortToast(R.string.please_check_network)
            return false
        }
        return true
    }

    private fun getLoginType(): Int {
        return when (LoginStatus.getOriginLoginType()) {
            LoginStatus.LoginType.LOGIN_MOBILE -> TYPE_MOBILE
            LoginStatus.LoginType.LOGIN_QQ -> TYPE_QQ
            LoginStatus.LoginType.LOGIN_WX -> TYPE_WX
            else -> TYPE_MOBILE
        }
    }

    interface OnLoginActionListener {
        fun onStartLogin()
    }

    companion object {
        private const val TAG = "LoginCustomView"

        const val TYPE_NONE = 0
        const val TYPE_MOBILE = 1
        const val TYPE_QQ = 2
        const val TYPE_WX = 3

        // 绑定手机号
        const val TYPE_BIND_MOBILE = 4

        // 灰度首页
        const val TYPE_GRAY = 5

        // 验证码倒计时总时长
        private const val COUNTDOWN_VERIFY_TOTAL = 60000L

        // 语音验证码在发送文字验证码20s后显示
        private const val COUNTDOWN_UNRECEIVE_TOTAL = 20000L
        private const val COUNTDOWN_INTERVAL = 1000L
        private const val GET_MOBILE_TIMEOUT = 1500L
    }
}