package com.tencent.edu.module.login.nationdialog

import android.app.Activity
import android.graphics.Color
import android.graphics.Typeface
import android.view.View
import androidx.recyclerview.widget.LinearLayoutManager
import com.google.gson.reflect.TypeToken
import com.tencent.edu.R
import com.tencent.edu.common.utils.FileUtils
import com.tencent.edu.common.utils.LogUtils
import com.tencent.edu.common.utils.MiscUtils
import com.tencent.edu.common.utils.PixelUtil
import com.tencent.edu.commonview.dialogplus.DialogPlus
import com.tencent.edu.commonview.dialogplus.ViewHolder
import com.tencent.edu.module.login.nationdialog.indexlib.suspension.SuspensionDecoration
import kotlinx.android.synthetic.main.dialog_login_nation_code.view.*

/**
 * @author aidanyang
 * @date 2020-5-25
 */
class LoginNationCodeDialog(
        val activity: Activity
) {
    private var mAdapter: NationAdapter? = null

    private lateinit var mHeaderAdapter: HeaderRecyclerAndFooterWrapperAdapter
    private lateinit var mManager: LinearLayoutManager
    private lateinit var mDecoration: SuspensionDecoration

    private var mView: View = View.inflate(activity, R.layout.dialog_login_nation_code, null)
    private var mDialog: DialogPlus
    private val mData: ArrayList<NationCodeItem> = arrayListOf()

    private var mListener: NationCodeDialogListener? = null

    companion object {
        private const val TAG = "LoginNationCodeDialog"

        private const val PATH = "nation_code.json"
    }

    init {
        LogUtils.le(TAG, "init")
        mDialog = DialogPlus.Builder(activity)
                .setContentHolder(ViewHolder(mView))
                .setGravity(DialogPlus.Gravity.BOTTOM)
                .setContentBackgroundColorResourceId(R.color.transparent)
                .setCancelable(true)
                .setOnCancelListener {
                    mListener?.onCancel()
                }
                .create()
        initData()
        initViews()
    }

    private fun initData() {
        val type = object : TypeToken<ArrayList<NationCodeGroup>>() {}.type
        FileUtils.readFromJsonFile(activity, PATH, type)?.run {
            (this as ArrayList<NationCodeGroup>).forEach { group ->
                // 常用的放到顶部
                if (group.label == "常用") {
                    group.data.reversed().forEach {
                        it.top = true
                        it.baseIndexTag = group.label
                        mData.add(0, it)
                    }
                } else {
                    group.data.forEach {
                        it.baseIndexTag = group.label
                        mData.add(it)
                    }
                }
            }
        }
    }

    private fun initViews() {
        initRecyclerView()
        mView.nation_code_close?.setOnClickListener {
            mListener?.onCancel()
            dismiss()
        }
    }

    private fun initRecyclerView() {
        mManager = LinearLayoutManager(activity)
        mAdapter = NationAdapter(activity, mData).apply {
            setOnItemClickListener { v, item, index ->
                LogUtils.li(TAG, "clickItem index:$index,data:$item")
                mListener?.onSelected(item)
                dismiss()
            }
        }
        mHeaderAdapter = object : HeaderRecyclerAndFooterWrapperAdapter(mAdapter) {
            override fun onBindHeaderHolder(holder: NationCodeViewHolder?, headerPos: Int, layoutId: Int, o: Any?) {
                holder?.setText(R.id.nation_name_tv, (o as String?) ?: "")
            }
        }
        mDecoration = SuspensionDecoration(activity, mData)
                .setHeaderViewCount(mHeaderAdapter.headerViewCount)
                .setColorTitleBg(Color.WHITE)
                .setColorTitleFont(PixelUtil.dp2px(16f))
                .setTitleHeight(PixelUtil.dp2px(44f))
                .setTitleFontStyle(Typeface.DEFAULT_BOLD)
                .setColorTitleFont(Color.BLACK)

        mView.nation_code_rv.run {
            layoutManager = mManager
            adapter = mHeaderAdapter
            addItemDecoration(mDecoration)
        }

        mView.nation_code_index
                .setPressedShowTextView(mView.nation_code_index_hint) //设置HintTextView
                .setNeedRealIndex(true) //设置需要真实的索引
                .setLayoutManager(mManager) //设置RecyclerView的LayoutManager
                .setSourceData(mData) //设置数据
                .setHeaderViewCount(mHeaderAdapter.headerViewCount) //设置HeaderView数量
                .invalidate()
    }

    fun dismiss() {
        LogUtils.le(TAG, "dismiss")

        mDialog.dismiss()
    }

    fun show(): Boolean {
        LogUtils.le(TAG, "show")

        if (!MiscUtils.isActivityValid(activity) || mDialog.isShowing) {
            return false
        }

        mDialog.show()
        return true
    }

    fun setListener(listener: NationCodeDialogListener): LoginNationCodeDialog {
        mListener = listener
        return this
    }

    interface NationCodeDialogListener {
        fun onSelected(item: NationCodeItem)

        fun onCancel()
    }

}
