package com.tencent.edu.module.mobileverify

import android.app.Activity
import android.app.Dialog
import android.text.TextUtils
import com.tencent.edu.BuildConfig
import com.tencent.edu.R
import com.tencent.edu.common.callback.VoidCallback
import com.tencent.edu.common.core.ThreadMgr
import com.tencent.edu.common.permission.PermissionManager
import com.tencent.edu.common.permission.PermissionsDispatcher
import com.tencent.edu.common.utils.DeviceInfo
import com.tencent.edu.common.utils.LogUtils
import com.tencent.edu.common.utils.MiscUtils
import com.tencent.edu.common.utils.Tips
import com.tencent.edu.commonview.dialog.DialogUtil
import com.tencent.edu.kernel.AppRunTime
import com.tencent.edu.kernel.report.Report
import com.tencent.edu.module.report.AutoReportMgr
import com.tencent.edu.module.report.ReportConstant
import com.tencent.edu.module.shoppingfestival.WebDialogMgr
import com.tencent.edu.module.utils.ExtraUtils
import com.tencent.edu.utils.EduLog
import com.tencent.oedmobileverifyexample.OEDMobileVerify
import com.tencent.oedmobileverifyexample.OEDMobileVerify.GetLocalPhoneNumberCallback
import com.tencent.oedmobileverifyexample.OEDMobileVerifyMgr
import com.tencent.oedmobileverifyexample.log.IMVLogPrinter
import com.tencent.oedmobileverifyexample.log.MVLog
import org.json.JSONException
import org.json.JSONObject
import java.util.*

/**
 * @author ulexzhong. Created on 2019/9/18.
 */
class MobileVerifyCenter(private val mActivity: Activity) {
    private var mMobileVerifyDialog: MobileVerifyDialog? = null
    private var mCallback: IMobileVerifyCallback? = null

    @Volatile
    private var mIsVerifying = false
    private var mRunnable: Runnable? = null

    private var mCourseId: String? = null
    private var mAgencyId: String? = null

    constructor(activity: Activity, callback: IMobileVerifyCallback) : this(activity) {
        mCallback = callback
    }

    constructor(activity: Activity, courseId: String?, agencyId: String?, callback: IMobileVerifyCallback) : this(activity, callback) {
        mCourseId = courseId
        mAgencyId = agencyId
    }

    companion object {
        private const val TAG = "MobileVerifyCenter"
        private const val BUSINESS_ID = 600001

        private const val APP_KEY = "0S100FLLQQ1LTGSF" //从 AndroidManifest 获取
        private const val DNS_ID = "8908"
        private const val DNS_KEY = "mqJ118UJ"
        private const val DOMAIN = "ke.qq.com"
        private const val APP_ID = "edu"
        private const val VERIFY_LIMIT_TIME = 2000 //2s超时
        private const val ADDITIONAL = "ketang"
        const val SCENE = "edu_login"

        private var mPermissionManager = PermissionManager()
        private var mOedMobileInfo: OEDMobileVerify.OEDMobileInfo? = null
        private var mVerifyCodeCallback: OEDMobileVerifyMgr.VerifyCallback? = null

        init {
            val initInfo = OEDMobileVerifyMgr.OEDMobileInitInfo()
            initInfo.domain = DOMAIN
            initInfo.dnsKey = DNS_KEY
            initInfo.dnsId = DNS_ID
            initInfo.appKey = APP_KEY
            initInfo.appId = APP_ID
            initInfo.isDebug = BuildConfig.DEBUG
            initInfo.timeOut = VERIFY_LIMIT_TIME
            initInfo.businessId = BUSINESS_ID
            initInfo.deviceId = DeviceInfo.getDeviceId(AppRunTime.getApplicationContext())
            initInfo.additional = ADDITIONAL

            MVLog.setLogPrinter(object : IMVLogPrinter {
                override fun i(tag: String?, msg: String?): Int {
                    LogUtils.i(tag, msg)
                    return 0
                }

                override fun i(tag: String?, msg: String?, tr: Throwable?): Int {
                    LogUtils.i(tag, msg, tr)
                    return 0
                }

                override fun w(tag: String?, msg: String?): Int {
                    LogUtils.w(tag, msg)
                    return 0
                }

                override fun w(tag: String?, msg: String?, tr: Throwable?): Int {
                    LogUtils.w(tag, msg, tr)
                    return 0
                }

                override fun e(tag: String?, msg: String?): Int {
                    LogUtils.e(tag, msg)
                    return 0
                }

                override fun e(tag: String?, msg: String?, tr: Throwable?): Int {
                    LogUtils.e(tag, msg, tr)
                    return 0
                }

                override fun d(tag: String?, msg: String?): Int {
                    LogUtils.d(tag, msg)
                    return 0
                }

                override fun d(tag: String?, msg: String?, tr: Throwable?): Int {
                    LogUtils.d(tag, msg, tr)
                    return 0
                }

            })
            OEDMobileVerifyMgr.getInstance().initVerifyMobile(AppRunTime.getApplicationContext(), initInfo)

            LogUtils.li(TAG, "init OEDMobileVerifySDK")
        }

        //预获取手机信息
        @JvmStatic
        fun verifyPre() {
            EduLog.i(TAG, "verifyPre")
            OEDMobileVerify.verifyMobile(AppRunTime.getApplicationContext(), this.BUSINESS_ID, "", ADDITIONAL) { oedMobileInfo ->
                val code = oedMobileInfo.bizErrCode
                val resultJson = JSONObject()
                resultJson.put("phone_number", oedMobileInfo.mobile)
                resultJson.put("v_type", oedMobileInfo.vType)
                resultJson.put("carrier_type", oedMobileInfo.carrierType)
                resultJson.put("encrypted_phone", oedMobileInfo.encryptedPhone)
                resultJson.put("msg_id", oedMobileInfo.msgId)

                val result = resultJson.toString()
                EduLog.i(TAG, "verifyPre code:$code,result:$result")
                if (code == 0 && !TextUtils.isEmpty(result)) {
                    MobileVerifyPresenter.saveUserPhoneInfo(result)
                }
            }
        }

        @JvmStatic
        fun getLocalPhoneNumber(activity: Activity, callback: GetLocalPhoneNumberCallback) {
            if (mOedMobileInfo != null) {
                callback.onVerifyLocalPhoneNumber(mOedMobileInfo)
                return
            }

            mPermissionManager.unregisterGrantObserver()
            //Android6.0需要动态获取权限
            mPermissionManager.registerGrantObserver { requestCode: Int, _: Array<String?>?, grantResults: IntArray? ->
                if (requestCode == PermissionsDispatcher.REQUEST_GROUP_PHONE && PermissionsDispatcher.isGrant(grantResults)) {
                    getLocalPhoneNumber(callback)
                } else {
                    LogUtils.w(TAG, "getLocalPhoneNumber error:READ_PHONE_STATE not granted")
                    callback.onVerifyLocalPhoneNumber(null)
                }
            }
            mPermissionManager.checkPhonePermission(activity)
        }

        private fun getLocalPhoneNumber(callback: GetLocalPhoneNumberCallback) {
            LogUtils.i(TAG, "getLocalPhoneNumber start")
            OEDMobileVerifyMgr.getInstance().getLocalPhoneNumber(AppRunTime.getApplicationContext()) { oedMobileInfo: OEDMobileVerify.OEDMobileInfo ->
                if (oedMobileInfo.bizErrCode != 0) {
                    LogUtils.w(TAG, "getLocalPhoneNumber error:$oedMobileInfo")
                } else {
                    mOedMobileInfo = oedMobileInfo
                    LogUtils.i(TAG, "getLocalPhoneNumber success:$oedMobileInfo")
                }
                callback.onVerifyLocalPhoneNumber(oedMobileInfo)
            }
        }

        @JvmStatic
        fun requireSmsVerify(phoneNumber: String, nationCode: String, page: String?) {
            mLastVerifyTypeIsSms = true
            mLastVerifyPhoneNumber = phoneNumber
            mLastVerifyNationCode = nationCode
            mLastVerifyPage = page
            OEDMobileVerifyMgr.getInstance().requireSmsVerify(SCENE, nationCode.replace("+", ""), phoneNumber) {
                LogUtils.i(TAG, "requireSmsVerify:$it")
                getVerifyCodeCallback(page, phoneNumber, false).onVerifyProcess(it)
            }
        }

        @JvmStatic
        fun requireVoiceVerify(phoneNumber: String, nationCode: String, page: String?) {
            mLastVerifyTypeIsSms = false
            mLastVerifyPhoneNumber = phoneNumber
            mLastVerifyNationCode = nationCode
            mLastVerifyPage = page
            OEDMobileVerifyMgr.getInstance().requireVoiceVerify(SCENE, nationCode.replace("+", ""), phoneNumber) {
                LogUtils.i(TAG, "requireVoiceVerify:$it")
                getVerifyCodeCallback(page, phoneNumber, true).onVerifyProcess(it)
            }
        }

        @JvmStatic
        private var mLastVerifyTypeIsSms = true
        private var mLastVerifyPhoneNumber = ""
        private var mLastVerifyNationCode = ""
        private var mLastVerifyPage: String? = null

        @JvmStatic
        fun resendVerifyCode() {
            if (mLastVerifyPhoneNumber.isBlank() || mLastVerifyNationCode.isBlank()) {
                return
            }

            if (mLastVerifyTypeIsSms) {
                requireSmsVerify(mLastVerifyPhoneNumber, mLastVerifyNationCode, mLastVerifyPage)
            } else {
                requireVoiceVerify(mLastVerifyPhoneNumber, mLastVerifyNationCode, mLastVerifyPage)
            }
        }

        private fun getVerifyCodeCallback(page: String?, phoneNumber: String, isVoice: Boolean): OEDMobileVerifyMgr.VerifyCallback {
            if (mVerifyCodeCallback == null) {
                mVerifyCodeCallback = object : OEDMobileVerifyMgr.VerifyCallback {
                    override fun onVerifyProcess(oedMobileVerifyRsp: OEDMobileVerifyMgr.OEDMobileVerifyRsp?) {
                        // 页面已回收不再响应监听
                        if (!MiscUtils.isActivityValid(AppRunTime.getInstance().currentActivity) || oedMobileVerifyRsp == null) {
                            return
                        }

                        if (oedMobileVerifyRsp.bizErrCode == OEDMobileVerifyMgr.waterproofCode) {
                            // 最后的参数表示防水墙页面透明
                            val url = oedMobileVerifyRsp.waterproofWallUrl
                            if (!TextUtils.isEmpty(oedMobileVerifyRsp.waterproofWallUrl)) {
                                ThreadMgr.postToUIThread {
                                    //此时已在主线程中，更新UI
                                    WebDialogMgr.getInstance().showDialog(AppRunTime.getInstance().currentActivity, url, null)

                                    reportWaterExposure()
                                }
                            }

                            // 收起输入法
                            MiscUtils.hideSoftInput(AppRunTime.getInstance().currentActivity.window.decorView)
                        }
                    }

                    private fun reportWaterExposure() {
                        if (page == null) {
                            return
                        }

                        val reportExtraInfo = AutoReportMgr.getReportExtraInfo(AppRunTime.getInstance().currentActivity)
                                ?: return
                        reportExtraInfo.eventCode = ExtraUtils.EXTRA_REPORT_EXPOSURE_ACTION
                        reportExtraInfo.page = page
                        reportExtraInfo.module = "imagecode_" + if (isVoice) "voice" else "msg"

                        val customReportDatas: MutableMap<String, String> = HashMap()
                        reportExtraInfo.customDatas = customReportDatas
                        customReportDatas[ReportConstant.MOBILE] = phoneNumber
                        customReportDatas[ReportConstant.CONTENT_ID] = "msgcode"

                        Report.autoReportData(reportExtraInfo)
                    }
                }
            }

            return mVerifyCodeCallback!!
        }
    }

    @JvmOverloads
    fun verify(agreementType: Int, source: String, forbidSkip: Boolean = false) {
        //本地缓存校验
        val localUserPhoneInfo = MobileVerifyPresenter.getUserPhoneInfo()
        if (!TextUtils.isEmpty(localUserPhoneInfo)) {
            EduLog.i(TAG, "mobile verify use local data:$localUserPhoneInfo")
            showMobileVerifyDialog(localUserPhoneInfo, agreementType, source, forbidSkip)
            return
        }
        //超时降级逻辑
        ThreadMgr.postToUIThread(Runnable {
            if (mIsVerifying) {
                EduLog.i(TAG, "超2s降级到输入页面")
                if (BuildConfig.DEBUG) {
                    Tips.showShortToast("超2s降级到输入页面")
                }
                mRunnable = null
                mIsVerifying = false
                mCallback?.showWebPage(false)
                ThreadMgr.getInstance().uiThreadHandler.removeCallbacks(mRunnable)
            }
        }.also { mRunnable = it }, VERIFY_LIMIT_TIME.toLong())
        //sdk网络校验
        mIsVerifying = true
        MobileVerifyReport.mobileVerifyReq(BUSINESS_ID)
        OEDMobileVerify.verifyMobile(mActivity, BUSINESS_ID, "", ADDITIONAL, GetLocalPhoneNumberCallback { oedMobileInfo ->
            val code = oedMobileInfo.bizErrCode
            val resultJson = JSONObject()
            resultJson.put("phone_number", oedMobileInfo.mobile)
            resultJson.put("v_type", oedMobileInfo.vType)
            resultJson.put("carrier_type", oedMobileInfo.carrierType)
            val result = resultJson.toString()
            EduLog.i(TAG, "verify code:$code,result:$result")
            MobileVerifyReport.mobileVerifyResult(BUSINESS_ID, code, result)
            if (!mIsVerifying) {
                EduLog.i(TAG, "verify timeout,has show webPage")
                if (code == 0 && !TextUtils.isEmpty(result)) {
                    MobileVerifyPresenter.saveUserPhoneInfo(result)
                }
                return@GetLocalPhoneNumberCallback
            }
            mIsVerifying = false
            if (mRunnable != null) {
                ThreadMgr.getInstance().uiThreadHandler.removeCallbacks(mRunnable)
            }
            if (code == 0 && !TextUtils.isEmpty(result)) {
                ThreadMgr.postToUIThread {
                    showMobileVerifyDialog(result, agreementType, source, forbidSkip)
                    MobileVerifyPresenter.saveUserPhoneInfo(result)
                }
                return@GetLocalPhoneNumberCallback
            }
            EduLog.i(TAG, "校验失败code:$code,msg:$result")
            ThreadMgr.postToUIThread { mCallback?.showWebPage(false) }
        })
    }

    /**
     * 一键绑定
     */
    private fun showMobileVerifyDialog(result: String, agreementType: Int, source: String, forbidSkip: Boolean) {
        if (mMobileVerifyDialog != null && mMobileVerifyDialog!!.isShowing) {
            mMobileVerifyDialog!!.close()
        }
        mMobileVerifyDialog = MobileVerifyDialog(mActivity, result, mCourseId, mAgencyId, agreementType, source, forbidSkip, object : MobileVerifyDialog.ICallback {
            override fun onChangeNumber() {
                mCallback?.showWebPage(true)
            }

            override fun onIgnore() {
                mCallback?.onVerifyResult(false)
            }

            override fun onVerify() {
                verify(result)
            }

            override fun onCancel() {
                cancel()
            }
        })
        if (mCallback is IMobileVerifyWithCancelCallback) {
            (mCallback as IMobileVerifyWithCancelCallback).onDialogShow()
        }
        mMobileVerifyDialog!!.showDialog()
    }

    private fun cancel() {
        if (mCallback is IMobileVerifyWithCancelCallback) {
            (mCallback as IMobileVerifyWithCancelCallback).onCancel()
        }
    }

    private fun verify(result: String) {
        try {
            val json = JSONObject(result)
            val phone = json.optString("phone_number")
            val encryptPhone = json.optString("encrypted_phone")
            val msgId = json.optString("msg_id")
            MobileVerifyPresenter.request(encryptPhone, phone, msgId, object : VoidCallback {
                override fun onSucc() {
                    if (mMobileVerifyDialog != null) {
                        mMobileVerifyDialog!!.close()
                    }
                    mCallback?.onVerifyResult(true)
                }

                override fun onError(errorCode: Int, errorMsg: String) {
                    EduLog.i(TAG, "code:$errorCode,msg:$errorMsg")
                    if (mMobileVerifyDialog != null) {
                        mMobileVerifyDialog!!.close()
                    }
                    if (errorCode == 13002) { //手机号被其他账号绑定了
                        verifyFailHadBind()
                        return
                    }
                    cancel()
                    if (!TextUtils.isEmpty(errorMsg)) {
                        Tips.showShortToast("$errorMsg($errorCode)")
                    } else {
                        Tips.showShortToast(R.string.net_work_error)
                    }
                }
            })
        } catch (e: JSONException) {
            e.printStackTrace()
        }
    }

    private fun verifyFailHadBind() {
        try {
            val dialog: Dialog = DialogUtil.createOneBtnDialog(mActivity, "提示",
                    "此手机号已经被其他账号绑定。如遇到特殊问题，请联系客服QQ80082734",
                    "我知道了") { dialog, which ->
                dialog.dismiss()
                if (mMobileVerifyDialog != null) {
                    mMobileVerifyDialog!!.show()
                }
            }
            dialog.show()
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    interface IMobileVerifyCallback {
        /**
         * @param verify 校验结果
         */
        fun onVerifyResult(verify: Boolean)

        fun showWebPage(isChangeNumber: Boolean)
    }

    interface IMobileVerifyWithCancelCallback : IMobileVerifyCallback {
        fun onCancel()
        fun onDialogShow()
    }

}