package com.tencent.edu.module.setting

import android.app.Activity
import android.graphics.Color
import android.text.TextUtils
import android.view.View
import android.widget.CompoundButton
import android.widget.TextView
import android.widget.ToggleButton
import androidx.core.app.ActivityCompat
import com.tencent.edu.R
import com.tencent.edu.arm.player.ARMPlayer
import com.tencent.edu.common.callback.ResultCallback
import com.tencent.edu.common.core.ThreadMgr
import com.tencent.edu.common.event.EventMgr
import com.tencent.edu.common.event.EventObserver
import com.tencent.edu.common.misc.LocalUri
import com.tencent.edu.common.utils.SimpleBeaconReportUtil
import com.tencent.edu.common.utils.Tips
import com.tencent.edu.commonview.activity.BasePresenter
import com.tencent.edu.commonview.dialog.DialogUtil
import com.tencent.edu.download.storage.StorageDevice
import com.tencent.edu.flutter.EduFlutterChannel
import com.tencent.edu.framework.EduFramework
import com.tencent.edu.framework.settings.CacheManager
import com.tencent.edu.kernel.AppRunTime
import com.tencent.edu.kernel.KernelEvent
import com.tencent.edu.kernel.account.AccountLoginOrBindMgr
import com.tencent.edu.kernel.login.mgr.LoginMgr
import com.tencent.edu.kernel.mgr.AppUpdateMgr
import com.tencent.edu.kernel.report.Report
import com.tencent.edu.kernel.report.UserActionPathReport
import com.tencent.edu.module.log.CosMgr
import com.tencent.edu.module.log.LogMgr
import com.tencent.edu.module.login.LoadingDialog
import com.tencent.edu.module.offlinedownload.CourseDownloadManager
import com.tencent.edu.module.offlinedownload.DownloadMgrActivity
import com.tencent.edu.module.offlinedownload.DownloadStorageSelector
import com.tencent.edu.module.offlinedownload.widget.OfflineResolutionSelector
import com.tencent.edu.module.update.UpdatePromptMgr
import java.util.*

/**
 * Created by yibozhou on 2017/10/31.
 *
 */
class KSettingWrap(private var mView: IView) : BasePresenter<String> {

    override fun onResume() {
        switchDebugViewVisibility()
        updateOfflineDownloadView()
        initAccountAndSafetyView()
    }

    override fun init() {
        initViews()
    }

    override fun onPause() {

    }

    override fun onStop() {

    }

    override fun onDestroy() {
        EventMgr.getInstance().delEventObserver(KernelEvent.EVENT_UPDATE_GET_INFO, mGetUpDataInfo)
        EventMgr.getInstance().delEventObserver(KernelEvent.EVENT_LOGIN, mLoginStatusObserver)
        EventMgr.getInstance().delEventObserver(KernelEvent.EVENT_LOGOUT, mLoginStatusObserver)
        EventMgr.getInstance().delEventObserver(KernelEvent.EVENT_START_LOGIN, mLoginStatusObserver)
        EventMgr.getInstance().delEventObserver(SettingUtil.EVENT_OFFLINE_DEV_SETTING_CHANGE, downloadPathChangeEvt)
    }

    private var mAllowBackgroundPlayView: SettingItemView? = null
    private var mAllow23GDownloadView: SettingItemView? = null

    private var mAllowCourseReminderView: SettingItemView? = null
    private var mClearCacheView: SettingItemView? = null
    private var mRateAppView: SettingItemView? = null
    private var mSuggestionFeedbackView: SettingItemView? = null

    private var mUpgradeAppView: SettingItemView? = null
    private var mAboutAppView: SettingItemView? = null
    private var mDebugAppView: SettingItemView? = null
    private var mSwitchLoginView: SettingItemView? = null

    private var mOfflineSelectDev: SettingItemView? = null
    private var mOfflineSelectResolution: SettingItemView? = null
    private var mOfflineDownloadView: SettingItemView? = null
    private var mUploadLogView: SettingItemView? = null

    private var mEnableRecommendedView: SettingItemView? = null

    internal var hasFinished = false

    private val mLoginStatusObserver = object : EventSetObserver(null) {
        override fun onEvent(eventName: String?, data: Any?) {
            updateLoginText()
            initAccountAndSafetyView()
        }
    }

    private val mGetUpDataInfo = object : EventSetObserver(null) {
        override fun onEvent(eventName: String?, data: Any?) {
            refreshNewVersionInfo()
        }
    }

    private val downloadPathChangeEvt = object : EventSetObserver(null) {
        override fun onEvent(eventName: String?, data: Any?) {
            updateDevSettingView()
        }
    }

    private fun findView() {
        mAllow23GDownloadView = mView.getViewById(R.id.allow_23g_net_download) as SettingItemView?
        mAllowCourseReminderView = mView.getViewById(R.id.allow_course_app_reminder) as SettingItemView?
        mAllowBackgroundPlayView = mView.getViewById(R.id.allow_background_play) as SettingItemView?
        mClearCacheView = mView.getViewById(R.id.clear_cache) as SettingItemView?
        mRateAppView = mView.getViewById(R.id.rating_me) as SettingItemView?
        mSuggestionFeedbackView = mView.getViewById(R.id.suggestion_feedback) as SettingItemView?
        mUpgradeAppView = mView.getViewById(R.id.upgrade_app) as SettingItemView?
        mAboutAppView = mView.getViewById(R.id.about_app) as SettingItemView?
        mDebugAppView = mView.getViewById(R.id.debug_app) as SettingItemView?
        mSwitchLoginView = mView.getViewById(R.id.exit_login) as SettingItemView?
        mOfflineSelectDev = mView.getViewById(R.id.offline_select_dev) as SettingItemView?
        mOfflineSelectResolution = mView.getViewById(R.id.offline_select_resolution) as SettingItemView?
        mOfflineDownloadView = mView.getViewById(R.id.offline_download) as SettingItemView?
        mEnableRecommendedView = mView.getViewById(R.id.enable_recommended) as SettingItemView?

        mUploadLogView = mView.getViewById(R.id.upload_log) as SettingItemView?
    }

    private fun initViews() {

        findView()

        init23GDownloadSwitchView()

        initEnableRecommendedSwitchView()

        initPlayInBackgroundView()
        // 更新缓存大小.
        updateCacheSizeView()

        initCacheView()

        mDebugAppView?.setItemClickAction {
            DebugAppActivity.startDebugAppActivity(mView.getContext())
        }

        mRateAppView?.setItemClickAction {
            SettingUtil.rateAppInMarkets(mView.getContext())
        }
        // 评分逻辑.
        mRateAppView!!.setItemClickAction { SettingUtil.rateAppInMarkets(mView.getContext()) }

        // 离线下载
        mOfflineDownloadView!!.setItemClickAction { DownloadMgrActivity.startDownloadMgrActivity(mView.getContext()) }
        updateOfflineDownloadView()

        // 软件升级.
        mUpgradeAppView!!.setItemClickAction {
            UpdatePromptMgr.getInstance().checkVersion(true)
            Report.reportClick("setting_upgrade")
        }

        // 关于
        mAboutAppView!!.setItemClickAction {
            Report.reportClick("setting_about")
            UserActionPathReport.pushPath("about")
            LocalUri.jumpToEduUri("tencentedu://openpage/aboutapp")
        }

        initOfflineSelect()

        updateDevSettingView()
//        downloadPathChangeEvt = object : EventObserver(null) {
//            override fun onEvent(eventName: String?, data: Any?) {
//                updateDevSettingView()
//            }
//        }
        EventMgr.getInstance().addEventObserver(SettingUtil.EVENT_OFFLINE_DEV_SETTING_CHANGE, downloadPathChangeEvt)

        initOfflineSelectResolution()

        updateResolutionSettingView()

        // 判断更新.
        refreshNewVersionInfo()


        // 设置是否上课提醒
        initCourseReminderSwitchView()

        // 账号与安全
        initAccountAndSafetyView()

        // 退出登录.
        initLoginStatusView()

        // debug设置页面
        switchDebugViewVisibility()

        // 上传日志
        initUploadLog()

        EventMgr.getInstance().addEventObserver(KernelEvent.EVENT_UPDATE_GET_INFO, mGetUpDataInfo)
    }

    private fun initAccountAndSafetyView() {
        AccountLoginOrBindMgr.isAssGray(ResultCallback<Boolean> { result ->
            ThreadMgr.postToUIThread {
                mView.getViewById(R.id.account_and_safety)?.run {

                    // 判断是否有多个有资产的账户，如果有则显示，没有则不显示
                    visibility = if (result && LoginMgr.getInstance().isLogin) {
                        View.VISIBLE
                    } else {
                        View.GONE
                    }

                    (this as SettingItemView).setItemClickAction {
                        AccountLoginOrBindMgr.jump2AccountAndSafePage()


                    }
                }
            }
        })
    }

    private fun initOfflineSelectResolution() {
        mOfflineSelectResolution!!.setItemClickAction {
            val resolutionSelector = OfflineResolutionSelector()
            resolutionSelector.showSelector(mView.getContext(),
                    SettingUtil.getOfflineResolution()
            ) { resolutionType ->
                SettingUtil.saveOfflineResolution(resolutionType)
                updateResolutionSettingView()

                val info = HashMap<String, String>()
                info["definition"] = (resolutionType).toString()
                Report.reportCustomData("setting_downloadsharpness", true, -1, info, false)
            }
        }
    }

    private fun initOfflineSelect() {
        mOfflineSelectDev!!.setItemClickAction {
            val device = CourseDownloadManager.getInstance().currentStorageDevice
            val downloadStorageSelector = DownloadStorageSelector(mView.getContext())
            downloadStorageSelector.setTitle(R.string.offline_download_path_select)
            downloadStorageSelector.showSelector(device
            ) { selectorItemView ->
                val d = selectorItemView.data as StorageDevice
                if (!TextUtils.equals(device.storageId, d.storageId)) {
                    CourseDownloadManager.getInstance().switchStorage(d)
                    updateDevSettingView()
                    updateOfflineDownloadView()
                }
            }
        }
    }

    private fun initCacheView() {
        mClearCacheView!!.setItemClickAction {
            CacheManager.getInstance().clearCache()
            // 清空ARMPlayer m3u8缓存
            ARMPlayer.cleanPreload()
            // 清理缓存.
            UserActionPathReport.addAction("clean")
            SettingUtil.cleanAppCacheInBackground {
                Tips.showShortToast(R.string.clean_app_cache_success)

                // 再刷新一遍缓存大小,更新界面.
                updateCacheSizeView()
                Report.reportClick("setting_deletecache")
            }
        }
    }

    private fun init23GDownloadSwitchView() {
        val g23CacheSwitch = mAllow23GDownloadView!!.findViewById<ToggleButton>(
                R.id.setting_enable_switch)
        g23CacheSwitch.isChecked = !SettingUtil.getIsAllow23GDownloadSetting()
        g23CacheSwitch.setOnCheckedChangeListener { _, isChecked ->
            SettingUtil.saveIsAllow23GDownloadSetting(!isChecked)
            Report.reportClick(if (!isChecked) "setting_downloadonmobileneton" else "setting_downloadonmobilenetoff")
        }
    }

    private fun initEnableRecommendedSwitchView() {
        val enableRecommendedSwitch = mEnableRecommendedView?.findViewById(
                R.id.setting_enable_switch) as ToggleButton?

        enableRecommendedSwitch?.isChecked = SettingUtil.getIsEnableRecommendedSetting()

        enableRecommendedSwitch?.setOnCheckedChangeListener(object : CompoundButton.OnCheckedChangeListener {
            private var isCancelDialog = false
            override fun onCheckedChanged(buttonView: CompoundButton, isChecked: Boolean) {
                if (isCancelDialog) {
                    isCancelDialog = false
                    return
                }

                if (!isChecked) {
                    DialogUtil.createDialog(mView.getContext(), "",
                            "关闭后我们将无法向您提供个性化的课程、内容推荐，是否确认关闭？",
                            "确认关闭", "取消", { dialog, _ ->
                        SettingUtil.saveIsEnableRecommendedSetting(false)
                        callFlutterChannel(false)
                        enableRecommendedSwitch.isChecked = false
                        dialog.dismiss()
                    }, { dialog, _ ->
                        isCancelDialog = true
                        enableRecommendedSwitch.isChecked = true
                        dialog.dismiss()
                    }, { dialog ->
                        isCancelDialog = true
                        enableRecommendedSwitch.isChecked = true
                        dialog.dismiss()
                    }).setLeftBtnColor(ActivityCompat.getColor(mView.getContext(), R.color.gray))
                            .setRightBtnColor(ActivityCompat.getColor(mView.getContext(), R.color.red))
                            .show()
                    return
                }

                SettingUtil.saveIsEnableRecommendedSetting(true)
                callFlutterChannel(true)
                enableRecommendedSwitch.isChecked = true
            }
        })
    }

    private fun callFlutterChannel(enable: Boolean) {
        val params: MutableMap<String, Any> = HashMap()
        params["flag"] = if (enable) 1 else 0
        EduFlutterChannel.callDartWithModule("DebugSetting", "setPersonality", params)
    }

    private fun initCourseReminderSwitchView() {
        val courseReminderBtn = mAllowCourseReminderView!!.findViewById<ToggleButton>(
                R.id.setting_enable_switch)

        courseReminderBtn.isChecked = !SettingUtil.getIsForbiddenCourseReminderSetting()
        courseReminderBtn.setOnCheckedChangeListener { _, isChecked ->
            SettingUtil.saveIsForbiddenCourseReminderSetting(isChecked)
            Report.reportClick(if (isChecked) "setting_classalarmon" else "setting_classalarmoff")
        }
    }

    private fun initPlayInBackgroundView() {
        val backgroundPlaySwitch = mAllowBackgroundPlayView!!.findViewById<ToggleButton>(R.id.setting_enable_switch)
        backgroundPlaySwitch.isChecked = SettingUtil.isAllowBackgroundPlay()

        backgroundPlaySwitch.setOnCheckedChangeListener { _: CompoundButton, isChecked: Boolean ->
            SettingUtil.saveBackgroundPlaySetting(isChecked)
            val params: MutableMap<String, Any> = HashMap()
            params["flag"] = if (isChecked) 1 else 0
            EduFlutterChannel.callDartWithModule("DebugSetting", "setBackgroundPlay", params)
        }
    }

    private fun initLoginStatusView() {
        updateLoginText()
        mSwitchLoginView?.setItemClickAction {

            mView.setOnItemClick(R.id.exit_login)

        }

        // 添加登陆状态事件监听.
        EventMgr.getInstance().addEventObserver(
                KernelEvent.EVENT_LOGIN, mLoginStatusObserver)
        EventMgr.getInstance().addEventObserver(
                KernelEvent.EVENT_LOGOUT, mLoginStatusObserver)
        EventMgr.getInstance().addEventObserver(
                KernelEvent.EVENT_START_LOGIN, mLoginStatusObserver)
    }

    private fun switchDebugViewVisibility() {
        mDebugAppView!!.visibility = if (EduFramework.isDeveloperDebugging()) View.VISIBLE else View.GONE
    }


    private fun updateCacheSizeView() {
        val cacheText = mView.getViewById(R.id.clear_cache)!!
                .findViewById<TextView>(R.id.setting_cache_size_text)
        cacheText.text = SettingUtil.getCacheTextDescription()
    }

    private fun updateDevSettingView() {
        val devText = mView.getViewById(R.id.offline_select_dev)!!
                .findViewById<TextView>(R.id.setting_dev_name_text)
        devText.text = CourseDownloadManager.getInstance().currentStorageDevice.name
    }

    private fun updateResolutionSettingView() {
        val resolutionText = mView.getViewById(R.id.offline_select_resolution)!!.findViewById<TextView>(R.id.setting_dev_name_text)
        resolutionText.setText(SettingUtil.getOfflineResolution().definitionRes)
    }

    private fun updateOfflineDownloadView() {
        val downloadText = mView.getViewById(R.id.offline_download)!!.findViewById<TextView>(R.id.setting_download_size_text)
        downloadText.text = SettingUtil.getDownloadTextDescription()
    }

    private fun initUploadLog() {
        mUploadLogView!!.setRightDetailIconVisible(false)
        mUploadLogView!!.setOnClickListener {
            DialogUtil.createDialog(AppRunTime.getInstance().currentActivity, null,
                    "确定要上传日志？",
                    "取消",
                    "确定",
                    { dialog, _ -> dialog.dismiss() },
                    { dialog, _ ->
                        dialog.dismiss()
                        uploadLog()
                    }).setMsgMaxLines(1).show()
        }
    }

    private fun uploadLog() {
        if (!LoginMgr.getInstance().isLogin) {
            mView.showLoginDlg()

            return
        }
        hasFinished = false
        val dialog = LoadingDialog.createLoginWaitingDialog(mView.getContext() as Activity?)
        (dialog.findViewById<TextView>(R.id.tv_wording)).text = "上传中， 请稍候"
        dialog.show()

        LogMgr.getInstance().autoUploadLog(object : CosMgr.ICosCallback {
            override fun onSuccess() {
                if (hasFinished) {
                    return
                }
                dialog.dismiss()
                Tips.showToast("上传成功")
                hasFinished = true
            }

            override fun onProgress(complete: Long, total: Long) {

            }

            override fun onFailed(code: Int, msg: String?) {
                if (hasFinished) {
                    return
                }
                dialog.dismiss()
                Tips.showToast(String.format("上传失败:code=%d", code))
                hasFinished = true
            }
        })
//        LogMgr.getInstance().autoUploadLog(object : LogMgr.IUploadLogCallback {
//            override fun onProgress(totalSize: Long, progress: Long) {}
//
//            override fun onFinished() {
//                if (hasFinished) {
//                    return
//                }
//                dialog.dismiss()
//                Tips.showToast("上传成功")
//                hasFinished = true
//            }
//
//            override fun onFailed(code: Int, msg: String?) {
//                if (hasFinished) {
//                    return
//                }
//                dialog.dismiss()
//                Tips.showToast(String.format("上传失败:code=%d", code))
//                hasFinished = true
//            }
//        })
    }

    private fun refreshNewVersionInfo() {
        val hasNewVersionIcon = mUpgradeAppView!!.findViewById<View>(
                R.id.setting_about_new_version_icon)

        val res = AppUpdateMgr.getInstance().checkVersion()
        if ((res == AppUpdateMgr.UpdateResult.Res_Can_Update_later || res == AppUpdateMgr.UpdateResult.Res_Force_Udpate)) {
            hasNewVersionIcon.visibility = View.VISIBLE
        } else {
            hasNewVersionIcon.visibility = View.INVISIBLE
        }
    }

    private fun updateLoginText() {
        mView.getViewById(R.id.exit_login)?.findViewById<TextView>(R.id.setting_login_text)?.run {
            if (LoginMgr.getInstance().isLogin) {
                text = mView.getContext().getString(R.string.setting_quit_login)
                setTextColor(Color.parseColor("#F44336"))
            } else {
                text = mView.getContext().getString(R.string.setting_login)
                setTextColor(Color.parseColor("#23B8FF"))
            }
        }
    }

}