package com.tencent.edu.module.setting

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.view.View

import com.tencent.edu.R
import com.tencent.edu.common.misc.LocalUri
import com.tencent.edu.common.utils.LogUtils
import com.tencent.edu.commonview.activity.EduCompatActivity
import com.tencent.edu.commonview.dialog.DialogUtil
import com.tencent.edu.kernel.login.mgr.LoginMgr
import com.tencent.edu.kernel.report.Report
import com.tencent.edu.kernel.report.UserActionPathReport
import com.tencent.edu.module.login.LoginDialogWrapper

/**
 * 系统设置页面.
 *
 * @author arnozhang
 *
 * updated by zyb on 2017/11/2.
 *
 */
class KSystemSettingActivity : EduCompatActivity(), IView {

    private var mLoginDialogWrapper: LoginDialogWrapper? = null

    private var mIsOnClickedLogout = false
    private var mWrap: KSettingWrap? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_system_setting)

        setCommonActionBar()

        setActionBarTitle(R.string.settings)

        setOnBackClickListener { onGoBack() }

        mWrap = KSettingWrap(this)
        mWrap?.init()

        Report.reportExposed("setting_display", null, true)
    }

    override fun onDestroy() {
        mLoginDialogWrapper?.cancel()

        mWrap?.onDestroy()

        super.onDestroy()
    }

    override fun onResume() {
        super.onResume()
        mWrap?.onResume()

        UserActionPathReport.pushPath("setup")
    }

    override fun getViewById(id: Int): View? = findViewById(id)

    override fun getContext(): Context = this

    override fun setOnItemClick(id: Int) {
        when (id) {
            R.id.exit_login -> onLoginClick()
        }
    }

    override fun showLoginDlg() {
        if (mLoginDialogWrapper == null) {
            mLoginDialogWrapper = LoginDialogWrapper()
        }

        mLoginDialogWrapper?.show(this@KSystemSettingActivity)
    }

    private val HOMEPAGE = "tencentedu://openpage/homepage"

    private fun onGoBack() {
        val intent = intent
        val isFromHomepage = intent.getBooleanExtra("fromType", false)

        if (!isFromHomepage && mIsOnClickedLogout) {
            LocalUri.jumpToEduUri(HOMEPAGE)
            mIsOnClickedLogout = false
        }
        finish()
    }

    private fun onLoginClick() {
        if (!LoginMgr.getInstance().isLogin) {
            if (mLoginDialogWrapper == null) {
                mLoginDialogWrapper = LoginDialogWrapper()
            }
            mLoginDialogWrapper?.show(this@KSystemSettingActivity)

            Report.reportClick("setting_login")
        } else {
            Report.reportClick("setting_logout")
            val dlg = DialogUtil.createDialog(
                    this@KSystemSettingActivity, "",
                    getString(R.string.need_logout_confirm),
                    getString(R.string.cancel),
                    getString(R.string.logout_confirmed),
                    { dialog, _ -> dialog.dismiss() }, { dialog, _ ->
                LogUtils.i(TAG, "call logout for user intent")
                LoginMgr.getInstance().logout()
                dialog.dismiss()
            })

            dlg.show()
            mIsOnClickedLogout = true
        }
    }

    companion object {
        fun startSystemSettingActivity(
                context: Context, fromHomepage: Boolean) {
            val intent = Intent(context, KSystemSettingActivity::class.java)
            intent.putExtra("fromType", fromHomepage)
            context.startActivity(intent)
        }
    }
}
