/*
 * Copyright © 1998 - 2021 Tencent. All Rights Reserved.
 * 腾讯公司 版权所有
 */

package com.tencent.edu.module.setting

import android.os.Build
import android.os.Bundle
import android.os.Looper
import android.provider.Settings
import android.util.Log
import android.view.View
import android.widget.TextView
import com.tencent.edu.R
import com.tencent.edu.common.misc.WeakReference
import com.tencent.edu.common.permission.PermissionManager
import com.tencent.edu.common.permission.PermissionManager.GrantListener
import com.tencent.edu.common.permission.PermissionsDispatcher
import com.tencent.edu.common.utils.GpsUtil
import com.tencent.edu.common.utils.LogUtils
import com.tencent.edu.common.utils.ToastUtil
import com.tencent.edu.commonview.activity.BaseActivity
import com.tencent.edu.kernel.AppRunTime
import com.tencent.edu.utils.IEduListener
import com.tencent.map.geolocation.TencentLocation
import com.tencent.map.geolocation.TencentLocationListener
import com.tencent.map.geolocation.TencentLocationManager
import com.tencent.map.geolocation.TencentLocationRequest


/**
 * LBS定位.
 *
 * @author   wenjiewu
 * @date     2021/3/31
 */
class LbsLocationActivity : BaseActivity() {

    private val TAG: String = LbsLocationActivity::class.java.getSimpleName()

    var tvContent: TextView? = null
    var mLocationManager: TencentLocationManager? = null
    var mLocationListener: InnerLocationListener? = null;
    var mLocationPermissionManager: PermissionManager? = null


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_lbs_location)
        tvContent = findViewById<TextView>(R.id.tv_content)

        mLocationManager = AppRunTime.getInstance().applicationProxy.locationManager;
        mLocationListener = InnerLocationListener(WeakReference(this))
        mLocationPermissionManager = PermissionManager()


        if (Build.VERSION.SDK_INT >= 23) {
            // 动态申请权限
            requirePermission(object : IEduListener<String> {
                override fun onComplete(arg1: Int, data: String) {}
                override fun onError(code: Int, message: String) {}
            })
        }

        checkIfOpenGps()
    }

    private fun checkIfOpenGps() {
        if (!GpsUtil.isOPen(this)) {
            //跳转到设置页面让用户自己手动开启
            GpsUtil.jumpToOpenGpsService(this)
        }
    }

    //申请权限
    fun requirePermission(listener: IEduListener<String>?) {
        mLocationPermissionManager?.registerGrantObserver(GrantListener { requestCode, permissions, grantResults ->
            if (requestCode == PermissionsDispatcher.REQUEST_GROUP_LOCATION && PermissionsDispatcher.isGrant(grantResults)) {
                LogUtils.d(TAG, "location permission isGranted: true")

                // 检查是否开启GPS位置服务，不然位置信息也会获取失败
                if (GpsUtil.isOPen(this)) {
                    ToastUtil.shortToast(this, "授权ok")
                    listener?.onComplete(0, "授权ok")
                } else {
                    ToastUtil.shortToast(this, "请开启GPS位置服务")
                    GpsUtil.jumpToOpenGpsService(this)
                }
            }
        })
        mLocationPermissionManager?.checkLocationPermissions(this)
    }

    private fun judgeLocationServerState(): Boolean {
        try {
            return Settings.Secure.getInt(this.contentResolver, Settings.Secure.LOCATION_MODE) > 1
        } catch (e: Settings.SettingNotFoundException) {
            e.printStackTrace()
        }
        return false
    }

    fun startSingleLocation(view: View) {
        // 动态申请权限
        requirePermission(object : IEduListener<String> {
            override fun onError(code: Int, message: String) {
                Log.e(TAG, "授权失败")
            }

            override fun onComplete(arg1: Int, data: String?) {
                var request: TencentLocationRequest = TencentLocationRequest.create().setInterval(5 * 1000).setAllowGPS(true)
                        .setQQ("10001").setRequestLevel(3)// 设置定位level
                var re = mLocationManager?.requestSingleFreshLocation(request, mLocationListener, Looper.getMainLooper())
                Log.i(TAG, "re: $re")
            }

        })
    }

    fun clearAll(view: View) {
        tvContent?.text = ""
    }

    inner class InnerLocationListener : TencentLocationListener {
        private lateinit var mActivityWRF: WeakReference<LbsLocationActivity>

        constructor(activityWRF: WeakReference<LbsLocationActivity>?) {
            if (activityWRF != null) {
                mActivityWRF = activityWRF
            }
        }

        override fun onStatusUpdate(name: String?, status: Int, desc: String?) {
            Log.i(TAG, "name: " + name + "status: " + status + "desc: " + desc)
        }

        override fun onLocationChanged(location: TencentLocation?, error: Int, reson: String?) {
            val mainActivity: LbsLocationActivity? = mActivityWRF.get()
            if (mainActivity != null) {
                mainActivity.tvContent?.append(location.toString())
                mainActivity.tvContent?.append("\r\n")
            }
        }

    }

}