package com.tencent.edu.module.shortvideo.playerview

import android.content.Context
import android.graphics.BitmapFactory
import android.graphics.Color
import android.hardware.display.DisplayManager
import android.media.AudioManager
import android.os.Build
import android.provider.Settings
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.MotionEvent
import android.view.View
import android.widget.ImageView
import android.widget.ProgressBar
import android.widget.RelativeLayout
import android.widget.TextView
import com.tencent.edu.R
import com.tencent.edu.arm.player.ARMVideoFramePreview
import com.tencent.edu.common.core.ThreadMgr
import com.tencent.edu.common.utils.*
import com.tencent.edu.commonview.activity.BaseActivity
import com.tencent.edu.framework.EduFramework
import com.tencent.edu.kernel.report.Report
import com.tencent.edu.media.PlayerState
import com.tencent.edu.module.setting.SettingUtil
import com.tencent.edu.module.vodplayer.player.EduARMVideoView
import com.tencent.edu.module.vodplayer.widget.StepProgressBar
import com.tencent.edu.utils.EduLog
import com.tencent.edu.video.preview.EduVodPreview
import java.util.*

/**
 * @author hwajaywang.
 * Created on 2017/6/1.
 */
class ShortPlayerGestureView : RelativeLayout {
    private val One_Px = 1 // 最小响应滑动步进距离
    private var mContext: Context? = null
    private var mVolumeBrightnessLayout: View? = null
    private var mOperationBg: ImageView? = null
    private var mProgressBar: StepProgressBar? = null
    private var mPlayTimeLayout: View? = null
    private var mCurTime: TextView? = null
    private var mSlideDurationProgress: ProgressBar? = null
    var isProcessAdjusting = false
        private set
    private var isVolAdjusting = false
    private var isBriAdjusting = false
    private var mAudioManager: AudioManager? = null

    //最大调节进度
    private val mMaxProgress = 16

    //最大声音
    private var mMaxVolume = 0

    //当前声音
    private var mVolume = -1

    //当前亮度
    private var mBrightness = -1f
    private var mSlidePos: Long = -1
    private var mMediaPlayer: EduARMVideoView? = null
    private var isFullScreen = false
    private var mPreView: ImageView? = null
    private var mPreviewLayer: View? = null
    private var mDisplayListener: DisplayManager.DisplayListener? = null
    private var mGestureSeekListener: GestureSeekListener? = null

    constructor(context: Context) : super(context) {
        init(context)
    }

    constructor(context: Context, attrs: AttributeSet?) : super(context, attrs) {
        init(context)
    }

    constructor(context: Context, attrs: AttributeSet?, defStyleAttr: Int) : super(context, attrs, defStyleAttr) {
        init(context)
    }

    private fun init(context: Context) {
        mContext = context
        val inflater = LayoutInflater.from(context)
        inflater.inflate(R.layout.layout_player_gesture_view, this)
        mVolumeBrightnessLayout = findViewById(R.id.operation_volume_brightness)
        mOperationBg = findViewById<View>(R.id.operation_bg) as ImageView
        mProgressBar = findViewById<View>(R.id.progressbar) as StepProgressBar
        mPlayTimeLayout = findViewById(R.id.timecontainer)
        mCurTime = findViewById<View>(R.id.curtime) as TextView
        mSlideDurationProgress = findViewById(R.id.slide_duration_progress)
        mPreView = findViewById(R.id.preview)
        initData(context)
        initPreView(context)
    }

    fun setMediaPlayer(player: EduARMVideoView?) {
        mMediaPlayer = player
    }

    private fun initData(context: Context) {
        mAudioManager = context.getSystemService(Context.AUDIO_SERVICE) as AudioManager
        mMaxVolume = mAudioManager!!.getStreamMaxVolume(AudioManager.STREAM_MUSIC)
    }

    fun onVideoScaleChange(xOffset: Int, yOffset: Int, scale: Float) {
        if (mMediaPlayer != null) {
            mMediaPlayer!!.setVideoScale(xOffset, yOffset, scale)
        }
    }

    fun onVolumeSlide(percent: Float) {
        if (isProcessAdjusting) {
            return
        }
        isVolAdjusting = true
        if (mVolume == -1) {
            mVolume = mAudioManager!!.getStreamVolume(AudioManager.STREAM_MUSIC)
            if (mVolume < 0) {
                mVolume = 0
            }
        }
        var curVolume = (percent * mMaxVolume).toInt() + mVolume
        LogUtils.d(TAG, "onVolumeSlide:mVolume:%d, curVolume:%d, mMaxVolume:%d", mVolume, curVolume, mMaxVolume)
        curVolume = if (curVolume > mMaxVolume) mMaxVolume else if (curVolume < 0) 0 else curVolume
        //设置调整后音量
        mAudioManager!!.setStreamVolume(AudioManager.STREAM_MUSIC, curVolume, 0)

        //设置调整后进度
        val curProgress = if (mMaxVolume > 0) (curVolume.toFloat() / mMaxVolume * mMaxProgress).toInt() else 0
        mProgressBar!!.update(curProgress, mMaxProgress)
        mOperationBg!!.setImageResource(if (curVolume == 0) R.drawable.vol_none else R.drawable.vol_high)
        val params = mOperationBg!!.layoutParams as MarginLayoutParams
        params.topMargin = PixelUtil.dp2px(32f)
        mOperationBg!!.layoutParams = params
        mVolumeBrightnessLayout!!.visibility = View.VISIBLE
        val reporInfo: MutableMap<String, String> = HashMap()
        reporInfo["gesture"] = "volume"
        Report.reportExposed("gesture_fullscreen", reporInfo, false)
    }

    fun onBrightnessSlide(percent: Float) {
        if (isProcessAdjusting) {
            return
        }
        isBriAdjusting = true
        if (mBrightness < 0) {
            mBrightness = (mContext as BaseActivity?)!!.window.attributes.screenBrightness
            if (mBrightness == -1f) { //随系统
                try {
                    val brightness = Settings.System.getInt(mContext!!.contentResolver, Settings.System.SCREEN_BRIGHTNESS)
                    mBrightness = brightness * 1.0f / 255
                } catch (e: Settings.SettingNotFoundException) {
                    e.printStackTrace()
                } catch (e: SecurityException) {
                    e.printStackTrace()
                }
            }
            if (mBrightness <= 0.00f) {
                mBrightness = 0.50f
            } else if (mBrightness < 0.01f) {
                mBrightness = 0.01f
            }
        }
        val lpa = (mContext as BaseActivity?)!!.window.attributes
        lpa.screenBrightness = mBrightness + percent
        if (lpa.screenBrightness > 1.0f) {
            lpa.screenBrightness = 1.0f
        } else if (lpa.screenBrightness < 0.01f) {
            lpa.screenBrightness = 0.01f
        }
        (mContext as BaseActivity?)!!.window.attributes = lpa
        SettingUtil.setScreenBrightness(lpa.screenBrightness) //保存调整后亮度，保证整个app运行期间亮度一致
        mProgressBar!!.update((lpa.screenBrightness * mMaxProgress).toInt(), mMaxProgress)
        mOperationBg!!.setImageResource(R.drawable.bri_high)
        val params = mOperationBg!!.layoutParams as MarginLayoutParams
        params.topMargin = PixelUtil.dp2px(23f)
        mOperationBg!!.layoutParams = params
        mVolumeBrightnessLayout!!.visibility = View.VISIBLE
        val reporInfo: MutableMap<String, String> = HashMap()
        reporInfo["gesture"] = "brightness"
        Report.reportExposed("gesture_fullscreen", reporInfo, false)
    }

    fun onProgressSlide(distanceX: Float) {
        if (mMediaPlayer == null) {
            return
        }
        if (isBriAdjusting || isVolAdjusting) {
            return
        }
        isProcessAdjusting = true
        mPlayTimeLayout!!.visibility = View.VISIBLE
        var pos: Long = 0
        pos = if (mSlidePos == -1L) {
            mMediaPlayer!!.position
        } else {
            mSlidePos
        }
        val dur = mMediaPlayer!!.duration
        //单次步进500ms
        if (distanceX >= One_Px) { // 快退
            pos -= STEP_SCROLL_MS.toLong()
            if (pos < 0) {
                pos = 0
            }
        } else if (distanceX <= -One_Px) { // 快进
            pos += STEP_SCROLL_MS.toLong()
            if (pos > dur) {
                pos = dur
            }
        }
        if (dur / 3600000 > 0) {
            mCurTime!!.text = DateUtil.stringOfTime(pos / 1000)
        } else {
            mCurTime!!.text = DateUtil.StringOfTimeEx(pos / 1000)
        }
        setSlideProgressData(pos, dur)
        mSlidePos = pos
        if (mGestureCallback != null) {
            mGestureCallback!!.onGestureSlide()
        }
        if (isFullScreen) {
            showPreview(pos / 1000)
        } else {
            mPreView!!.visibility = View.GONE
        }
        setPreviewLayerVisibility(View.VISIBLE)
        LogUtils.d(TAG, "progress: %s", pos)
        if (mGestureSeekListener != null) {
            mGestureSeekListener!!.onGestureSeeking(pos)
            mGestureSeekListener!!.onGestureSeekState(true)
        }
    }

    private fun setSlideProgressData(pos: Long, dur: Long) {
        val durSec = (dur / 1000).toInt()
        val posSec = (pos / 1000).toInt()
        if (mSlideDurationProgress!!.max != durSec) {
            mSlideDurationProgress!!.max = durSec
        }
        mSlideDurationProgress!!.progress = posSec
        val width = mCurTime!!.width
        val layoutParams = mSlideDurationProgress!!.layoutParams
        if (layoutParams.width != width) {
            layoutParams.width = width
            mSlideDurationProgress!!.layoutParams = layoutParams
        }
    }

    /**
     * 手势结束
     */
    fun endGesture() {
        mVolume = -1
        mBrightness = -1f
        if (isProcessAdjusting) {
            if (mMediaPlayer != null && mSlidePos >= 0) {
                mMediaPlayer!!.seekTo(mSlidePos.toInt())
                mSlidePos = -1
            }
        }
        if (mGestureCallback != null) {
            mGestureCallback!!.onGestureEnd()
        }
        EduFramework.getUiHandler().removeCallbacks(mDismissRunnable)
        EduFramework.getUiHandler().postDelayed(mDismissRunnable, 500)
    }

    private val mDismissRunnable = Runnable {
        mVolumeBrightnessLayout!!.visibility = View.GONE
        mPlayTimeLayout!!.visibility = View.GONE
        setPreviewLayerVisibility(View.GONE)
        if (mGestureSeekListener != null) {
            mGestureSeekListener!!.onGestureSeekState(false)
        }
        if (isProcessAdjusting) {
            isProcessAdjusting = false
            val reporInfo: MutableMap<String, String> = HashMap()
            reporInfo["gesture"] = "seek"
            Report.reportExposed("gesture_fullscreen", reporInfo, false)
        }
        if (isBriAdjusting) {
            isBriAdjusting = false
            val reporInfo: MutableMap<String, String> = HashMap()
            reporInfo["gesture"] = "brightness"
            Report.reportExposed("gesture_fullscreen", reporInfo, false)
        }
        if (isVolAdjusting) {
            isVolAdjusting = false
            val reporInfo: MutableMap<String, String> = HashMap()
            reporInfo["gesture"] = "volume"
            Report.reportExposed("gesture_fullscreen", reporInfo, false)
        }
    }

    fun onDoubleTap(motionEvent: MotionEvent?) {
        if (ClickTooFastProtectUtil.isClickTooFast(ClickTooFastProtectUtil.KEY_PlayerGestureView, ClickTooFastProtectUtil.TIME_500_MS.toLong())) {
            LogUtils.d(TAG, "double tap too fast")
            return
        }
        if (mMediaPlayer != null) {
            if (mMediaPlayer!!.playerState == PlayerState.State_Running) {
                mMediaPlayer!!.pause()
                Tips.showShortToast(R.string.player_pause)
            } else {
                mMediaPlayer!!.start()
            }
        }
        val reporInfo: MutableMap<String, String> = HashMap()
        reporInfo["gesture"] = "pause"
        Report.reportExposed("gesture_fullscreen", reporInfo, false)
    }

    private var mGestureCallback: GestureCallback? = null
    fun setGestureCallback(gestureCallback: GestureCallback?) {
        mGestureCallback = gestureCallback
    }

    interface GestureCallback {
        fun onGestureEnd()
        fun onGestureSlide()
    }

    fun switchScreenOrientation(isFullScreen: Boolean) {
        LogUtils.d(TAG, "switchScreenOrientation %s", isFullScreen)
        this.isFullScreen = isFullScreen
        if (isFullScreen) {
            mSlideDurationProgress!!.visibility = View.GONE
            setPreViewVisibility()
        } else {
            mSlideDurationProgress!!.visibility = View.VISIBLE
        }
    }

    private fun setPreViewVisibility() {
        if (!EduVodPreview.getInstance().isEnable) {
            mPreView!!.visibility = View.GONE
            LogUtils.i(TAG, "preview visibility gone ")
        }
    }

    //拖动进度时
    fun setVideoPreviewVisible(show: Boolean, progress: Long) {
        if (show) {
            mPlayTimeLayout!!.visibility = View.VISIBLE
            val dur = mMediaPlayer!!.duration
            if (dur / 3600000 > 0) {
                mCurTime!!.text = DateUtil.stringOfTime(progress)
            } else {
                mCurTime!!.text = DateUtil.StringOfTimeEx(progress)
            }
            setPreviewLayerVisibility(View.VISIBLE)
            showPreview(progress)
        } else {
            mPlayTimeLayout!!.visibility = View.GONE
            setPreviewLayerVisibility(View.GONE)
        }
        if (mGestureSeekListener != null) {
            mGestureSeekListener!!.onGestureSeekState(show)
        }
    }

    private fun showPreview(progress: Long) {
        if (!EduVodPreview.getInstance().isEnable) {
            return
        }
        mPreView!!.visibility = View.VISIBLE
        mPreView!!.setBackgroundColor(Color.BLACK)
        val timestamp = progress * 1000000 //s -> us
        LogUtils.d(TAG, "show preview start %s", timestamp)
        EduVodPreview.getInstance().seekLoadFrame(timestamp)
    }

    fun setPreviewLayer(previewLayer: View?) {
        mPreviewLayer = previewLayer
    }

    private fun setPreviewLayerVisibility(visibility: Int) {
        if (mPreviewLayer == null) {
            return
        }
        if (isFullScreen && EduVodPreview.getInstance().isEnable) {
            mPreviewLayer?.visibility = visibility
        } else {
            mPreviewLayer?.visibility = visibility
        }
    }

    private fun initPreView(context: Context) {
        val dbPath = FileUtils.getStorageDirectory() + "/preview"
        EduVodPreview.getInstance().setCachePath(dbPath)
        EduVodPreview.getInstance().setPreViewSize(context, 120, 90) //dp
        EduVodPreview.getInstance().setCacheSize(20971520) //20MB
        EduVodPreview.getInstance().setFrameCallback(object : ARMVideoFramePreview.Callback {
            override fun onSuccess(bytes: ByteArray) {
                ThreadMgr.postToUIThread {
                    mPreView!!.visibility = View.VISIBLE
                    val bitmap = BitmapFactory.decodeByteArray(bytes, 0, bytes.size)
                    mPreView!!.setImageBitmap(bitmap)
                    LogUtils.d(TAG, "show preview done %s")
                }
            }

            override fun onFailure() {
                EduLog.d(TAG, "preview onFailure")
            }
        })
        registerDisplayListener()
    }

    //180 翻转屏幕时，设置不可见
    private fun registerDisplayListener() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR1) {
            mDisplayListener = object : DisplayManager.DisplayListener {
                override fun onDisplayAdded(displayId: Int) {}
                override fun onDisplayChanged(displayId: Int) {
                    ThreadMgr.postToUIThread({
                        mPlayTimeLayout!!.visibility = View.GONE
                        mPreviewLayer?.visibility = View.GONE
                        mVolumeBrightnessLayout!!.visibility = View.GONE
                        //seek 后旋转，这里需要更新视频进度
                        if (isProcessAdjusting) {
                            if (mMediaPlayer != null && mSlidePos >= 0) {
                                mMediaPlayer!!.seekTo(mSlidePos.toInt())
                                mSlidePos = -1
                            }
                        }
                    }, 1000)
                    if (mGestureSeekListener != null) {
                        mGestureSeekListener!!.onGestureSeekState(false)
                    }
                }

                override fun onDisplayRemoved(displayId: Int) {}
            }
            val displayManager = mContext!!.getSystemService(Context.DISPLAY_SERVICE) as DisplayManager
            displayManager.registerDisplayListener(mDisplayListener, ThreadMgr.getInstance().uiThreadHandler)
        }
    }

    private fun unRegisterDisplayListener() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR1) {
            if (mDisplayListener != null) {
                val displayManager = mContext!!.getSystemService(Context.DISPLAY_SERVICE) as DisplayManager
                displayManager.unregisterDisplayListener(mDisplayListener)
            }
        }
    }

    fun unInit() {
        unRegisterDisplayListener()
    }

    interface GestureSeekListener {
        fun onGestureSeeking(progress: Long)
        fun onGestureSeekState(seeking: Boolean)
    }

    fun setGestureSeekListener(gestureSeekListener: GestureSeekListener?) {
        mGestureSeekListener = gestureSeekListener
    }

    companion object {
        private const val TAG = "edu_PlayerGestureView"
        private const val STEP_SCROLL_MS = 500.0 // 滑动单次步进
    }
}