package com.tencent.edu.module.shortvideo.playerview

import android.app.Activity
import android.content.Context
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.RectF
import android.graphics.drawable.BitmapDrawable
import android.graphics.drawable.Drawable
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.view.View.OnClickListener
import android.view.ViewGroup
import android.view.WindowManager
import android.view.animation.AlphaAnimation
import android.widget.LinearLayout
import android.widget.SeekBar
import androidx.vectordrawable.graphics.drawable.Animatable2Compat
import com.github.penfeizhou.animation.apng.APNGDrawable
import com.tencent.edu.R
import com.tencent.edu.common.core.ThreadMgr
import com.tencent.edu.common.event.EventMgr
import com.tencent.edu.common.utils.ClickTooFastProtectUtil
import com.tencent.edu.common.utils.DateUtil
import com.tencent.edu.common.utils.LogUtils
import com.tencent.edu.common.utils.MiscUtils
import com.tencent.edu.dlna.DLNAGlobalConfig
import com.tencent.edu.framework.EduFramework
import com.tencent.edu.kernel.KernelEvent
import com.tencent.edu.media.PlayerState
import com.tencent.edu.module.setting.SettingUtil
import com.tencent.edu.module.shortvideo.ShortVideoRequester
import com.tencent.edu.module.shortvideo.ShortVideoRequester.UpdateLiveStatusListener
import com.tencent.edu.module.shortvideo.bean.VideoBean
import com.tencent.edu.module.shortvideo.event.ShortVideoEvent
import com.tencent.edu.module.shortvideo.report.ShortVideoReport
import com.tencent.edu.module.shortvideo.util.ShortVideoUtil
import com.tencent.edu.module.vodplayer.event.EventCenter
import com.tencent.edu.module.vodplayer.event.EventObserver
import com.tencent.edu.module.vodplayer.event.PlayEventDef
import com.tencent.edu.module.vodplayer.player.EduARMVideoView
import com.tencent.edu.module.vodplayer.player.SpeedRatioType
import com.tencent.edu.module.vodplayer.report.VodDurationReport
import com.tencent.edu.module.vodplayer.widget.ISpeedChangeListener
import com.tencent.edu.module.vodplayer.widget.speedratio.SpeedRatioSelector
import kotlinx.android.synthetic.main.view_shortvideo_playcontrol_landscape.view.*

class ShortVideoLandscapeControllerView(context: Context, attrs: AttributeSet?) : LinearLayout(context, attrs) {

    private var mVideoDetail: VideoBean? = null
    private var mContext: Context = context
    private var mMediaPlayer: EduARMVideoView? = null
    private var mSpeedRatioSelector: SpeedRatioSelector? = null
    var isOnSeeking = false
        private set
    private var mIsOnTracking = false
    private var mPlayBtnRes = 0
    private var mPauseBtnRes = 0
    private var mLayoutRes = 0
    private var mLimitDuration = 0L
    private var mIsGestureSeeking = false
    private var mSpeedChangeListener: ISpeedChangeListener? = null
    private var mIsAutoHideCallback: ((Boolean) -> Unit)? = null

    // 喜欢
    private var mIsFavorite = false
    private var mFavoriteCount = 0
    private var mIsFavExec = false

    // 收藏
    private var mIsCollect = false
    private var mCollectCount = 0
    private var mIsCollExec = false

    private val mShortBusObserver: com.tencent.edu.common.event.EventObserver<*> = object : com.tencent.edu.common.event.EventObserver<Any?>(null) {
        override fun onEvent(eventName: String, data: Any?) {
            if (data !is ShortVideoEvent
                    || mVideoDetail == null || mVideoDetail!!.fileId != data.mFileId) {
                return
            }

            if (ShortVideoEvent.EVENT_UPDATE_COLLECT == eventName) {
                mVideoDetail!!.isCollect = data.mIsCollect
                mVideoDetail!!.collectNum = data.mCollectNum
                initCollect(data.mIsCollect, data.mCollectNum)
            } else {
                mVideoDetail!!.isLike = data.mIsLike
                mVideoDetail!!.likeNum = data.mLikeNum
                initFavorite(data.mIsLike, data.mLikeNum)
            }
        }
    }

    init {
        mPlayBtnRes = R.drawable.ic_play
        mPauseBtnRes = R.drawable.ic_pause
        mLayoutRes = R.layout.view_shortvideo_playcontrol_landscape
        init(mLayoutRes)
        initEvent()
    }

    fun setMediaPlayer(player: EduARMVideoView) {
        mMediaPlayer = player

        // 获取播放速度并设置
        val radio = SpeedRatioType.ratioOf(player.playSpeedRatio)
        setRadio(radio)
        onPlayStateChanged(player.playerState, null)
    }

    private fun setRadio(radio: SpeedRatioType) {
        // 初始化速度
        setCurSpeedType(radio)
        setPlaySpeed(radio.typeName)
    }

    fun uninit() {
        EventCenter.getInstance().delObserver(this)
        ThreadMgr.getInstance().uiThreadHandler.removeCallbacks(mTimer)

        EventMgr.getInstance().delEventObserver(ShortVideoEvent.EVENT_UPDATE_COLLECT, mShortBusObserver)
        EventMgr.getInstance().delEventObserver(ShortVideoEvent.EVENT_UPDATE_LIKE, mShortBusObserver)
    }

    fun init(layoutResID: Int) {
        LayoutInflater.from(mContext).inflate(layoutResID, this, true)
        play.setBackgroundResource(mPlayBtnRes)
        seekBar.isEnabled = false

        speed_radio_layout.setOnClickListener {
            mIsAutoHideCallback?.invoke(false)
            showSpeedOptWithAnimation(true)
        }
        fullscreen_layout.setOnClickListener { MiscUtils.switchOrientation(false, mContext as Activity, false) }
        favorite_layout.setOnClickListener {
            mIsAutoHideCallback?.invoke(true)
            setFavorite(!mIsFavorite)
        }
        collect_layout.setOnClickListener {
            mIsAutoHideCallback?.invoke(true)
            setCollect(!mIsCollect)
        }

        mSpeedRatioSelector = SpeedRatioSelector(speed_radio)
        initPlayControlListener()
        EventCenter.getInstance().addObserver(this)
    }

    /**
     * 第一次点击收藏，展示tips
     */
    private fun showTips() {
        if (SettingUtil.isShowShortVideoPlayTips()) {
            SettingUtil.saveIsShowShortVideoPlayTips(false)

            play_collect_tips.visibility = View.VISIBLE
            play_collect_tips.setTips(R.string.svideo_tips, R.drawable.bg_short_video_tips_down_bubble)
            play_collect_tips.setOnClickListener {
                mIsAutoHideCallback?.invoke(true)
                hideTips()
            }
        }
    }

    fun hideTips() {
        play_collect_tips.visibility = View.GONE
    }

    private fun initEvent() {
        EventMgr.getInstance().addEventObserver(ShortVideoEvent.EVENT_UPDATE_COLLECT, mShortBusObserver)
        EventMgr.getInstance().addEventObserver(ShortVideoEvent.EVENT_UPDATE_LIKE, mShortBusObserver)
    }

    @EventObserver(event = PlayEventDef.EVENT_PLAY_STATE_CHANGED)
    fun onPlayStateChanged(state: PlayerState, taskId: String?) {
        when (state) {
            PlayerState.State_Running -> {
                setPlayWithContentDescription(true)
                play.setBackgroundResource(mPauseBtnRes)
                play.contentDescription
                seekBar.isEnabled = true
                val duration = mMediaPlayer!!.duration
                LogUtils.i("PlayControlView", "mMediaPlayer.getDuration():$duration")
                all_time.text = DateUtil.stringOfTime(duration / 1000)
                seekBar.max = duration.toInt() / 1000
                val pos = mMediaPlayer!!.position
                seekBar.progress = pos.toInt() / 1000
                refreshSeekBar()
                val activity = EduFramework.getAppLifeMonitor().currentActivity
                activity?.window?.addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON)
            }
            PlayerState.State_Stopped -> {
                setPlayWithContentDescription(false)
                play.setBackgroundResource(mPlayBtnRes)
                seekBar.isEnabled = false
                val activity = EduFramework.getAppLifeMonitor().currentActivity
                activity?.window?.clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON)
            }
            else -> {
                setPlayWithContentDescription(false)
                play.setBackgroundResource(mPlayBtnRes)
            }
        }
    }

    private fun refreshSeekBar() {
        ThreadMgr.postToUIThread(mTimer, 500)
    }

    fun setLimitDuration(limitDuration: Long) {
        mLimitDuration = limitDuration
    }

    private val mTimer: Runnable = object : Runnable {
        override fun run() {
            if (mMediaPlayer == null) {
                return
            }
            if (!isOnSeeking) {
                if (DLNAGlobalConfig.getInstance().isEnableDLNA) {
                    val duation = mMediaPlayer!!.duration
                    all_time.text = DateUtil.stringOfTime(duation / 1000)
                    seekBar.max = duation.toInt() / 1000
                }
                var pos = mMediaPlayer!!.position
                if (0L == mMediaPlayer!!.duration) {
                    pos = 0
                }
                if (shouldStopPlayer(pos)) {
                    return
                }
                //这里跟 iOS 对齐，手势seek的时候，seekBar和CurPos也跟着改变，所以先屏蔽视频播放引起的进度改变
                if (!mIsGestureSeeking) {
                    seekBar.progress = pos.toInt() / 1000
                }
                if (mMediaPlayer!!.playerState == PlayerState.State_Running) {
                    ThreadMgr.postToUIThread(this, 500)
                }
            } else {
                ThreadMgr.postToUIThread(this, 500)
            }
        }
    }

    private fun shouldStopPlayer(pos: Long): Boolean {
        if (mLimitDuration in 1..pos) {
            mMediaPlayer!!.stop()
            EventMgr.getInstance().notify(KernelEvent.EVENT_VIDEO_STOPPED, null)
            return true
        }
        return false
    }

    private fun initPlayControlListener() {
        seekBar.setOnSeekBarChangeListener(object : SeekBar.OnSeekBarChangeListener {
            override fun onProgressChanged(arg0: SeekBar, progress: Int,
                                           arg2: Boolean) {
                pos_time.text = DateUtil.stringOfTime(progress.toLong())
                if (mIsOnTracking) {
                    mIsAutoHideCallback?.invoke(false)
                }
            }

            override fun onStartTrackingTouch(arg0: SeekBar) {
                isOnSeeking = true
                mIsOnTracking = true
                VodDurationReport.setSeekStartTime(seekBar.progress)
            }

            override fun onStopTrackingTouch(arg0: SeekBar) {
                VodDurationReport.setSeekEndTime(seekBar.progress)
                if (mMediaPlayer != null) {
                    var pos = arg0.progress * 1000
                    if (shouldStopPlayer(pos.toLong())) {
                        return
                    }
                    //临时解决 seek 到结尾 不会重新播放问题，由于 ArmPlay 做了优化，目前先这样处理
                    val duation = mMediaPlayer!!.duration
                    if (pos >= duation) {
                        //确保不能seek到结尾
                        pos -= 100
                        LogUtils.i("PlayControlView", "onStopTrackingTouch total duration $duation pos = $pos")
                    }
                    mMediaPlayer?.seekTo(pos)
                }
                mIsOnTracking = false
                isOnSeeking = false
                mIsAutoHideCallback?.invoke(true)
            }
        })
        val listener = OnClickListener {
            mIsAutoHideCallback?.invoke(true)
            val run = Runnable {
                if (ClickTooFastProtectUtil.isClickTooFast(ClickTooFastProtectUtil.KEY_PlayControlView, ClickTooFastProtectUtil.TIME_500_MS.toLong())) {
                    LogUtils.d(TAG, "click playBtn too fast")
                    return@Runnable
                }
                if (mMediaPlayer == null) {
                    return@Runnable
                }
                when (mMediaPlayer!!.playerState) {
                    PlayerState.State_Running -> {
                        mMediaPlayer!!.pause()
                    }
                    PlayerState.State_Stopped -> {
                        mMediaPlayer!!.play(mMediaPlayer!!.mediaInfoPacket)
                    }
                    else -> {
                        mMediaPlayer!!.start()
                    }
                }
            }
            ThreadMgr.getInstance().uiThreadHandler.post(run)
        }
        val speedSelectListener = OnClickListener { v: View ->
            mIsAutoHideCallback?.invoke(true)
            MiscUtils.hideSystemUI((context as Activity).window.decorView)
            mSpeedRatioSelector!!.dismiss()
            when (v.id) {
                R.id.speed_ratio0_8 -> onSpeedChange(SpeedRatioType.SPEED0_8)
                R.id.speed_ratio1_0 -> onSpeedChange(SpeedRatioType.SPEED1_0)
                R.id.speed_ratio1_25 -> onSpeedChange(SpeedRatioType.SPEED1_25)
                R.id.speed_ratio1_5 -> onSpeedChange(SpeedRatioType.SPEED1_5)
                R.id.speed_ratio2_0 -> onSpeedChange(SpeedRatioType.SPEED2_0)
            }
        }

        mSpeedRatioSelector!!.setOnDismissListener {
            mIsAutoHideCallback?.invoke(true)
            MiscUtils.hideSystemUI((context as Activity).window.decorView)
        }

        mSpeedRatioSelector!!.setOnClickListener(speedSelectListener)
        play.setOnClickListener(listener)
        play_wraper.setOnClickListener(listener)
    }

    private fun onSpeedChange(ratioType: SpeedRatioType) {
        if (mSpeedChangeListener != null) {
            mSpeedChangeListener!!.onSelect(ratioType)
        }
        if (mMediaPlayer != null) {
            mMediaPlayer!!.playSpeedRatio = ratioType.ratio
        }
        setRadio(ratioType)
        setPlaySpeed(ratioType.typeName)
    }

    fun getNewDrawable(restId: Int, dstWidth: Int, dstHeight: Int): BitmapDrawable? {
        try {
            val bitmap = BitmapFactory.decodeResource(
                    mContext.resources, restId)
            val bmp = Bitmap.createScaledBitmap(bitmap, dstWidth, dstHeight, true)
            val d = BitmapDrawable(bmp)
            d.setTargetDensity(mContext.resources.displayMetrics)
            return d
        } catch (e: OutOfMemoryError) {
        }
        return null
    }

    fun setPlaySpeedViewClickListener(playSpeedViewClickListener: OnClickListener?) {
        speed_radio_layout.setOnClickListener(playSpeedViewClickListener)
    }

    fun setPlaySpeed(text: String) {
        speed_radio.text = text
        speed_radio.contentDescription = SpeedRatioType.ratioOf(mMediaPlayer!!.playSpeedRatio).contentDescription
    }

    fun setPlaySpeed(playSpeedRatio: Float) {
        setPlaySpeed(SpeedRatioType.ratioOf(playSpeedRatio).typeName)
    }

    fun showPlaySpeed(show: Boolean) {
        speed_radio_layout.visibility = if (show) View.VISIBLE else View.GONE
    }

    fun showFullScreenBtn(show: Boolean) {
        fullscreen_layout.visibility = if (show) View.VISIBLE else View.GONE
    }

    fun show(show: Boolean) {
        clearAnimation()
        visibility = if (show) View.VISIBLE else View.GONE
        if (!show) {
            hideTips()
        }
    }

    fun showWithAnimation(show: Boolean, animation: AlphaAnimation?) {
        // 防止隐藏后再隐藏，控制面板闪一下
        if (visibility == (if (show) View.VISIBLE else View.GONE)) {
            return
        }
        show(show)
        startAnimation(animation)
    }

    fun initFavorite(favorite: Boolean, count: Int) {
        setLikeWithContentDescription(favorite)
        mFavoriteCount = count

        showFavAnim(favorite)
        setFavoriteCount(count)
    }

    private fun setFavoriteCount(count: Int) {
        favorite_tv.text = ShortVideoUtil.getCount(count)
        favorite_tv.contentDescription = "${favorite_tv.text}人点赞"
    }

    private fun showFavAnim(favorite: Boolean) {
        val drawable = if (favorite) APNGDrawable.fromResource(mContext, R.raw.ic_favorite_y_apng) else APNGDrawable.fromResource(mContext, R.raw.ic_favorite_n_apng)
        drawable.setLoopLimit(1)
        drawable.registerAnimationCallback(object : Animatable2Compat.AnimationCallback() {
            override fun onAnimationEnd(drawable: Drawable) {
                favorite_iv_dynamic.visibility = View.GONE
                favorite_iv.visibility = View.VISIBLE
                setFavoriteRes(favorite)
            }
        })
        favorite_iv_dynamic.setImageDrawable(drawable)
        favorite_iv.visibility = View.GONE
        favorite_iv_dynamic.visibility = View.VISIBLE
    }

    private fun setFavoriteRes(favorite: Boolean) {
        favorite_iv.setImageResource(if (favorite) R.drawable.ic_favorite_y else R.drawable.ic_favorite_n)
    }

    fun initCollect(collect: Boolean, count: Int) {
        setCollectedWithContentDescription(collect)
        mCollectCount = count
        setCollectRes(collect)
        setCollectCount(count)
    }

    fun setFavorite(favorite: Boolean) {
        if (mIsFavExec || mIsFavorite == favorite || mVideoDetail == null || mVideoDetail!!.fileId == null) {
            return
        }
        mIsFavExec = true

        val fileId = mVideoDetail!!.fileId
        ShortVideoRequester.setFavStatus(fileId,
                1,
                if (favorite) 1 else 2,
                object : UpdateLiveStatusListener {
                    override fun onSuccess() {
                        mIsFavExec = false
                        mIsFavorite = favorite
                        if (favorite) {
                            ShortVideoReport.reportVideoFavorite(mContext, fileId)
                            ++mFavoriteCount
                        } else {
                            ShortVideoReport.reportVideoCancelFavorite(mContext, fileId)
                            --mFavoriteCount
                        }

                        EventMgr.getInstance().notify(ShortVideoEvent.EVENT_UPDATE_LIKE
                                , ShortVideoEvent(
                                fileId,
                                mIsLike = mIsFavorite,
                                mLikeNum = mFavoriteCount
                        ))
                    }

                    override fun onError() {
                        mIsFavExec = false
                    }
                })
    }

    fun setCollect(collect: Boolean) {
        if (mIsCollExec || mIsCollect == collect || mVideoDetail == null || mVideoDetail!!.fileId == null) {
            return
        }
        mIsCollExec = true

        val fileId = mVideoDetail!!.fileId
        val retType = mVideoDetail!!.retType
        ShortVideoRequester.setCollectStatus(
                fileId,
                retType,
                if (collect) 1 else 2,
                object : ShortVideoRequester.UpdateCollectStatusListener {
                    override fun onSuccess() {
                        mIsCollExec = false
                        mIsCollect = collect

                        if (collect) {
                            showTips()
                            mCollectCount++
                            ShortVideoReport.reportVideoCollect(mContext, fileId)
                        } else {
                            mCollectCount--
                            ShortVideoReport.reportVideoCancelCollect(mContext, fileId)
                        }

                        EventMgr.getInstance().notify(ShortVideoEvent.EVENT_UPDATE_COLLECT
                                , ShortVideoEvent(
                                fileId,
                                mIsCollect,
                                mCollectCount
                        ))
                    }

                    override fun onError() {
                        mIsCollect = false
                    }
                }
        )
    }

    private fun setCollectCount(count: Int) {
        collect_tv.text = ShortVideoUtil.getCount(count)
        collect_tv.contentDescription = "${collect_tv.text}人收藏"
    }

    private fun setCollectRes(collect: Boolean) {
        collect_iv.setImageResource(if (collect) R.drawable.ic_collect_48_y else R.drawable.ic_collect_white48_n)
    }

    private fun setCollectedWithContentDescription(collected: Boolean) {
        mIsCollect = collected

        // 直接调用setContentDescription()会立即触发一次重新朗读
        // 因此先屏蔽该按钮的无障碍焦点，然后再修改朗读信息，最后恢复无障碍焦点，从而避免上述问题
        val origI4A: Int = collect_tv.importantForAccessibility
        collect_tv.importantForAccessibility = View.IMPORTANT_FOR_ACCESSIBILITY_NO
        collect_tv.contentDescription = if (collected) "取消收藏" else "收藏"
        collect_tv.importantForAccessibility = origI4A

        val origI4B: Int = collect_layout.importantForAccessibility
        collect_layout.importantForAccessibility = View.IMPORTANT_FOR_ACCESSIBILITY_NO
        collect_layout.contentDescription = if (collected) "取消收藏" else "收藏"
        collect_layout.importantForAccessibility = origI4B
    }

    private fun setLikeWithContentDescription(like: Boolean) {
        mIsFavorite = like

        // 直接调用setContentDescription()会立即触发一次重新朗读
        // 因此先屏蔽该按钮的无障碍焦点，然后再修改朗读信息，最后恢复无障碍焦点，从而避免上述问题
        val origI4A: Int = favorite_iv.importantForAccessibility
        favorite_iv.importantForAccessibility = View.IMPORTANT_FOR_ACCESSIBILITY_NO
        favorite_iv.contentDescription = if (like) "取消点赞" else "点赞"
        favorite_iv.importantForAccessibility = origI4A

        val origI4B: Int = favorite_layout.importantForAccessibility
        favorite_layout.importantForAccessibility = View.IMPORTANT_FOR_ACCESSIBILITY_NO
        favorite_layout.contentDescription = if (like) "取消点赞" else "点赞"
        favorite_layout.importantForAccessibility = origI4B
    }

    private fun setPlayWithContentDescription(isPlaying: Boolean) {
        // 直接调用setContentDescription()会立即触发一次重新朗读
        // 因此先屏蔽该按钮的无障碍焦点，然后再修改朗读信息，最后恢复无障碍焦点，从而避免上述问题
        val origI4A: Int = play.importantForAccessibility
        play.importantForAccessibility = View.IMPORTANT_FOR_ACCESSIBILITY_NO
        play.contentDescription = if (isPlaying) "暂停" else "播放"
        play.importantForAccessibility = origI4A
    }

    fun setSpeedChangeListener(listener: ISpeedChangeListener?) {
        mSpeedChangeListener = listener
    }

    /**
     * 隐藏选择速度弹出框
     */
    fun showSpeedOptWithAnimation(show: Boolean) {
        mSpeedRatioSelector!!.popup(true, show)
    }

    /**
     * 隐藏弹出框
     */
    fun hideSpeedOptLayout() {
        mSpeedRatioSelector!!.dismiss()
    }

    /**
     * 选择速度弹出框是否显示
     */
    val isSpeedOptShow: Boolean
        get() = mSpeedRatioSelector!!.isShowing

    /**
     * 设置弹出框当前的速度
     */
    fun setCurSpeedType(speedRatioType: SpeedRatioType?) {
        mSpeedRatioSelector!!.setCurSpeedType(speedRatioType)
    }

    /**
     * 设置弹出框当前的速度
     */
    fun setCurSpeedType(radio: Float) {
        setCurSpeedType(SpeedRatioType.ratioOf(radio))
    }

    /**
     * 判断点击是否在弹出框内
     */
    fun isClickSpeedOptLayout(x: Float, y: Float): Boolean {
        return isInViewGroup(mSpeedRatioSelector!!.speedRatioLayout, x, y) || isInViewGroup(speed_radio_layout, x, y)
    }

    /**
     * 判断点击是否在弹出框内
     */
    fun isClickControlViewLayout(x: Float, y: Float): Boolean {
        return isInViewGroup(mSpeedRatioSelector!!.speedRatioLayout, x, y) || isInViewGroup(control_root, x, y)
    }

    /**
     * 判断点击事件是否在viewGroup中
     */
    fun isInViewGroup(viewGroup: ViewGroup?, x: Float, y: Float): Boolean {
        val mSpeedRatioOptLocation = IntArray(2)
        // 获取调速弹出框在屏幕中的位置
        viewGroup!!.getLocationOnScreen(mSpeedRatioOptLocation)
        val speedOptRectF = RectF(mSpeedRatioOptLocation[0].toFloat(), mSpeedRatioOptLocation[1].toFloat(),
                (mSpeedRatioOptLocation[0] + viewGroup.width).toFloat(), (mSpeedRatioOptLocation[1] + viewGroup.height).toFloat())
        return speedOptRectF.contains(x, y)
    }

    val playPos: Long
        get() = mMediaPlayer!!.position

    val playDuration: Long
        get() = mMediaPlayer!!.duration

    fun refreshSeeBarWithGesture(progress: Long) {
        seekBar.isEnabled = true
        seekBar.progress = (progress / 1000).toInt()
        pos_time.text = DateUtil.stringOfTime(progress / 1000)
    }

    fun updateGestureSeekState(isSeeking: Boolean) {
        mIsGestureSeeking = isSeeking
    }

    fun setVideoDetail(data: VideoBean) {
        this.mVideoDetail = data
        initCollect(data.isCollect, data.collectNum)
        initFavorite(data.isLike, data.likeNum)

        // 清除请求状态，重置各参数
        mIsFavExec = false
        mIsCollExec = false
    }

    fun setIsAutoHideCallback(callback: (Boolean) -> Unit) {
        mIsAutoHideCallback = callback
    }

    companion object {
        private const val TAG = "PlayControlView"
    }

}