package com.tencent.edu.module.userinterest.bindingadapter

import android.content.Context
import android.content.res.ColorStateList
import android.graphics.Color
import android.graphics.drawable.GradientDrawable
import android.graphics.drawable.StateListDrawable
import android.util.TypedValue
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.databinding.BindingAdapter
import com.tencent.edu.R
import com.tencent.edu.module.userinterest.data.UserInterestInfo
import com.tencent.edu.module.userinterest.view.EduInterestTagFlowLayout
import com.tencent.edu.module.userinterest.view.EduInterestTagFlowLayout.EduTagAdapter
import com.tencent.edu.module.userinterest.view.EduTagTextView

/**
 * 兴趣选择页BindingAdapter
 * 用于生成各个tag，并监听其点击事件
 *
 * @author aidanyang
 * @date 2020-11-17
 */
object UserInterestGuideBindingAdapter {

    private var mTagColors: IntArray? = null

    @JvmStatic
    @BindingAdapter(value = ["interestInfo", "interestClickListener"], requireAll = true)
    fun setInfo(view: View, info: UserInterestInfo?,
                clickListener: EduInterestTagFlowLayout.OnInterestTagClickListener?) {
        if (info == null || clickListener == null) {
            return
        }

        initTagsColor(view.context)

        generateViews(view as ViewGroup, info, clickListener)
    }

    /**
     * 从资源文件里读颜色配置
     */
    private fun initTagsColor(context: Context) {
        if (mTagColors != null) {
            return
        }

        mTagColors = context.resources.getIntArray(R.array.tagcolors)
    }

    /**
     * 创建View
     */
    private fun generateViews(viewGroup: ViewGroup, info: UserInterestInfo,
                              clickListener: EduInterestTagFlowLayout.OnInterestTagClickListener) {
        for (i in info.mCategoryList.indices) {
            val tagParent: UserInterestInfo.UserCategoryTag = info.mCategoryList[i]
            val tvInterest = TextView(viewGroup.context)
            val lp = ViewGroup.MarginLayoutParams(
                    ViewGroup.LayoutParams.WRAP_CONTENT,
                    ViewGroup.LayoutParams.WRAP_CONTENT)
            tvInterest.layoutParams = lp
            tvInterest.setTextSize(TypedValue.COMPLEX_UNIT_SP, 18f)
            tvInterest.setTextColor(viewGroup.context.resources.getColor(R.color.black))
            val space: Int = viewGroup.context.resources.getDimensionPixelSize(R.dimen.guide_space_margin)
            tvInterest.setPadding(0, space, 0, space)
            val tp = tvInterest.paint
            tp.isFakeBoldText = true //设置加粗
            tvInterest.text = tagParent.mCategoryName
            viewGroup.addView(tvInterest)

            //添加子标签
            val mFlowLayout = EduInterestTagFlowLayout(viewGroup.context)
            val mFlp = ViewGroup.MarginLayoutParams(
                    ViewGroup.LayoutParams.MATCH_PARENT,
                    ViewGroup.LayoutParams.WRAP_CONTENT)
            mFlowLayout.layoutParams = mFlp
            mFlowLayout.setHorizSpace(space.toFloat())
            mFlowLayout.setVerticalSpace(space.toFloat())
            mFlowLayout.setPadding(0, 0, 0, space)
            viewGroup.addView(mFlowLayout)

            //对mTagColors再次判空
            val color = if (mTagColors != null && mTagColors!!.isNotEmpty()) mTagColors!![i % mTagColors!!.size] else Color.parseColor("#23B8FF")
            mFlowLayout.setAdapter(object : EduTagAdapter<UserInterestInfo.UserCategoryTag>(tagParent.mCategoryList) {
                override fun getView(parent: EduInterestTagFlowLayout?, position: Int, userCategoryTag: UserInterestInfo.UserCategoryTag): View {
                    val tv = EduTagTextView(viewGroup.context)
                    tv.text = userCategoryTag.mCategoryName
                    tv.contentDescription = if (tv.isChecked) "已选中：" + tv.text else tv.text
                    val lp = ViewGroup.MarginLayoutParams(
                            ViewGroup.LayoutParams.WRAP_CONTENT,
                            ViewGroup.LayoutParams.WRAP_CONTENT)
                    tv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 14f)
                    tv.setTextColor(createColorStateList(color))
                    tv.background = createStateListDrawable(viewGroup.context, color)
                    tv.layoutParams = lp
                    return tv
                }
            })

            mFlowLayout.setOnInterestTagClickListener(clickListener)
        }
    }

    /**
     * 创建颜色数组
     */
    fun createColorStateList(color: Int): ColorStateList? {
        return ColorStateList(arrayOf(intArrayOf(android.R.attr.state_checked), intArrayOf(-android.R.attr.state_checked)), intArrayOf(color, Color.BLACK))
    }

    /**
     * 通过颜色创建drawable数组
     */
    fun createStateListDrawable(context: Context, color: Int): StateListDrawable? {
        val normalDrawable = ContextCompat.getDrawable(context,
                R.drawable.bg_tv_interest_tag_normal) as GradientDrawable?
        val checkDrawable = ContextCompat.getDrawable(context,
                R.drawable.bg_tv_interest_tag_check) as GradientDrawable?
        normalDrawable!!.setStroke(1, Color.parseColor("#BBBBBB"))
        //checkDrawable.setStroke(1, color);
        val colorBg = Color.argb(25, Color.red(color), Color.green(color), Color.blue(color))
        checkDrawable!!.setColor(colorBg)
        checkDrawable.setStroke(1, color)
        val stateChecked = android.R.attr.state_checked
        val stateList = StateListDrawable()
        stateList.addState(intArrayOf(-stateChecked), normalDrawable)
        stateList.addState(intArrayOf(stateChecked), checkDrawable)
        return stateList
    }

}