package com.tencent.edu.module.vodplayer.widget

import android.content.Context
import android.content.res.TypedArray
import android.graphics.drawable.AnimationDrawable
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.view.animation.AlphaAnimation
import android.widget.ImageView
import android.widget.RelativeLayout
import com.tencent.edu.R
import com.tencent.edu.common.core.ThreadMgr

/**
 * Created by vokenfan on 2018/3/6.
 */
class PlayerScreenLockView : RelativeLayout {
    var isLockState = false
        private set
    private var mLockImage: ImageView? = null
    private var mLockGifDrawable: AnimationDrawable? = null
    private var mUnlockGifDrawble: AnimationDrawable? = null
    private var mListener: IScreenLockListener? = null
    private var isAutoHide = true

    constructor(context: Context) : super(context) {
        initView(context, null)
    }

    constructor(context: Context, attrs: AttributeSet?) : super(context, attrs) {
        initView(context, attrs)
    }

    private enum class DestLockStatus {
        LOCK, UNLOCK
    }

    private enum class CurrentLockStatus {
        LOCKED, UNLOCKED
    }

    private fun updateLockStatusWithGifDrawable(destStatus: DestLockStatus) {
        if (destStatus == DestLockStatus.LOCK) {
            this.contentDescription = "点击锁定播放器"
            mLockImage!!.setImageDrawable(mLockGifDrawable)
            mLockGifDrawable!!.start()
        } else {
            this.contentDescription = "点击解锁播放器"
            mLockImage!!.setImageDrawable(mUnlockGifDrawble)
            mUnlockGifDrawble!!.start()
        }
    }

    private fun updateLockStatusWithStaticImage(currentStatus: CurrentLockStatus) {
        if (currentStatus == CurrentLockStatus.UNLOCKED) {
            this.contentDescription = "点击锁定播放器"
            mLockImage!!.setImageResource(R.drawable.unlock15)
        } else {
            this.contentDescription = "点击解锁播放器"
            mLockImage!!.setImageResource(R.drawable.lock15)
        }
    }

    private fun initView(context: Context, attrs: AttributeSet?) {
        val inflater = LayoutInflater.from(context)
        inflater.inflate(R.layout.view_fs_lockscreen, this)

        val a: TypedArray = context.obtainStyledAttributes(attrs, R.styleable.PlayerScreenLockView)
        isAutoHide = a.getBoolean(R.styleable.PlayerScreenLockView_autoHide, true)
        a.recycle()

        mLockImage = findViewById<View>(R.id.lockimage) as ImageView
        this.contentDescription = "点击锁定播放器"

        setOnClickListener {
            ThreadMgr.getInstance().uiThreadHandler.removeCallbacks(mUnlockRunnable)
            ThreadMgr.getInstance().uiThreadHandler.removeCallbacks(mLockRunnable)
            if (isLockState) {
                if (mUnlockGifDrawble == null) {
                    initUnLockGifDrawable()
                }
                updateLockStatusWithGifDrawable(DestLockStatus.UNLOCK)
//                    mLockImage.setImageDrawable(mUnlockGifDrawble);
//                    mUnlockGifDrawble.start();

                ThreadMgr.getInstance().uiThreadHandler.postDelayed(mUnlockRunnable, 550)
            } else {
                if (mLockGifDrawable == null) {
                    initLockGifDrawable()
                }
                updateLockStatusWithGifDrawable(DestLockStatus.LOCK)
//                    mLockImage.setImageDrawable(mLockGifDrawable);
//                    mLockGifDrawable.start();
                ThreadMgr.getInstance().uiThreadHandler.postDelayed(mLockRunnable, 550)
            }
            isLockState = !isLockState
        }

        if (isAutoHide) {
            ThreadMgr.getInstance().uiThreadHandler.removeCallbacks(mGoneRunnable)
            ThreadMgr.getInstance().uiThreadHandler.postDelayed(mGoneRunnable, 5000)
        }
    }

    private fun initLockGifDrawable() {
        if (mLockGifDrawable == null) {
            // 为AnimationDrawable添加动画帧
            mLockGifDrawable = AnimationDrawable()
            mLockGifDrawable!!.addFrame(resources.getDrawable(R.drawable.lock1), 40)
            mLockGifDrawable!!.addFrame(resources.getDrawable(R.drawable.lock2), 40)
            mLockGifDrawable!!.addFrame(resources.getDrawable(R.drawable.lock3), 40)
            mLockGifDrawable!!.addFrame(resources.getDrawable(R.drawable.lock4), 40)
            mLockGifDrawable!!.addFrame(resources.getDrawable(R.drawable.lock5), 40)
            mLockGifDrawable!!.addFrame(resources.getDrawable(R.drawable.lock6), 40)
            mLockGifDrawable!!.addFrame(resources.getDrawable(R.drawable.lock7), 40)
            mLockGifDrawable!!.addFrame(resources.getDrawable(R.drawable.lock8), 40)
            mLockGifDrawable!!.addFrame(resources.getDrawable(R.drawable.lock9), 40)
            mLockGifDrawable!!.addFrame(resources.getDrawable(R.drawable.lock10), 40)
            mLockGifDrawable!!.addFrame(resources.getDrawable(R.drawable.lock11), 40)
            mLockGifDrawable!!.addFrame(resources.getDrawable(R.drawable.lock12), 40)
            mLockGifDrawable!!.addFrame(resources.getDrawable(R.drawable.lock13), 40)
            mLockGifDrawable!!.addFrame(resources.getDrawable(R.drawable.lock14), 40)
            mLockGifDrawable!!.addFrame(resources.getDrawable(R.drawable.lock15), 40)
            mLockGifDrawable!!.isOneShot = true
        }
    }

    private fun initUnLockGifDrawable() {
        if (mUnlockGifDrawble == null) {
            // 为AnimationDrawable添加动画帧
            mUnlockGifDrawble = AnimationDrawable()
            mUnlockGifDrawble!!.addFrame(resources.getDrawable(R.drawable.unlock1), 40)
            mUnlockGifDrawble!!.addFrame(resources.getDrawable(R.drawable.unlock2), 40)
            mUnlockGifDrawble!!.addFrame(resources.getDrawable(R.drawable.unlock3), 40)
            mUnlockGifDrawble!!.addFrame(resources.getDrawable(R.drawable.unlock4), 40)
            mUnlockGifDrawble!!.addFrame(resources.getDrawable(R.drawable.unlock5), 40)
            mUnlockGifDrawble!!.addFrame(resources.getDrawable(R.drawable.unlock6), 40)
            mUnlockGifDrawble!!.addFrame(resources.getDrawable(R.drawable.unlock7), 40)
            mUnlockGifDrawble!!.addFrame(resources.getDrawable(R.drawable.unlock8), 40)
            mUnlockGifDrawble!!.addFrame(resources.getDrawable(R.drawable.unlock9), 40)
            mUnlockGifDrawble!!.addFrame(resources.getDrawable(R.drawable.unlock10), 40)
            mUnlockGifDrawble!!.addFrame(resources.getDrawable(R.drawable.unlock11), 40)
            mUnlockGifDrawble!!.addFrame(resources.getDrawable(R.drawable.unlock12), 40)
            mUnlockGifDrawble!!.addFrame(resources.getDrawable(R.drawable.unlock13), 40)
            mUnlockGifDrawble!!.addFrame(resources.getDrawable(R.drawable.unlock14), 40)
            mUnlockGifDrawble!!.addFrame(resources.getDrawable(R.drawable.unlock15), 40)
            mUnlockGifDrawble!!.isOneShot = true
        }
    }

    private val mGoneRunnable = Runnable { show(false) }
    private val mLockRunnable = Runnable {
        if (mLockGifDrawable != null) {
            mLockGifDrawable!!.stop()
        }
        //mLockImage.setImageResource(R.drawable.lock15);
        updateLockStatusWithStaticImage(CurrentLockStatus.LOCKED)
        if (mListener != null) {
            mListener!!.onLockScreenShow(true)
        }
    }
    private val mUnlockRunnable = Runnable {
        if (mUnlockGifDrawble != null) {
            mUnlockGifDrawble!!.stop()
        }
        //mLockImage.setImageResource(R.drawable.unlock15);
        updateLockStatusWithStaticImage(CurrentLockStatus.UNLOCKED)
        if (mListener != null) {
            mListener!!.onLockScreenShow(false)
        }
    }

    fun show(show: Boolean) {
        if (show) {
            this.visibility = View.VISIBLE
            if (isLockState) {
                //mLockImage.setImageResource(R.drawable.lock15);
                updateLockStatusWithStaticImage(CurrentLockStatus.LOCKED)
            } else {
                //mLockImage.setImageResource(R.drawable.unlock15);
                updateLockStatusWithStaticImage(CurrentLockStatus.UNLOCKED)
            }

            if (isAutoHide) {
                ThreadMgr.getInstance().uiThreadHandler.removeCallbacks(mGoneRunnable)
                ThreadMgr.getInstance().uiThreadHandler.postDelayed(mGoneRunnable, 5000)
            }
        } else {
            this.visibility = View.GONE
        }
    }

    fun setIScreenLockListener(listener: IScreenLockListener) {
        mListener = listener
    }

    fun showWithAnimation(show: Boolean, animation: AlphaAnimation) {
        // 防止隐藏后再隐藏，控制面板闪一下
        if (visibility == (if (show) View.VISIBLE else View.GONE)) {
            return
        }

        show(show)
        startAnimation(animation)
    }

    interface IScreenLockListener {
        fun onLockScreenShow(lock: Boolean)
    }
}