package com.tencent.edu.module.vodplayer.widget

import android.content.Context
import android.graphics.Color
import android.util.AttributeSet
import android.view.View
import android.widget.LinearLayout
import com.tencent.edu.common.utils.PixelUtil

class StepProgressBar @JvmOverloads constructor(
        context: Context,
        attrs: AttributeSet? = null,
        defStyleAttr: Int = 0
) : LinearLayout(context, attrs, defStyleAttr) {

    private val defaultHeight = PixelUtil.dp2px(5f)

    private var needInitial = true

    var max: Int = 10
        set(value) {
            field = value
            makeStepView()
        }

    var step: Int = 0
        set(value) {
            field = value
            makeStepView()
        }

    var stepDoneColor = Color.WHITE
        set(value) {
            field = value
            makeStepView()
        }

    var stepMargin = PixelUtil.dp2px(2f)


    init {
        orientation = HORIZONTAL
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        val width = getDefaultSize(suggestedMinimumWidth, widthMeasureSpec)
        val height = getDefaultHeight(defaultHeight, heightMeasureSpec)
        super.onMeasure(widthMeasureSpec, heightMeasureSpec)
        if (needInitial) {
            needInitial = false
            makeStepView(width, height)
        }
    }

    private fun getDefaultHeight(size: Int, measureSpec: Int): Int {
        val specMode = MeasureSpec.getMode(measureSpec)
        val specSize = MeasureSpec.getSize(measureSpec)
        return when (specMode) {
            MeasureSpec.EXACTLY -> specSize
            MeasureSpec.UNSPECIFIED, MeasureSpec.AT_MOST -> size
            else -> size
        }
    }

    private fun makeStepView(width: Int = getWidth(), height: Int = getHeight()) {
        if (needInitial) {
            return
        }

        removeAllViewsInLayout()

        val totalViewWidth = width - stepMargin * (max - 1)
        val undoneViewWidth = (totalViewWidth.toDouble() / max).toInt()
        var index = 0
        repeat(step) {
            addDoneView(undoneViewWidth, height, if (index++ == 0) 0 else stepMargin)
        }
        // 剩余没有填充的部分
        addView(View(context).apply {
            val lessWidth = (stepMargin + undoneViewWidth) * (max - step) - if (step == 0) stepMargin else 0
            if (lessWidth == 0){
                return
            }
            layoutParams = LayoutParams(lessWidth, height)
            setBackgroundColor(Color.parseColor("#495559"))
        })
    }

    private fun addDoneView(width: Int, height: Int, lMargin: Int) {
        // 音量间的间隔
        if (lMargin != 0) {
            addView(View(context).apply {
                layoutParams = LayoutParams(lMargin, height)
                setBackgroundColor(Color.parseColor("#495559"))
            })
        }
        // 音量step
        addView(View(context).apply {
            layoutParams = LayoutParams(width, height)
            setBackgroundColor(stepDoneColor)
        })
    }

    fun update(offset: Int, total: Int) {
        step = offset
        max = total
        makeStepView()
    }

}